	SUBROUTINE CHITET(IQPNT,IOMG)
*****************************************************************
*  Calculate weight function for the integration over BZ        *
*  the following susceptibility like expressions:               *
*                                                               *
*  Real part:                                                   *
*                                                               *
*  ___    /  3    f(Ef-E(kj))*(1-f(Ef-E(k+qj'))                 *
*  >      \ d k   -----------------------------  R(k,k+q,jj')   *
*  ---    /             E(kj)-E(k+qj")+Omega      LL't          *
*  jj'     BZ                                                   *
*                                                               *
*  ___    /  3                                                  *
*  >      \ d k   f(Ef-E(kj))*f(Ef-E(k+qj') R(k,k+q,jj')        *
*  ---    /                                  LL't               *
*  jj'     BZ                                                   *
*                                                               *
*                                                               *
*  Imaginary part:                                              *
*                                                               *
*  ___    /  3                                                  *
*  >      \ d k  [f(Ef-E(kj))-f(Ef-E(k+qj')]*R(k,k+q,jj')*      *
*  ---    /          d(E(kj)-E(k+qj')-W)                        *
*  jj'     BZ                                                   *
*                                                               *
*  ___    /  3                                                  *
*  >      \ d k  [f(Ef-E(kj))-f(Ef-E(k+qj')]*R(k,k+q,jj')*      *
*  ---    /          d(E(kj)-E(k+qj')-W)*(v(kj)-v(k+qj')^2      *
*  jj'     BZ                                                   *
*                                                               *
*  Imaginary part when omega->0:                                *
*                                                               *
*  ___    /  3                                                  *
*  >      \ d k   d(Ef-E(kj))*d(Ef-E(k+qj')*R(k,k+q,jj')        *
*  ---    /                                                     *
*  jj'     BZ                                                   *
*                                                               *
*                                                               *
*  ___    /  3                                                  *
*  >      \ d k d(Ef-E(kj))*d(Ef-E(k+qj')*R(k,k+q,jj')          *
*  ---    /                                 *(v(kj)-v(k+qj')^2  *
*  jj'     BZ                                                   *
*                                                               *
*  where f(x) are occupation numbers and R are some matrix      *
*  elements. Tetrahedron method is used.                        *
*****************************************************************
C  SET ARRAY-SIZE DATA
      USE DIM_LMTART
C  SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                           !!! from setup
      USE COM_CONTROL ,ONLY: LIFT,LMTO,IASA,NPFR,NOVR          !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS           !!! from readini
      USE COM_LATTICE ,ONLY: RBAS,GBAS,AKBAS,RKBAS             !!! from readstr
      USE COM_LATPAR  ,ONLY: PAR,ORTH,OMEGAC                   !!! from readini
      USE COM_NBANDS  ,ONLY: NBNDF,NBNDE,NBNDT,NBNDC,POLUS     !!! from readini
      USE COM_KMESH   ,ONLY: NPNT,PNT,IPQ,IGQ,KCUT,WGT         !!! from tetra
      USE COM_WMESH   ,ONLY: NPNW,PNW,IPW,IGW,KCUW             !!! from tetra
      USE COM_INPTTR  ,ONLY: NDIV,NDIC,NDIW                    !!! from readini
      USE COM_QMESH   ,ONLY: QPNT                              !!! from readpnt
      USE COM_FREQU   ,ONLY: OMGCHI                            !!! from readpnt
	USE COM_FERMI   ,ONLY: EF                                !!! from bzint
      USE COM_GROUPS  ,ONLY: NGROUP,INV,IKOV                   !!! from groups
      USE COM_SUBGROUP,ONLY: NSGQ,NGEQ,ISG,KSYMPN,MSYMPN,IGS   !!! from delmsh
      USE COM_SYMMESH ,ONLY: IG0IG1,IK0IK1                     !!! from delmsh
      USE COM_SYMMESW ,ONLY: IG0IGW=>IG0IG1,IK0IKW=>IK0IK1     !!! from delmsh
	USE COM_ENERGY  ,ONLY: EARR,EARW,WEIGH,WEIGD             !!! from bands
	USE COM_CHANNELS,ONLY: IUTTR,IUTTW                       !!! from channels
      USE COM_MASHDEP ,ONLY: LENCMP,NVAL                       !!! from readini
      USE COM_LMTOUT  ,ONLY: SCRFILE                           !!! from readini
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                       !!! from setup
C  SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER KTQ(4),KT(4),KTT(4),KGG(4),KTW(4)
	REAL*8, ALLOCATABLE :: WEIGK(:,:,:)
	REAL*8, ALLOCATABLE :: WEIGQ(:,:,:,:)
	REAL*8, ALLOCATABLE :: WGTOM(:,:,:,:)
      DIMENSION VEL(3),SPDOS(2),SPTOS(2),SPVEL(3,2)
     &   ,REC(3,3),PKT(3,4),PKW(3,4),CRW(3,4),QWGT(4,3)
     &   ,E(4),H(4),WGTQ(4),AT(4),AD(4),ADS(4,2),ATS(4,2)
     &   ,VEL0(3),PQT(3,4),VK(3),VQ(3),VELM(3,3),SPVELM(3,3,2)
	CHARACTER*130 FILE
	LOGICAL EXIST,LNEG,LPOS
	DATA PI/3.141592653589793D0/,NODEBUG/0/,EPS/1.D-10/
C  MEMORY ALLOCATIONS
	ALLOCATE(WEIGK(NBNDC(1)*NSPINMEM,4,4))
	ALLOCATE(WEIGQ(NBNDC(1)*NSPINMEM,NBNDC(1)*NSPINMEM,6,4))
	ALLOCATE(WGTOM(NBNDC(1)*NSPINMEM,NBNDC(1)*NSPINMEM,4,4))
C  SET Q-VECTOR
      IF(IQPNT.NE.0)THEN
      QPNTX=QPNT(1,IQPNT)
      QPNTY=QPNT(2,IQPNT)
      QPNTZ=QPNT(3,IQPNT)
      ELSE
      QPNTX=0.D0
      QPNTY=0.D0
      QPNTZ=0.D0
      ENDIF
C  GET INFORMATION ABOUT THE FILE
	FILE=SCRFILE(IUTTW)
      NRECL=NVAL/2
      NRECL=NVAL/2*NBNDC(1)**2*NSPIN**2*6+NRECL
	NRECL=NVAL/2*NBNDC(1)*NSPIN*4+NRECL
      NRECL=NVAL/2*NBNDC(1)**2*NSPIN**2*4+NRECL
	NRECL=MAX(NRECL,NVAL/2*13)
      OPEN(UNIT=IUTTW,FILE=FILE,STATUS='NEW',
     &FORM='UNFORMATTED',ACCESS='DIRECT',RECL=NRECL)
	WRITE(IUTTW,REC=1)NRECL,NDIV(1),NDIV(2),NDIV(3),NPNT(1),
     &NBNDF(1),NBNDE(1),NBNDT(1),NSPIN,NORBS,LWHI,POLUS,
     &OMGCHI(IOMG)
C  OPEN SCRATCH FILE FOR FAST ACCESS
      OPEN(UNIT=IUTTR,FILE=SCRFILE(IUTTR),STATUS='NEW',
     &FORM='UNFORMATTED',ACCESS='DIRECT',RECL=NRECL)
	WRITE(IUTTR,REC=1)NRECL,NDIV(1),NDIV(2),NDIV(3),NPNT(1),
     &NBNDF(1),NBNDE(1),NBNDT(1),NSPIN,NORBS,LWHI,POLUS,
     &OMGCHI(IOMG)
C  INITIALLY SET WEIGHTS FOR ALL POINTS TO ZERO
      DOS0=0.D0
      VEL0(1)=0.D0
      VEL0(2)=0.D0
      VEL0(3)=0.D0
	DO ISGQ=1,NSGQ
	DO IPNT=1,NPNT(1)
	WGTQ(1)=0.D0
	DO IBNDS1=1,NSPIN
	DO IBNDC1=1,NBNDC(1)
	IBANC1=IBNDS1+NSPIN*(IBNDC1-1)
	DO IC=1,4
	WEIGK(IBANC1,IC,1)=0.D0
	ENDDO
	DO IBNDS=1,NSPIN
	DO IBNDC=1,NBNDC(1)
	IBANC=IBNDS+NSPIN*(IBNDC-1)
C  WEIGHTS FOR THE REAL PART
	DO IC=1,6
	WEIGQ(IBANC,IBANC1,IC,1)=0.D0
	ENDDO
C  WEIGHTS FOR THE IMAGINARY PART
	DO IC=1,4
	WGTOM(IBANC,IBANC1,IC,1)=0.D0
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	IREC=1+IPNT+NPNT(1)*(ISGQ-1)
	WRITE(IUTTR,REC=IREC)WGTQ(1),(((WEIGQ(IBANC,IBANC1,IC,1),
     &IBANC=1,NBNDC(1)*NSPIN),IBANC1=1,NBNDC(1)*NSPIN),
     &IC=1,6),((WEIGK(IBANC,IC,1),IBANC=1,NBNDC(1)*NSPIN),
     &IC=1,4),(((WGTOM(IBANC,IBANC1,IC,1),IBANC=1,NBNDC(1)*
     &NSPIN),IBANC1=1,NBNDC(1)*NSPIN),IC=1,4)
	ENDDO
	ENDDO
C------------------------------------------------------------
C  DO OVER WHOLE BRILLOUIN ZONE REPRESENTED BY MICROCELLS AND 
C  INTEGRATE OVER THE TETRAHEDRA
C------------------------------------------------------------
	VOLIBZ=1.D0
      OT=1.D0/DBLE(NDIV(1)*NDIV(2)*NDIV(3))/6.D0
      OW=1.D0/DBLE(NDIW(1)*NDIW(2)*NDIW(3))/6.D0
	DO 4200 I3=0,NDIV(3)-1               !!! over cells along G3
	DO 4200 I2=0,NDIV(2)-1               !!! over cells along G2
	DO 4200 I1=0,NDIV(1)-1               !!! over cells along G1
C  THE MICROCELL IS SET UP BY I1,I2,I3
C  SET UP TETRAHEDRA INSIDE MICROCELL
	DO 4100 ITTR=1,6                    !!! over tetrahedra
	DO I=1,4
C  COORDINATES OF THE TETRAHEDRON AGE GIVEN BY KTT, SHIFTED - BY KTT1
	J1=I1+KCUT(1,I,ITTR)
	J2=I2+KCUT(2,I,ITTR)
	J3=I3+KCUT(3,I,ITTR)
	KTT(I)=IPQ(J1,J2,J3)
	KGG(I)=IGS(J1,J2,J3)
	ENDDO
C  TWO TETRAHEDRA FOUND, DEFINE CASE
	DO I=1,4
	KT(I)=0
	ENDDO
	DO I=1,4
	IF(KT(I).EQ.0)THEN
	KT(I)=I
	DO J=I+1,4
      IF(KTT(I).EQ.KTT(J).AND.KGG(I).EQ.KGG(J).AND.KT(J).EQ.0)KT(J)=I  
	ENDDO
	ENDIF
	ENDDO
C  READ WEIGHTS FOR THE POINTS
	DO I=4,1,-1 ! looping backwards to avoid bug in shit IBM compiler
	IPNT=KTT(I)
	ISGQ=KGG(I)
	IREC=1+IPNT+NPNT(1)*(ISGQ-1)
	READ(IUTTR,REC=IREC)WGTQ(KT(I)),(((WEIGQ(IBANC,IBANC1,IC,KT(I)),
     &IBANC=1,NBNDC(1)*NSPIN),IBANC1=1,NBNDC(1)*NSPIN),
     &IC=1,6),((WEIGK(IBANC,IC,KT(I)),IBANC=1,NBNDC(1)*NSPIN),
     &IC=1,4),(((WGTOM(IBANC,IBANC1,IC,KT(I)),
     &IBANC=1,NBNDC(1)*NSPIN),IBANC1=1,NBNDC(1)*NSPIN),
     &IC=1,4)
	ENDDO
C  SET GEOMETRICAL WEIGHT
	DO I=1,4
	WGTQ(KT(I))=WGTQ(KT(I))+OT/4.D0       !!! geometrical weight f(kj)=1
	ENDDO
C  SET UP POINTS "K" DESCRIBING TETRAHEDRON IN THE SPACE
      DO I=1,4
      J1=I1+KCUT(1,I,ITTR)
      J2=I2+KCUT(2,I,ITTR)
      J3=I3+KCUT(3,I,ITTR)
      PKT(1,I)=(J1*AKBAS(1,1,1)+
     &          J2*AKBAS(1,2,1)+
     &          J3*AKBAS(1,3,1))*2.D0*PI/PAR
      PKT(2,I)=(J1*AKBAS(2,1,1)+
     &          J2*AKBAS(2,2,1)+
     &          J3*AKBAS(2,3,1))*2.D0*PI/PAR
      PKT(3,I)=(J1*AKBAS(3,1,1)+
     &          J2*AKBAS(3,2,1)+
     &          J3*AKBAS(3,3,1))*2.D0*PI/PAR
	ENDDO
C  SHIFT COORDINATE SYSTEM TO THE FIRST POINT
      DO I=2,4
      PKT(1,I)=PKT(1,I)-PKT(1,1)
      PKT(2,I)=PKT(2,I)-PKT(2,1)
      PKT(3,I)=PKT(3,I)-PKT(3,1)
	ENDDO
C  SET RECIPROCALS "REC" TO "PKT"
      CALL CROSS(REC(1,1),PKT(1,3),PKT(1,4))
      CALL CROSS(REC(1,2),PKT(1,4),PKT(1,2))
      CALL CROSS(REC(1,3),PKT(1,2),PKT(1,3))
      W=REC(1,1)*PKT(1,2)+REC(2,1)*PKT(2,2)+REC(3,1)*PKT(3,2)
      IF(ABS(W).LE.1.D-10)STOP'<DELTET>:: INTERNAL ERROR: W=0'
      DO I=1,3
      REC(I,1)=REC(I,1)/W
      REC(I,2)=REC(I,2)/W
      REC(I,3)=REC(I,3)/W
	ENDDO
C--------------------------------------------
C  DO OVER DIVISIONS OF THE MICROCELL
C--------------------------------------------
      DO 3200 ID3=0,NDIW(3)/NDIV(3)-1
      DO 3200 ID2=0,NDIW(2)/NDIV(2)-1
      DO 3200 ID1=0,NDIW(1)/NDIV(1)-1
C  THE SMALL MICROCELL IS SET UP BY IW1,IW2,IW3
      IW1=I1*NDIW(1)/NDIV(1)+ID1
      IW2=I2*NDIW(2)/NDIV(2)+ID2
      IW3=I3*NDIW(3)/NDIV(3)+ID3
C  SET UP SMALL TETRAHEDRON INSIDE SMALL MICROCELL
      DO 3100 ITTRD=1,6
      DO I=1,4
      JW1=IW1+KCUT(1,I,ITTRD)
      JW2=IW2+KCUT(2,I,ITTRD)
      JW3=IW3+KCUT(3,I,ITTRD)
      KTW(I)=IPW(JW1,JW2,JW3)
	KTQ(I)=IK0IKW(KTW(I),IGW(JW1,JW2,JW3))
	ENDDO
C  SET UP POINTS K DESCRIBING SMALL TETRAHEDRON IN THE SPACE
      DO I=1,4
      JW1=IW1+KCUT(1,I,ITTRD)
      JW2=IW2+KCUT(2,I,ITTRD)
      JW3=IW3+KCUT(3,I,ITTRD)
      PKW(1,I)=(JW1*AKBAS(1,1,3)+
     &          JW2*AKBAS(1,2,3)+
     &          JW3*AKBAS(1,3,3))*2.D0*PI/PAR
      PKW(2,I)=(JW1*AKBAS(2,1,3)+
     &          JW2*AKBAS(2,2,3)+
     &          JW3*AKBAS(2,3,3))*2.D0*PI/PAR
      PKW(3,I)=(JW1*AKBAS(3,1,3)+
     &          JW2*AKBAS(3,2,3)+
     &          JW3*AKBAS(3,3,3))*2.D0*PI/PAR
	ENDDO
C  SHIFT COORDINATE SYSTEM TO THE FIRST POINT
      DO I=1,4
      PKW(1,I)=PKW(1,I)-PKT(1,1)
      PKW(2,I)=PKW(2,I)-PKT(2,1)
      PKW(3,I)=PKW(3,I)-PKT(3,1)
      PQT(1,I)=PKW(1,I)+QPNTX*2.D0*PI/PAR
      PQT(2,I)=PKW(2,I)+QPNTY*2.D0*PI/PAR
      PQT(3,I)=PKW(3,I)+QPNTZ*2.D0*PI/PAR
	ENDDO
C  CHECK WHETHER A SMALL TETRAHEDRON "PKW" IS INSIDE  TETRAHEDRON "PKT"
      DO I=1,4
      X1=PKW(1,I)*REC(1,1)+PKW(2,I)*REC(2,1)+PKW(3,I)*REC(3,1)
      X2=PKW(1,I)*REC(1,2)+PKW(2,I)*REC(2,2)+PKW(3,I)*REC(3,2)
      X3=PKW(1,I)*REC(1,3)+PKW(2,I)*REC(2,3)+PKW(3,I)*REC(3,3)
      IF(.NOT.((X1.GE.-EPS.AND.X1.LE.1.D0+EPS).AND.
     &         (X2.GE.-EPS.AND.X2.LE.1.D0+EPS).AND.
     &         (X3.GE.-EPS.AND.X3.LE.1.D0+EPS).AND.
     &         (X1+X2+X3.LE.1.D0+EPS)))GOTO3100
      CRW(1,I)=X1
      CRW(2,I)=X2
      CRW(3,I)=X3
	ENDDO
C---------------------------------------------
C  CALCULATE CONTRIBUTIONS FOR THESE POINTS
C---------------------------------------------
	IPAN=1
C  START LOOPING OVER BANDS
	DO 2200 IBNDS=1,NSPIN                 !!! over spin subbands
	DO 2100 IBNDT=NBNDE(1),NBNDF(1)       !!! over all bands
	IBNDC=IBNDT-NBNDE(1)+1
	IBANC=IBNDS+NSPIN*(IBNDC-1)
	IBAND=IBNDS+NSPIN*(IBNDT-1)
	DO I=1,4
	E(I)=EARW(IBAND,KTW(I),1)             !!! E(k,js)
	H(I)=EARW(IBAND,KTQ(I),1)             !!! E(g*k+q,js)
	ENDDO
C-----------------------------------------------------
C  SET WEIGHTS FOR THE NUMBER OF STATES
C-----------------------------------------------------
      CALL GETVEL(E,PKW,VK)                 !!! dE(kjs)/dk
      CALL MCTTRINT0(E,EF,OW,AT,AD,TS,DS,'B','W')
      DOS0=DOS0+DS
      DO IVEL=1,3
      VEL0(IVEL)=VEL0(IVEL)+DS*VK(IVEL)**2  !!! velocities
      ENDDO
      DO IT=1,4
      DO I=1,4
      IF(IT.EQ.1)WF=1.D0-CRW(1,I)-CRW(2,I)-CRW(3,I)
      IF(IT.NE.1)WF=CRW(IT-1,I)
      WEIGK(IBANC,1,KT(IT))=WEIGK(IBANC,1,KT(IT))+AT(I)*WF !!! weight f(kjs)
      WEIGK(IBANC,3,KT(IT))=WEIGK(IBANC,3,KT(IT))+AD(I)*WF !!! weight d(kjs)
	ENDDO
	ENDDO
      CALL MCTTRINT0(H,EF,OW,AT,AD,TS,DS,'B','W')
      DO IT=1,4
      DO I=1,4
      IF(IT.EQ.1)WF=1.D0-CRW(1,I)-CRW(2,I)-CRW(3,I)
      IF(IT.NE.1)WF=CRW(IT-1,I)
      WEIGK(IBANC,2,KT(IT))=WEIGK(IBANC,2,KT(IT))+AT(I)*WF !!! weight f(k+qjs)
      WEIGK(IBANC,4,KT(IT))=WEIGK(IBANC,4,KT(IT))+AD(I)*WF !!! weight d(k+qjs)
	ENDDO
	ENDDO
C  START LOOPING OVER BANDS
	DO 1200 IBNDS1=1,NSPIN                !!! over spin subbands
	DO 1100 IBNDT1=NBNDE(1),NBNDF(1)      !!! over bands
	IBNDC1=IBNDT1-NBNDE(1)+1
	IBANC1=IBNDS1+NSPIN*(IBNDC1-1)
	IBAND1=IBNDS1+NSPIN*(IBNDT1-1)
	DIF=0.D0
	DO I=1,4
	H(I)=EARW(IBAND1,KTQ(I),1)            !!! E(g*k+q,j's')
	DIF=DIF+ABS(E(I)-H(I)+OMGCHI(IOMG))
	ENDDO
C-----------------------------------------------------
C  SET WEIGHTS FOR THE REAL PART OF SUSCEPTIBILITY
C-----------------------------------------------------
	IF(ABS(DIF).GT.1.D-05)THEN
C  ENERGIES ARE DIFFERENT, SET UP WEIGHTS FOR 1/[E(k,js)-E(g*k+q,j's')+Omega]
	CALL MCHEAD(E,H,EF,OMGCHI(IOMG),OW,QWGT)
      DO IT=1,4
      DO I=1,4
	WE=(E(I)-H(I)+OMGCHI(IOMG))**2/
     &  ((E(I)-H(I)+OMGCHI(IOMG))**2+POLUS**2)
      IF(IT.EQ.1)WF=1.D0-CRW(1,I)-CRW(2,I)-CRW(3,I)
      IF(IT.NE.1)WF=CRW(IT-1,I)
	WEIGQ(IBANC,IBANC1,1,KT(IT))=WEIGQ(IBANC,IBANC1,1,KT(IT))+
     &  QWGT(I,1)*WF                          !!! f(kj)*[1-f(k+qj')]
	WEIGQ(IBANC,IBANC1,2,KT(IT))=WEIGQ(IBANC,IBANC1,2,KT(IT))+
     &  QWGT(I,2)*WF                          !!! [1-f(kj)]*f(k+qj')
	WEIGQ(IBANC,IBANC1,5,KT(IT))=WEIGQ(IBANC,IBANC1,5,KT(IT))+
     &  QWGT(I,1)*WF*WE                       !!! f(kj)*[1-f(k+qj')]
	WEIGQ(IBANC,IBANC1,6,KT(IT))=WEIGQ(IBANC,IBANC1,6,KT(IT))+
     &  QWGT(I,2)*WF*WE                       !!! [1-f(kj)]*f(k+qj')
	ENDDO
	ENDDO
	ELSE
C  ENERGIES ARE THE SAME, WEIGHTS ARE JUST DENSITY OF STATES
	IF(ABS(OMGCHI(IOMG)).GT.1.D-10)
     &WRITE(IUT,*)'<CHITET>: CASE OMEGA.NE.0'
      CALL MCTTRINT0(E,EF,OW,ATS(1,1),ADS(1,1),TS,DS,'B','W')
      CALL MCTTRINT0(H,EF,OW,ATS(1,2),ADS(1,2),TS,DS,'B','W')
      DO IT=1,4
      DO I=1,4
      IF(IT.EQ.1)WF=1.D0-CRW(1,I)-CRW(2,I)-CRW(3,I)
      IF(IT.NE.1)WF=CRW(IT-1,I)
	WEIGQ(IBANC,IBANC1,3,KT(IT))=WEIGQ(IBANC,IBANC1,3,KT(IT))-
     &0.5D0*(ADS(I,1)+ADS(I,2))/2.D0*WF    !!! f(kj)*[1-f(k+qj')]=>-N(Ef)/2
	WEIGQ(IBANC,IBANC1,4,KT(IT))=WEIGQ(IBANC,IBANC1,4,KT(IT))+
     &0.5D0*(ADS(I,1)+ADS(I,2))/2.D0*WF    !!! [1-f(kj)]*f(k+qj')=>+N(Ef)/2
	ENDDO
	ENDDO
	ENDIF
C------------------------------------------------------
C  SET WEIGHTS FOR THE IMAGINARY PART OF SUSCEPTIBILITY
C------------------------------------------------------
      CALL GETVEL(H,PQT,VQ)                 !!! dE(gk+qj's')/dk
      VFAC=(VK(1)*VK(1)+VK(2)*VK(2)+VK(3)*VK(3))+
     &     (VQ(1)*VQ(1)+VQ(2)*VQ(2)+VQ(3)*VQ(3))-
     &   2*(VK(1)*VQ(1)+VK(2)*VQ(2)+VK(3)*VQ(3))
C  CHECK WHETHER THE ENERGIES AT THE CORNERS COINCIDE
      DIF=0.D0
      DO I=1,4
      DIF=DIF+ABS(E(I)-H(I)+OMGCHI(IOMG))
      ENDDO
C  CHECK WHETHER TRANSITION IS ALLOWED
      LNEG=.FALSE.
      LPOS=.FALSE.
      DO I=1,4
      IF(E(I)-H(I)+OMGCHI(IOMG).LE.0.D0)LNEG=.TRUE.
      IF(E(I)-H(I)+OMGCHI(IOMG).GE.0.D0)LPOS=.TRUE.
      ENDDO
      IF(ABS(DIF).GT.1.D-05.AND.LNEG.AND.LPOS.AND.
     &ABS(OMGCHI(IOMG)).GE.1.D-10)THEN
C  ENERGIES ARE DIFFERENT, SET UP WEIGHTS
      CALL MOHEAD(E,H,EF,OMGCHI(IOMG),OW,QWGT)
      DO IT=1,4
      DO I=1,4
      IF(IT.EQ.1)WF=1.D0-CRW(1,I)-CRW(2,I)-CRW(3,I)
      IF(IT.NE.1)WF=CRW(IT-1,I)
      WGTOM(IBANC,IBANC1,1,KT(IT))=WGTOM(IBANC,IBANC1,1,KT(IT))+
     &QWGT(I,1)*WF                          !!! f(kj)*[1-f(k+qj')]
      WGTOM(IBANC,IBANC1,2,KT(IT))=WGTOM(IBANC,IBANC1,2,KT(IT))+
     &QWGT(I,1)*WF*VFAC                     !!! f(kj)*[1-f(k+qj')]
      ENDDO
      ENDDO
      ENDIF
C------------------------------------------------------
C  SET WEIGHTS FOR THE IMAGINARY PART WHEN OMEGA-->ZERO
C------------------------------------------------------
C  CHECK WHETHER TRANSITION IS ALLOWED
      LNEG=.FALSE.
      LPOS=.FALSE.
      DO I=1,4
      IF(E(I)-H(I).LE.0.D0)LNEG=.TRUE.
      IF(E(I)-H(I).GE.0.D0)LPOS=.TRUE.
      ENDDO
      IF(LNEG.AND.LPOS)THEN        !!! transition is allowed
      CALL MDHEAD(E,H,EF,OW,QWGT)
      DO IT=1,4
      DO I=1,4
      IF(IT.EQ.1)WF=1.D0-CRW(1,I)-CRW(2,I)-CRW(3,I)
      IF(IT.NE.1)WF=CRW(IT-1,I)
      WGTOM(IBANC,IBANC1,3,KT(IT))=WGTOM(IBANC,IBANC1,3,KT(IT))+
     &QWGT(I,1)*WF
      WGTOM(IBANC,IBANC1,4,KT(IT))=WGTOM(IBANC,IBANC1,4,KT(IT))+
     &QWGT(I,1)*WF*VFAC
      ENDDO
      ENDDO
	ENDIF
1100  CONTINUE                             !!! over bands
1200  CONTINUE                             !!! over spin subbands
2100  CONTINUE                             !!! over bands
2200  CONTINUE                             !!! over spin subbands
C  WRITE CONTRIBUTIONS BACK TO THE FILE
	DO I=1,4
	IPNT=KTT(I)
	ISGQ=KGG(I)
	IREC=1+IPNT+NPNT(1)*(ISGQ-1)
	WRITE(IUTTR,REC=IREC)WGTQ(KT(I)),(((WEIGQ(IBANC,IBANC1,IC,KT(I)),
     &IBANC=1,NBNDC(1)*NSPIN),IBANC1=1,NBNDC(1)*NSPIN),
     &IC=1,6),((WEIGK(IBANC,IC,KT(I)),IBANC=1,NBNDC(1)*NSPIN),
     &IC=1,4),(((WGTOM(IBANC,IBANC1,IC,KT(I)),
     &IBANC=1,NBNDC(1)*NSPIN),IBANC1=1,NBNDC(1)*NSPIN),
     &IC=1,4)
	ENDDO
3100  CONTINUE                                  !!! over small tetrahedra
3200  CONTINUE                                  !!! over small microcells
4100  CONTINUE                                  !!! over tetrahedra
4200  CONTINUE                                  !!! over microcells
C  NORMALIZATION
	DO ISGQ=1,NSGQ                            !!! over star of the q-vector
	DO IPNT=1,NPNT(1)                         !!! over irreducible points
	IREC=1+IPNT+NPNT(1)*(ISGQ-1)
	READ(IUTTR,REC=IREC)WGTQ(1),(((WEIGQ(IBANC,IBANC1,IC,1),
     &IBANC=1,NBNDC(1)*NSPIN),IBANC1=1,NBNDC(1)*NSPIN),
     &IC=1,6),((WEIGK(IBANC,IC,1),IBANC=1,NBNDC(1)*NSPIN),
     &IC=1,4),(((WGTOM(IBANC,IBANC1,IC,1),
     &IBANC=1,NBNDC(1)*NSPIN),IBANC1=1,NBNDC(1)*NSPIN),
     &IC=1,4)
	WGTQ(1)=WGTQ(1)/VOLIBZ*2.D0/DBLE(NSPIN)
	DO IBNDS1=1,NSPIN
	DO IBNDC1=1,NBNDC(1)
	IBANC1=IBNDS1+NSPIN*(IBNDC1-1)
	DO IC=1,4
	WEIGK(IBANC1,IC,1)=WEIGK(IBANC1,IC,1)/VOLIBZ*2.D0/DBLE(NSPIN)
	ENDDO
	DO IBNDS=1,NSPIN
	DO IBNDC=1,NBNDC(1)
	IBANC=IBNDS+NSPIN*(IBNDC-1)
C  FOR THE REAL PART
	DO IC=1,6
	WEIGQ(IBANC,IBANC1,IC,1)=WEIGQ(IBANC,IBANC1,IC,1)/
     &VOLIBZ*2.D0/DBLE(NSPIN)
	ENDDO
C  FOR THE IMAGINARY PART
	DO IC=1,4
	WGTOM(IBANC,IBANC1,IC,1)=WGTOM(IBANC,IBANC1,IC,1)/
     &VOLIBZ*2.D0/DBLE(NSPIN)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	IREC=1+IPNT+NPNT(1)*(ISGQ-1)
	WRITE(IUTTW,REC=IREC)WGTQ(1),(((WEIGQ(IBANC,IBANC1,IC,1),
     &IBANC=1,NBNDC(1)*NSPIN),IBANC1=1,NBNDC(1)*NSPIN),IC=1,6),
     &((WEIGK(IBANC,IC,1),IBANC=1,NBNDC(1)*NSPIN),IC=1,4),
     &(((WGTOM(IBANC,IBANC1,IC,1),IBANC=1,NBNDC(1)*NSPIN),
     &IBANC1=1,NBNDC(1)*NSPIN),IC=1,4)
      ENDDO                                     !!! over irreducible points
      ENDDO                                     !!! over star of the q-vector
C  CHECK DOS
	CALL TTRVEL(EF,DOS,VEL,SPTOS,SPDOS,SPVEL,VELM,SPVELM,0)
      DOS0=DOS0/VOLIBZ*2.D0/DBLE(NSPIN)     !!! total dos
      VEL0(1)=VEL0(1)/VOLIBZ*2.D0/DBLE(NSPIN)!! velocities
      VEL0(2)=VEL0(2)/VOLIBZ*2.D0/DBLE(NSPIN)!! velocities
      VEL0(3)=VEL0(3)/VOLIBZ*2.D0/DBLE(NSPIN)!! velocities
	IF(ABS(DOS-DOS0).GE.1.D-10)WRITE(IUT,*)'<CHITET> WRONG DOS'
	IF((VEL(1)-VEL0(1))**2+(VEL(2)-VEL0(2))**2+
     &   (VEL(3)-VEL0(3))**2.GE.1.D-10)WRITE(IUT,*)'<CHITET> WRONG VEL'
      WRITE(IUN,1010)
      WRITE(IUN,1020)(VEL0(I),I=1,3)
      WRITE(IUN,1030)
      WRITE(IUN,1040)(13.6058D0*SQRT(8.D0*PI*VEL0(I)/OMEGAC),I=1,3)
1010  FORMAT(1X,'Calculated average square of electron velocities:')
1020  FORMAT(1X,'<Vx^2>=',G14.7,' ; <Vy^2>=',G14.7,' ; <Vz^2>=',G14.7)
1030  FORMAT(1X,'Calculated bare plasma frequencies (in units eV):')
1040  FORMAT(1X,'om_p^x=',G14.7,' ; om_p^y=',G14.7,' ; om_p^z=',G14.7)
	CLOSE(IUTTR,STATUS='DELETE')
	CLOSE(IUTTW,STATUS='KEEP')
	DEALLOCATE(WEIGK,WEIGQ,WGTOM)
	RETURN
	END
