	SUBROUTINE READHUB
*******************************************************
*  Reads HUBFILE which contains Slater integrals      *
*  necessary for constructing U-matrix U(m' k'; m k)  *
*  in the LDA+U method.                               *
*  Also reads occupation numbers for correlated       *
*  states if avaiable.                                *
*  To make constrained LDA, specify corrections to    *
*  the LDA potential, which will be read below.       *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_LMTINP  ,ONLY : ICHUB,HUBFILE                   !!! from readini
      USE COM_CHANNELS,ONLY : IUINP                           !!! from channels
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                     !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
      CHARACTER*150, ALLOCATABLE :: HUBDATA(:)
      CHARACTER STRING*150
      LOGICAL EXIST,MODIFIED
      DATA PI/3.141592653589793D0/
	IDMF=0
	IF(ICHUB.LE.0)RETURN
C CHECK EXISTENCE
      INQUIRE(FILE=HUBFILE,EXIST=EXIST)
	IF(.NOT.EXIST)CALL READHUB_MESG01
C ********************************
C UNDERSTAND THE LENGHT OF HUBFILE
      CALL FILE_LENGHT(HUBFILE,NLEN)
      ALLOCATE(HUBDATA(NLEN))
C ********************************
C READ HUBFILE TO INTERNAL FILE
      OPEN(UNIT=IUINP,FILE=HUBFILE,STATUS='OLD',FORM='FORMATTED')
      DO ILEN=1,NLEN
      READ(IUINP,FMT='(A150)')HUBDATA(ILEN)
      ENDDO
      CLOSE(IUINP)
C ********************************
C UNDERSTAND INPUT: CLASSIC or MODERN
      STRING=HUBDATA(1)
      CALL CONVLOW(STRING,150)
      MODERN=0
      DO IPOS=1,80
      IF(STRING(IPOS:IPOS+12).EQ.'input=classic')MODERN=1
      IF(STRING(IPOS:IPOS+11).EQ.'input=modern' )MODERN=2
      IF(MODERN.NE.0)EXIT
      ENDDO
      IF(MODERN.EQ.0)ISTART=1
      IF(MODERN.EQ.1)ISTART=2
      IF(MODERN.EQ.0)MODERN=1
      IF(MODERN.EQ.1)MODIFIED=.FALSE.
      IF(MODERN.EQ.2)MODIFIED=.TRUE.
C NO FURTHER SUPPORT FOR "CLASSIC"
      IF(MODERN.EQ.1)
     &CALL READHUB_MESG02('Classic input not supported in HUBFILE')
C READ CONTROL DATA
	CALL READHUB_CTRL(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ DMFT SETTINGS
	CALL READHUB_DMFT(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ CORRELATED ORBITALS DATA
	CALL READHUB_CORR(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ OCCUPATION NUMBERS, POTENTIAL
	CALL READHUB_OCCP(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ IMPURITY LEVELS
	IF(IDMF.EQ.2)
     &CALL READHUB_IMPL(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ SELF-ENERGY, HYBRIDIZATION
	IF(IDMF.EQ.2)
     &CALL READHUB_SIGW(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ ZCWA CORRELATIONAL FUNCTIONS
	IF(IDMF.EQ.2)
     &CALL READHUB_ZCWA(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C READ SELF-ENERGY SETTINGS
	IF(IDMF.EQ.3)
     &CALL READHUB_SGMP(HUBDATA,NLEN,NROR,NRSP,ICUBI)
C ********************************************************
C HUBFILE HAS BEEN READ, REFORMAT IT
      IF(MODIFIED)CALL READINI_OVWR(HUBFILE,HUBDATA,NLEN)
C WRITE OCCUPANCIES WITH THE NEW COORDINATES
	CALL WRITEHUB(1)
C MEMORY DEALLOCATIONS
	DEALLOCATE(HUBDATA)
C ERROR HANDLINGS
      CONTAINS
      SUBROUTINE READHUB_MESG01
      WRITE(IUT,*)'Terminated in <READHUB>:'
      WRITE(IUT,*)'Input HUBFILE ',HUBFILE
      WRITE(IUT,*)'cannot be found. Check path and'
      WRITE(IUT,*)'its existence.'
      CALL FULLSTOP(' ')
      END SUBROUTINE READHUB_MESG01
      SUBROUTINE READHUB_MESG02(MESSG)
      CHARACTER MESSG*(*)
      WRITE(IUT,*)'Terminated in <READHUB>:'
	WRITE(IUT,*)MESSG
      CALL FULLSTOP(' ')
      END SUBROUTINE READHUB_MESG02
      END SUBROUTINE READHUB






	SUBROUTINE READHUB_CTRL(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
* Reads control                                       *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                  !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                          !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB         !!! from readini
	USE COM_INPTTR  ,ONLY : TFERMI                                  !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE          !!! from readhub
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL                       !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                             !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                             !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      LOGICAL DEF,ASK,VAL,NTH,MDF
      DATA PI/3.141592653589793D0/
C **************************************
C READING THE DATA: SELECT CONTROLS 
C  SCHM='LDA       ' - simple LDA, (see HUBPOT for details)
C       'LDA+U     ' - LDA+U
C       'LDA+C     ' - constrained LDA, potential must be given.
C       'LDA+DMFT  ' - LDA+DMFT scheme
C       'LDA+SGM   ' - LDA+Input Self-energy scheme
C       'LDA+GWA   ' - LDA+GW approximation
C  RYEV='Ry' or 'eV' - input/output units
C  CUBC='Cubic' or 'Spherical' harmonics
C  REIM='Real' or 'Complex' - input/output
C MODERN INPUT IS USED, DEFAULTS ARE AVAIABLE
      CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=ctrl',13,'<section=',9,'ok',2)
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! CONTROL PARAMETERS:                   '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Process SCHM
      TCTRL='LDA+U1.1  '
      HELP='! LDA+U1.# / LDA+C / LDA+CU1.# / LDA+G1.#     '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'scheme',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
C Transform to integer keys
	CALL CONVLOW(TCTRL,10)
	CALL RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	ISCHM=-1
	IF(TCTRL(:NB-1).EQ.'lda'      )ISCHM=00
	IF(TCTRL(:NB-1).EQ.'lda+u'    )ISCHM=10
	IF(TCTRL(:NB-1).EQ.'lda+c'    )ISCHM=20
	IF(TCTRL(:NB-1).EQ.'lda+dmft' )ISCHM=30
	IF(TCTRL(:NB-1).EQ.'lda+sgm'  )ISCHM=40
	IF(TCTRL(:NB-1).EQ.'lda+gwa'  )ISCHM=50
	IF(ISCHM.EQ.-1)
     &CALL READHUB_MESG01('Wrong <<scheme>> keyword in HUBFILE')
	IDMF=0
	IF(ISCHM.GT.00.AND.ISCHM.LT.30)IDMF=1
	IF(ISCHM.GE.30.AND.ISCHM.LT.40)IDMF=2
	IF(ISCHM.GE.40.AND.ISCHM.LT.50)IDMF=3
	IF(ISCHM.GE.50.AND.ISCHM.LT.60)IDMF=4
C Process CUBC
      TCTRL='Spherical '
      HELP='! Cubic/Spherical harmonics (output)    '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'yharm',5,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	CALL CONVLOW(TCTRL,10)
	CALL RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	ICUBC=0
	IF(TCTRL(:NB-1).EQ.'cubic')ICUBC=1
	IF(TCTRL(:NB-1).EQ.'spherical')ICUBC=2
	IF(ICUBC.EQ.0)
     &CALL READHUB_MESG01('Wrong <<Yharm>> keyword in HUBFILE')
C Process CUBI
      TCTRL=TCTRL
      HELP='! Cubic/Spherical harmonics (input)     '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'iharm',5,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	CALL CONVLOW(TCTRL,10)
	CALL RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	ICUBI=0
	IF(TCTRL(:NB-1).EQ.'cubic')ICUBI=1
	IF(TCTRL(:NB-1).EQ.'spherical')ICUBI=2
	IF(ICUBI.EQ.0)
     &CALL READHUB_MESG01('Wrong <<Iharm>> keyword in HUBFILE')
C Process NROR
      IF(NORBS.EQ.1)TCTRL='One       '
      IF(NORBS.EQ.2)TCTRL='Both      '
      HELP='! One/Both orbits to read               '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'rorbs',5,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	CALL CONVLOW(TCTRL,10)
	CALL RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	NROR=0
	IF(TCTRL(:NB-1).EQ.'one')NROR=1
	IF(TCTRL(:NB-1).EQ.'both')NROR=2
	IF(NROR.EQ.0)
     &CALL READHUB_MESG01('Wrong <<Rorbs>> keyword in HUBFILE')
C Process NHUB
      NHUB=0
      HELP='! # of correlated states                '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'ncrl',4,NHUB,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
C Process NRSP
      IF(NSPIN.EQ.1)TCTRL='One       '
      IF(NSPIN.EQ.2)TCTRL='Both      '
      HELP='! One/Both spins  to read               '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'rspin',5,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	CALL CONVLOW(TCTRL,10)
	CALL RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	NRSP=0
	IF(TCTRL(:NB-1).EQ.'one')NRSP=1
	IF(TCTRL(:NB-1).EQ.'both')NRSP=2
	IF(NRSP.EQ.0)
     &CALL READHUB_MESG01('Wrong <<Rspin>> keyword in HUBFILE')
C Process REIM
      TCTRL='Complex   '
      HELP='! Real/Complex input/output             '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'format',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	CALL CONVLOW(TCTRL,10)
	CALL RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	IRE=0
	IF(TCTRL(:NB-1).EQ.'real')IRE=1
	IF(TCTRL(:NB-1).EQ.'complex')IRE=2
	IF(IRE.EQ.0)
     &CALL READHUB_MESG01('Wrong <<format>> keyword in HUBFILE')
C **************************************
      CALL READINI_ERRR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,'CTRL')
      DEALLOCATE(HUBCOPY)
	CONTAINS
      SUBROUTINE READHUB_MESG01(MESSG)
      CHARACTER MESSG*(*)
      WRITE(IUT,*)'Terminated in <READHUB>:'
	WRITE(IUT,*)MESSG
      CALL FULLSTOP(' ')
      END SUBROUTINE READHUB_MESG01
	END SUBROUTINE READHUB_CTRL








	SUBROUTINE READHUB_DMFT(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
* Reads DMFT settings                                 *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                  !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                          !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB         !!! from readini
	USE COM_INPTTR  ,ONLY : TFERMI                                  !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE          !!! from readhub
	USE COM_MSBMESH ,ONLY : TEMP,OBND,NMSB,NMSL,WMSB
     &                       ,WMSL,DMSB,DMSL,SMSB,SMSL
     &                       ,FMSB
	USE COM_DMFPAR  ,ONLY : NDIT,EPSDMF,BRDDMF,MIXBRD,ADMIXL,MIXFUN !!! from readhub
      USE COM_LMTOUT  ,ONLY : SCRFILE                                 !!! from readini
	USE COM_FERMI   ,ONLY : EF                                      !!! from ttrints
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                             !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                             !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	COMPLEX*16, ALLOCATABLE :: YMSB(:)
	COMPLEX*16, ALLOCATABLE :: YMSL(:)
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      LOGICAL DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
C **************************************
C READING THE DATA: SELECT DMFT SETTINGS
C NMSB - # of Matsubara energies
C TEMP - # Temperature in K
C NDIT - # of DMFT iterations
      CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=dmft',13,'<section=',9,'ok',2)
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! DMFT SETTINGS                         '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Process NMSB
      NMSB=2
      HELP='! # of Matsubara frequencies            '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'nmsb',4,NMSB,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	NMSBMEM=NMSB
C Process OBND
      OBND=1.D0
      HELP='! # of Matsubara frequencies            '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'obnd',4,IN,1,0,OBND,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	NMSBMEM=NMSB
C Process NMSL
      NMSL=1024
      HELP='! # of linear Matsubara frequencies     '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'nmsl',4,NMSL,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
C Check temperature
	TEMP=TFERMI
	IF(ABS(TEMP).LE.1.D-10.AND.IDMF.EQ.2)
     &CALL READHUB_MESG01('T should not be 0 if DMFT is running')
	TEV=TEMP*13.6058D0 ! eV
	TK=TEV*11605.D0    ! K
C Process NDIT
      NDIT=20
      HELP='! # of DMFT iterations                  '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'ndit',4,NDIT,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
C Use Broyden
      TCTRL='yes       '
      HELP='! Use Broyden mixing for DMFT loop      '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'broy',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
      CALL CONVLOW(TCTRL,10)
      CALL RMBLANK(TCTRL,10)
	MIXBRD=.TRUE.
	IF(TCTRL(1:3).EQ.'yes')MIXBRD=.TRUE.
	IF(TCTRL(1:2).EQ.'no' )MIXBRD=.FALSE.
C Mix Delta(w)/Sigma(w)
      TCTRL='Delta     '
      HELP='! Mix Delta(w)/Sigma(w) in DMFT loop    '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'mixfun',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
      CALL CONVLOW(TCTRL,10)
      CALL RMBLANK(TCTRL,10)
	MIXFUN=0
	IF(TCTRL(1:5).EQ.'delta')MIXFUN=1
	IF(TCTRL(1:5).EQ.'sigma')MIXFUN=2
	IF(MIXFUN.EQ.0)
     &CALL READHUB_MESG01('Mixing function not recognized')
C Process AdmixL
      ADMIXL=0.35D0
      HELP='! Mixing parameter                      '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'mixing',6,IN,1,0,ADMIXL,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	NMSBMEM=NMSB
C Process Fermi energy
      EF=EF ! not to overwrite the value defined above
      HELP='! Fermi energy (Ry)                     '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'efermi',6,IN,1,0,EF,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
C Process Accuracy
      EPSDMF=1.D-05
      HELP='! Accuracy                              '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'epsdmf',6,IN,1,0,EPSDMF,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
C Process Broadening
      BRDDMF=-1.D-03
      HELP='! Broadening                            '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'brddmf',6,IN,1,0,BRDDMF,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
C **************************************
      CALL READINI_ERRR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,'DMFT')
      DEALLOCATE(HUBCOPY)
C CONSTRUCT LINEAR MATSUBARA GRID
	NMSLMEM=NMSL
	ALLOCATE(WMSL(0:NMSLMEM))
	ALLOCATE(DMSL(0:NMSLMEM))
	DO IMSL=0,NMSL
	WMSL(IMSL)=PI*TEMP*DBLE(2*IMSL+1)
	DMSL(IMSL)=0.D0
	ENDDO
	SMSL=2.D0*PI*TEMP
C BUILD LOGARITHMIC MATSUBARA GRID
	ALLOCATE(WMSB(0:NMSBMEM))
	ALLOCATE(DMSB(0:NMSBMEM))
	ALLOCATE(FMSB(0:NMSBMEM))
C LOGARITHMIC GRID
C Correlated band parameters
	OMAX=WMSL(NMSL)-WMSL(0)
	IF(ABS(OMAX).LE.1.D-10)OMAX=1.D-10
C Distribute N points within bandwidth
	NB=NMSB/2
	SMSB=LOG(OMAX/OBND)/DBLE(NMSB-NB)
	BETB=OMAX/(EXP(SMSB*DBLE(NMSB))-1.D0)
	DO IMSB=0,NMSB
	WMSB(IMSB)=BETB*(EXP(SMSB*DBLE(IMSB))-1.D0)+WMSL(0)
	DMSB(IMSB)=BETB*(EXP(SMSB*DBLE(IMSB)))
	ENDDO
	WMSB(0)=WMSL(0)
	WMSB(NMSB)=WMSL(NMSL)
C CONSTRUCT WEIGHT FUNCTION TO PERFORM MATSUBARA SUMS
	ALLOCATE(YMSB(0:NMSB))
	ALLOCATE(YMSL(0:NMSL))
	DO IMSB=0,NMSB
	DO JMSB=0,NMSB
	YMSB(JMSB)=(0.D0,0.D0)
	ENDDO
	YMSB(IMSB)=(1.D0,0.D0)
	DO IMSL=0,NMSL
	YMSL(IMSL)=(0.D0,0.D0)
	ENDDO
	IF(IDMF.EQ.2)
     &CALL CSPLINES(NMSB+1,WMSB,YMSB,NMSL+1,WMSL,YMSL)
	FMSB(IMSB)=0.D0
	DO IMSL=0,NMSL
	FMSB(IMSB)=FMSB(IMSB)+DREAL(YMSL(IMSL))
	ENDDO
	ENDDO
	DEALLOCATE(YMSB,YMSL)
	CONTAINS
      SUBROUTINE READHUB_MESG01(MESSG)
      CHARACTER MESSG*(*)
      WRITE(IUT,*)'Terminated in <READHUB>:'
	WRITE(IUT,*)MESSG
      CALL FULLSTOP(' ')
      END SUBROUTINE READHUB_MESG01
	END SUBROUTINE READHUB_DMFT








	SUBROUTINE READHUB_CORR(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
* Reads correlated states data                        *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                                  !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                          !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB         !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                               !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                             !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE 
     &                       ,KSOLVER,KDBLCNT                         !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG         !!! from readhop
     &                       ,RINPINV,ROUTINV,CHSYS,CHAXE,CHANG,CHINV
	USE COM_IMPDAT  ,ONLY : KCRDSYS,NRUNIMP,KSTATES                 !!! from readhub
	USE COM_QMCDAT  ,ONLY : NTAUIMP,NSWPIMP                         !!! from readhub
      USE COM_FSLATR  ,ONLY : FSLG,FSLU                               !!! from readhub
	USE COM_MSBMESH ,ONLY : NMSB,NMSI,TEMP                          !!! from readhub
	USE COM_DMFPAR  ,ONLY : NDIT                                    !!! from readhub
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL                       !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                             !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                             !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	INTEGER, ALLOCATABLE :: KST(:)
	REAL*8, ALLOCATABLE :: FHUB(:)
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      COMPLEX*16 TS(7,-3:3,0:3)
      LOGICAL EXIST,MODIFIED,DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
     &   ,RSYS*10,RAXE*80,RANG*80,RINV*10,STRARR(80)*1
     &   ,CTHUB*2,CNHUB*1,CLHUB*1
      DATA PI/3.141592653589793D0/
C MEMORY ALLOCATIONS
	ALLOCATE(FSLG(0:2*LMAXBMEM,0:LMAXBMEM,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(FSLU(0:2*LMAXBMEM,0:LMAXBMEM,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(MCRL(NATOMMEM))
	ALLOCATE(LCRL(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KSOLVER(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KDBLCNT(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KCRDSYS(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(NRUNIMP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KSTATES(2*LIMBMEM,2*LIMBMEM,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(NTAUIMP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(NSWPIMP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KST(2*LIMBMEM))
	ALLOCATE(FHUB(0:LMAXBMEM))
      ALLOCATE(RINPAXE(3,0:LMAXBMEM,NATOMMEM))
      ALLOCATE(ROUTAXE(3,0:LMAXBMEM,NATOMMEM))
      ALLOCATE(RINPANG(0:LMAXBMEM,NATOMMEM))
      ALLOCATE(ROUTANG(0:LMAXBMEM,NATOMMEM))
      ALLOCATE(RINPINV(0:LMAXBMEM,NATOMMEM))
      ALLOCATE(ROUTINV(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(CHSYS(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(CHAXE(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(CHANG(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(CHINV(0:LMAXBMEM,NATOMMEM))
C INITIALIZE CORRELATION POINTERS
	DO IATOM=1,NATOM
	ISORT=IS(IATOM) 
	MCRL(IATOM)=0 
      DO L=0,LMAXBMEM
	LCRL(L,IATOM)=0
	KSOLVER(L,IATOM)=0
	KDBLCNT(L,IATOM)=0
	KCRDSYS(L,IATOM)=0
	NRUNIMP(L,IATOM)=0
	ENDDO
	ENDDO 
C INITIALIZE SLATER INTEGRALS
	DO IATOM=1,NATOM
	DO L1=0,LMAXBMEM
	DO L=0,LMAXBMEM
	DO L2=0,2*LMAXBMEM
	FSLU(L2,L1,L,IATOM)=0.D0
	FSLG(L2,L1,L,IATOM)=0.D0
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C LOCAL COORDINATE SYSTEMS ASSUME GLOBAL SYSTEM BY DEFAULT
      DO IATOM=1,NATOM
      ISORT=IS(IATOM)
      DO L=0,ILSB(ISORT)
      RINPAXE(1,L,IATOM)=0.D0
      RINPAXE(2,L,IATOM)=0.D0
      RINPAXE(3,L,IATOM)=1.D0
      ROUTAXE(1,L,IATOM)=0.D0
      ROUTAXE(2,L,IATOM)=0.D0
      ROUTAXE(3,L,IATOM)=1.D0
      RINPANG(L,IATOM)=0.D0
      ROUTANG(L,IATOM)=0.D0
      RINPINV(L,IATOM)=0.D0
      ROUTINV(L,IATOM)=0.D0
      ENDDO
      ENDDO
C **************************************
C READING THE DATA: SELECT CORRELATED STATES
C IAHUB - atom #
C CTHUB - <nl> for correlated state, e.g. '3d'
	ILOC=1
	DO 1000 IHUB=1,NHUB                       !!! over correlated states
      CALL READINI_SECT(HUBDATA(ILOC),NLEN-ILOC+1,KSEC,NSEC,MSEC,
     &'<section=corr',13,'<section=',9,'ok',2)
	ISEC=KSEC+ILOC-1
	ILOC=ISEC+1
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! CORRELATED STATES                     '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Process State
      SNOT='          '
      HELP='! Correlated state pointer              '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'cstate',6,IN,1,0,RE,1,0,CM,1,0,SNOT,20,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	CALL GET_STATE(SNOT,20,IAHUB,ISPIN,MQ,LHUB,M,ICUBC)
	IF(LHUB.GT.LMAXBMEM)
     &CALL READHUB_MESG01('Wrong l for correlated state') 
	MCRL(IAHUB)=1
      LCRL(LHUB,IAHUB)=MQ
C ***********************
C Input coordinate system
C System
      RSYS='          '
      HELP='! global/local coordinate system        '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'inpsys',6,IN,1,0,RE,1,0,CM,1,0,RSYS,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
      CALL CONVLOW(RSYS,10)
      CALL RMBLANK(RSYS,10)
C Rotation axis
      DO I=1,80
      RAXE(I:I)=' '
      ENDDO
      HELP='! rotational axe                        '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'inpaxis',7,IN,1,0,RE,1,0,CM,1,0,RAXE,80,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
      CALL CONVLOW(RAXE,80)
      CALL RMBLANK(RAXE,80)
C Rotation angle
      DO I=1,80
      RANG(I:I)=' '
      ENDDO
      HELP='! rotational angle                      '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'inpangle',8,IN,1,0,RE,1,0,CM,1,0,RANG,80,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
      CALL CONVLOW(RANG,80)
      CALL RMBLANK(RANG,80)
C Apply inversion
      DO I=1,10
      RINV(I:I)=' '
      ENDDO
      HELP='! apply inversion after rotation        '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'inpinv',6,IN,1,0,RE,1,0,CM,1,0,RINV,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
      CALL CONVLOW(RINV,10)
      CALL RMBLANK(RINV,10)
C Set input-system arrays
	IF(RSYS.EQ.'local')THEN
      DO I=1,80
      STRARR(I)=RAXE(I:I)
      ENDDO
      CALL COMPVEC(STRARR,RINPAXE(1,LHUB,IAHUB),3)
      DO I=1,80
      STRARR(I)=RANG(I:I)
      ENDDO
      CALL COMPVEC(STRARR,RINPANG(LHUB,IAHUB),1)
      IF(RINV(1:1).EQ.'n')RINPINV(LHUB,IAHUB)=0.D0
      IF(RINV(1:1).EQ.'y')RINPINV(LHUB,IAHUB)=1.D0
	ENDIF
C ************************
C Output coordinate system
C System
      RSYS='          '
      HELP='! global/local coordinate system        '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'outsys',6,IN,1,0,RE,1,0,CM,1,0,RSYS,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	CHSYS(LHUB,IAHUB)=RSYS
      CALL CONVLOW(RSYS,10)
      CALL RMBLANK(RSYS,10)
C Rotation axis
      DO I=1,80
      RAXE(I:I)=' '
      ENDDO
      HELP='! rotational axe                        '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'outaxis',7,IN,1,0,RE,1,0,CM,1,0,RAXE,80,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	CHAXE(LHUB,IAHUB)=RAXE
      CALL CONVLOW(RAXE,80)
      CALL RMBLANK(RAXE,80)
C Rotation angle
      DO I=1,80
      RANG(I:I)=' '
      ENDDO
      HELP='! rotational angle                      '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'outangle',8,IN,1,0,RE,1,0,CM,1,0,RANG,80,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	CHANG(LHUB,IAHUB)=RANG
      CALL CONVLOW(RANG,80)
      CALL RMBLANK(RANG,80)
C Apply inversion
      DO I=1,10
      RINV(I:I)=' '
      ENDDO
      HELP='! apply inversion after rotation        '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'outinv',6,IN,1,0,RE,1,0,CM,1,0,RINV,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	CHINV(LHUB,IAHUB)=RINV
      CALL CONVLOW(RINV,10)
      CALL RMBLANK(RINV,10)
C Set output-system arrays
	IF(RSYS.EQ.'local')THEN
      DO I=1,80
      STRARR(I)=RAXE(I:I)
      ENDDO
      CALL COMPVEC(STRARR,ROUTAXE(1,LHUB,IAHUB),3)
      DO I=1,80
      STRARR(I)=RANG(I:I)
      ENDDO
      CALL COMPVEC(STRARR,ROUTANG(LHUB,IAHUB),1)
      IF(RINV(1:2).EQ.'no' )ROUTINV(LHUB,IAHUB)=0.D0
      IF(RINV(1:3).EQ.'yes')ROUTINV(LHUB,IAHUB)=1.D0
	ENDIF
C **************************************
C Process Slater Integrals, F(0,ll),F(2,ll),F(4,ll), etc.
	DO L=0,LHUB
	FHUB(L)=0.D0
      HELP='! Slater integrals                      '
      LHLP=40
	WRITE(CLHUB,FMT='(I1)')2*L
	CTHUB='f'//CLHUB
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &CTHUB,2,IN,1,0,FHUB(L),1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	ENDDO
	ISORT=IS(IAHUB)
	DO L2=0,2*LHUB,2
	IF(ISCHM.EQ.0)FHUB(L2/2)=0.D0
	FSLU(L2,LHUB,LHUB,IAHUB)=FHUB(L2/2)
	FSLG(L2,LHUB,LHUB,IAHUB)=FHUB(L2/2)
	ENDDO
C **************************************
C Process impurity solver choice
      TCTRL='none      '
      HELP='! impurity solver choice                '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'solver',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
      CALL CONVLOW(TCTRL,10)
      CALL RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KSOLVER(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'none')KSOLVER(LHUB,IAHUB)=0
	IF(TCTRL(:NB-1).EQ.'hub1')KSOLVER(LHUB,IAHUB)=1
	IF(TCTRL(:NB-1).EQ.'qmcd')KSOLVER(LHUB,IAHUB)=2
	IF(TCTRL(:NB-1).EQ.'qmcm')KSOLVER(LHUB,IAHUB)=3
	IF(TCTRL(:NB-1).EQ.'qmcf')KSOLVER(LHUB,IAHUB)=4
	IF(TCTRL(:NB-1).EQ.'zcwa')KSOLVER(LHUB,IAHUB)=5
	IF(TCTRL(:NB-1).EQ.'flex')KSOLVER(LHUB,IAHUB)=6
	IF(KSOLVER(LHUB,IAHUB).EQ.-1)
     &CALL READHUB_MESG01('Impurity solver not recognized')
	IF(KSOLVER(LHUB,IAHUB).EQ.3)
     &CALL READHUB_MESG01('Impurity solver not implemented')
	IF(KSOLVER(LHUB,IAHUB).EQ.4)
     &CALL READHUB_MESG01('Impurity solver not implemented')
	IF(KSOLVER(LHUB,IAHUB).EQ.6)
     &CALL READHUB_MESG01('Impurity solver not implemented')
C **************************************
C Process double counting choice
      TCTRL='none      '
      HELP='! double counting choice                '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'dcount',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
      CALL CONVLOW(TCTRL,10)
      CALL RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KDBLCNT(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'none')KDBLCNT(LHUB,IAHUB)=0
	IF(TCTRL(:NB-1).EQ.'lda1')KDBLCNT(LHUB,IAHUB)=1
	IF(TCTRL(:NB-1).EQ.'lda2')KDBLCNT(LHUB,IAHUB)=2
	IF(TCTRL(:NB-1).EQ.'lda3')KDBLCNT(LHUB,IAHUB)=3
	IF(TCTRL(:NB-1).EQ.'lda4')KDBLCNT(LHUB,IAHUB)=4
	IF(TCTRL(:NB-1).EQ.'lda5')KDBLCNT(LHUB,IAHUB)=5
	IF(TCTRL(:NB-1).EQ.'sig0')KDBLCNT(LHUB,IAHUB)=6
	IF(TCTRL(:NB-1).EQ.'sigi')KDBLCNT(LHUB,IAHUB)=7
	IF(KDBLCNT(LHUB,IAHUB).EQ.-1)
     &CALL READHUB_MESG01('Double counting not recognized')
C Check for compatibility
	IF(KSOLVER(LHUB,IAHUB).EQ.0.AND.KDBLCNT(LHUB,IAHUB).GT.5)
     &CALL READHUB_MESG01('Wrong double counting for this solver')
	IF(KSOLVER(LHUB,IAHUB).EQ.2.AND.
     &(KDBLCNT(LHUB,IAHUB).NE.6.AND.KDBLCNT(LHUB,IAHUB).NE.7))
     &CALL READHUB_MESG01('Wrong double counting for this solver')
	IF(KSOLVER(LHUB,IAHUB).EQ.3.AND.
     &(KDBLCNT(LHUB,IAHUB).NE.6.AND.KDBLCNT(LHUB,IAHUB).NE.7))
     &CALL READHUB_MESG01('Wrong double counting for this solver')
	IF(KSOLVER(LHUB,IAHUB).EQ.4.AND.
     &(KDBLCNT(LHUB,IAHUB).NE.6.AND.KDBLCNT(LHUB,IAHUB).NE.7))
     &CALL READHUB_MESG01('Wrong double counting for this solver')
	IF(KSOLVER(LHUB,IAHUB).EQ.5.AND.
     &(KDBLCNT(LHUB,IAHUB).NE.6.AND.KDBLCNT(LHUB,IAHUB).NE.7))
     &CALL READHUB_MESG01('Wrong double counting for this solver')
	IF(KSOLVER(LHUB,IAHUB).EQ.6.AND.
     &(KDBLCNT(LHUB,IAHUB).NE.6.AND.KDBLCNT(LHUB,IAHUB).NE.7))
     &CALL READHUB_MESG01('Wrong double counting for this solver')
C **************************************
C Process coordinate system for impurity solver
      TCTRL='cubic     '
      HELP='! system cubic/spherical/diagonal       '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'crdsys',6,IN,1,0,RE,1,0,CM,1,0,TCTRL,10,1,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
      CALL CONVLOW(TCTRL,10)
      CALL RMBLANK(TCTRL,10)
	NB=INDEX(TCTRL,' ')
	KCRDSYS(LHUB,IAHUB)=-1
	IF(TCTRL(:NB-1).EQ.'cubic'    )KCRDSYS(LHUB,IAHUB)=1
	IF(TCTRL(:NB-1).EQ.'spherical')KCRDSYS(LHUB,IAHUB)=2
	IF(TCTRL(:NB-1).EQ.'diagonal' )KCRDSYS(LHUB,IAHUB)=3
	IF(KCRDSYS(LHUB,IAHUB).EQ.-1)
     &CALL READHUB_MESG01('Coordinate system not recognized')
C **************************************
C Process # of runs for impurity solver
	NRUN=1
      HELP='! # of runs for impurity solver         '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'imprun',6,NRUN,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	NRUNIMP(LHUB,IAHUB)=NRUN
C **************************************
C For each run process states which participate
	NDIM=2*(2*LHUB+1)
	DO IRUN=1,NRUNIMP(LHUB,IAHUB)  !!! over runs
	DO IDIM=1,NDIM
	KST(IDIM)=IDIM
	ENDDO
	CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'kstates(:)',10,KST,NDIM,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	DO IDIM=1,NDIM
	KSTATES(IDIM,IRUN,LHUB,IAHUB)=KST(IDIM)
	ENDDO
C Count degeneracies
	NDEG=0
	DO IDIM=1,NDIM
	IF(KSTATES(IDIM,IRUN,LHUB,IAHUB).NE.0)NDEG=NDEG+1
	ENDDO
	IF(MOD(NDEG,2).NE.0)
     &CALL READHUB_MESG01('Wrong kstates(:) pointer')
	ENDDO                          !!! over runs
C Process NSWP
      NSWP=10000
      HELP='! # of sweeps in QMC runs               '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'nswp',4,NSWP,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	NSWPIMP(LHUB,IAHUB)=NSWP
C Process NTAU
      NTAU=16
      HELP='! # of time slices in QMC runs          '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'ntau',4,NTAU,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
	NTAUIMP(LHUB,IAHUB)=NTAU
C **************************************
      CALL READINI_ERRR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,'CORR')
      DEALLOCATE(HUBCOPY)
 1000 CONTINUE                                  !!! over correlated states
C MEMORY DEALLOCATIONS
	DEALLOCATE(FHUB,KST)
	CONTAINS
      SUBROUTINE READHUB_MESG01(MESSG)
      CHARACTER MESSG*(*)
      WRITE(IUT,*)'Terminated in <READHUB>:'
	WRITE(IUT,*)MESSG
      CALL FULLSTOP(' ')
      END SUBROUTINE READHUB_MESG01
	END SUBROUTINE READHUB_CORR







	SUBROUTINE READHUB_IMPL(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
*  Reads correlation functions                        *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE  !!! from readhub
	USE COM_IMPDAT  ,ONLY : NREPIMP,NCNFIMP,KREPIMP,LREPIMP,
     &                        TPARIMP,UPARIMP,GPARIMP,ELOCIMP,
     &                        ZMASIMP,SIG0IMP,SIGIIMP
	USE COM_MSBMESH ,ONLY : NMSB,NMSI,TEMP                  !!! from readhub
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from readhub
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                     !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      LOGICAL DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
	ALLOCATE(NREPIMP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(NCNFIMP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(UPARIMP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(GPARIMP(0:LMAXBMEM,NATOMMEM))
	ALLOCATE(KREPIMP((2*LMAXBMEM+1)*2,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(LREPIMP((2*LMAXBMEM+1)*2,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(ZMASIMP((2*LMAXBMEM+1)*2,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(TPARIMP((2*LMAXBMEM+1)*2,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(ELOCIMP((2*LMAXBMEM+1)*2,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(SIG0IMP((2*LMAXBMEM+1)*2,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(SIGIIMP((2*LMAXBMEM+1)*2,0:LMAXBMEM,NATOMMEM))
C INITIALIZE CORRELATIONAL FUNCTIONS
	DO IATOM=1,NATOM
	DO L=0,LMAXBMEM
	NREPIMP(L,IATOM)=0
	NCNFIMP(L,IATOM)=0
	UPARIMP(L,IATOM)=0.D0
	ENDDO
	ENDDO
	DO IATOM=1,NATOM
	DO L=0,LMAXBMEM
	DO IREP=1,2*(2*LMAXBMEM+1)
	KREPIMP(IREP,L,IATOM)=0
	LREPIMP(IREP,L,IATOM)=0
	ZMASIMP(IREP,L,IATOM)=0.D0
	TPARIMP(IREP,L,IATOM)=0.D0
	ELOCIMP(IREP,L,IATOM)=0.D0
	SIG0IMP(IREP,L,IATOM)=0.D0
	SIGIIMP(IREP,L,IATOM)=0.D0
	ENDDO
	ENDDO
	ENDDO
C **************************************
C READING THE DATA: SELECT ZCWA SETTINGS
      CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=impl',13,'<section=',9,'ok',2)
	IF(MSEC.EQ.0)RETURN
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! IMPURITY LEVELS SETTINGS              '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
	IF(MSEC.EQ.0)GOTO 1000
	IREC=1
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	NDEG=2*(2*L+1)
	IREC=IREC+1
	READ(HUBCOPY(IREC),FMT='(A)')TEXT
	IREC=IREC+1
	READ(HUBCOPY(IREC),*)NREPIMP(L,IATOM),NCNFIMP(L,IATOM)
	DO IREP=1,NREPIMP(L,IATOM)
	IREC=IREC+1
	READ(HUBCOPY(IREC),*) KREPIMP(IREP,L,IATOM),LREPIMP(IREP,L,IATOM),
     &TPARIMP(IREP,L,IATOM),ELOCIMP(IREP,L,IATOM),ZMASIMP(IREP,L,IATOM),
     &SIG0IMP(IREP,L,IATOM),SIGIIMP(IREP,L,IATOM)
	ENDDO
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C **************************************
 1000 DEALLOCATE(HUBCOPY)
	END SUBROUTINE READHUB_IMPL




	SUBROUTINE READHUB_ZCWA(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
*  Reads correlation functions                        *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE  !!! from readhub
	USE COM_IMPDAT  ,ONLY : NREPIMP,NCNFIMP,KREPIMP,LREPIMP,
     &                        TPARIMP,UPARIMP,GPARIMP,ELOCIMP,
     &                        ZMASIMP,SIG0IMP,SIGIIMP
	USE COM_ZCWDAT  ,ONLY : PSUNIMP,XSUNIMP,YSUNIMP,
     &                        PCRFIMP,XCRFIMP,YCRFIMP         !!! from readhub
	USE COM_MSBMESH ,ONLY : NMSB,NMSI,TEMP                  !!! from readhub
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from readhub
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                     !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      LOGICAL DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
	NCNFMEM=1000
	NWAKMEM=5
	ALLOCATE(XSUNIMP(0:2*(2*LMAXBMEM+1),NSPINMEM*LIMBMEM,NATOMMEM))
	ALLOCATE(YSUNIMP(0:2*(2*LMAXBMEM+1),NSPINMEM*LIMBMEM,NATOMMEM))
	ALLOCATE(PSUNIMP(0:2*(2*LMAXBMEM+1),NSPINMEM*LIMBMEM,NATOMMEM))
	ALLOCATE(XCRFIMP(NCNFMEM,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(YCRFIMP(NCNFMEM,0:LMAXBMEM,NATOMMEM))
	ALLOCATE(PCRFIMP(NCNFMEM,0:LMAXBMEM,NATOMMEM))
C INITIALIZE CORRELATIONAL FUNCTIONS
	DO IATOM=1,NATOM
	DO L=0,LMAXBMEM
	DO ICNF=1,NCNFMEM
	PCRFIMP(ICNF,L,IATOM)=0.D0
	XCRFIMP(ICNF,L,IATOM)=0.D0
	YCRFIMP(ICNF,L,IATOM)=0.D0
	ENDDO
	ENDDO
	ENDDO
	DO IATOM=1,NATOM
	DO IDEG=0,2*(2*LMAXBMEM+1)
	DO IDIM=1,LIMBMEM*NSPINMEM
	PSUNIMP(IDEG,IDIM,IATOM)=0.D0
	XSUNIMP(IDEG,IDIM,IATOM)=0.D0
	YSUNIMP(IDEG,IDIM,IATOM)=0.D0
	ENDDO
	ENDDO
	ENDDO
C **************************************
C READING THE DATA: SELECT ZCWA SETTINGS
      CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=zcwa',13,'<section=',9,'ok',2)
	IF(MSEC.EQ.0)RETURN
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! ZCWA SETTINGS                         '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
	IF(MSEC.EQ.0)GOTO 1000
	IREC=1
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	NDEG=2*(2*L+1)
	IREC=IREC+1
	READ(HUBCOPY(IREC),FMT='(A)')TEXT
	DO ICNF=1,NCNFIMP(L,IATOM)             !!! over configurations
	IREC=IREC+1
	READ(HUBCOPY(IREC),*)IDL,XCRFIMP(ICNF,L,IATOM),
     &   YCRFIMP(ICNF,L,IATOM),PCRFIMP(ICNF,L,IATOM)
	ENDDO                                  !!! over configurations
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C **************************************
 1000 DEALLOCATE(HUBCOPY)
	END SUBROUTINE READHUB_ZCWA











	SUBROUTINE READHUB_OCCP(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
*  Reads occupation numbers                           *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE  !!! from readhub
      USE COM_HUBCOM  ,ONLY : VHUB,DHUB,VCNS,DLDA             !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV
	USE COM_MSBMESH ,ONLY : NMSB,NMSI,TEMP                  !!! from readhub
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from readhub
	USE COM_DMFDAT  ,ONLY : SIG0,DEL0,SIGW,DELW,GINV,OIMP,EIMP
     &                       ,SIGI,AINV,ALAM,ELAM
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                     !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	REAL*8, ALLOCATABLE :: OCR(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:)
	COMPLEX*16, ALLOCATABLE :: OCS(:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      COMPLEX*16 TS(7,-3:3,0:3)
      LOGICAL EXIST,MODIFIED,DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
	ALLOCATE(VHUB(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(DHUB(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(VCNS(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(DLDA(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(SIGI(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(SIG0(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(DEL0(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(SIGW(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(DELW(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(GINV(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(OIMP(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(EIMP(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(AINV(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(ALAM(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	ALLOCATE(ELAM(LIMB2MEM,NATOMMEM,NSPINMEM,NORBSMEM))
	CURMEM=CURMEM+14.D0*DBLE(SIZE(VHUB)*LENCMP)/1024.D0**2
C MEMORY ALLOCATIONS
	ALLOCATE(OCR(-LMAXBMEM:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM))
	ALLOCATE(OCS(LIMB2MEM))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
C INITIALIZE OCCUPANCIES "DHUB" AND HUBBARD POTENTIALS "VHUB"
      DO IORBS=1,NORBS                          !!! over iorbs
      DO IUPDN=1,NSPIN                          !!! over iupdn
	DO IATOM=1,NATOM
	ISORT=IS(IATOM) 
      DO L=0,LMAXBMEM                           !!! over l
      DO M=-L,L                                 !!! over m
      DO K=-L,L                                 !!! over k
      LM=L*(L+1)+M+1
      LK=L*(L+1)+K+1
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
C LDA+U ARRAYS
      VHUB(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      VCNS(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      DHUB(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      DLDA(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
C DMFT ARRAYS
      SIGI(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      SIG0(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      DEL0(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      SIGW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      DELW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      GINV(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      OIMP(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      EIMP(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      AINV(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ALAM(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ENDDO                                     !!! over k
      ENDDO                                     !!! over m
      ENDDO                                     !!! over l
      ENDDO                                     !!! over atoms
      ENDDO                                     !!! over iupdn
      ENDDO                                     !!! over iorbs
C READ PARTIAL OCCUPATION NUMBERS, CONSTRAINED POTENTIAL 
	DO 1000 IDVR=1,8
	IF(IDVR.EQ.1)
     &CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=dhub',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.2)
     &CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=dlda',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.3)
     &CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=vcns',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.4)
     &CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=vhub',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.5)
     &CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=sig0',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.6)
     &CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=del0',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.7)
     &CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=oinv',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.8)
     &CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=eimp',13,'<section=',9,'ok',2)
	IF(MSEC.EQ.0)GOTO 1000
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
	IF(IDVR.EQ.1)
     &HELP='! PARTIAL OCCUPANCIES                   '
	IF(IDVR.EQ.2)
     &HELP='! LDA OCCUPANCIES                       '
	IF(IDVR.EQ.3)
     &HELP='! CONSTRAINED POTENTIAL                 '
	IF(IDVR.EQ.4)
     &HELP='! LDA+U POTENTIAL                       '
	IF(IDVR.EQ.5)
     &HELP='! SELF-ENERGY(inf)                      '
	IF(IDVR.EQ.6)
     &HELP='! HYBRIDIZATION(inf)                    '
	IF(IDVR.EQ.7)
     &HELP='! INVERSE OVERLAP INVERSE               '
	IF(IDVR.EQ.8)
     &HELP='! IMPURITY LEVELS                       '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Prepare transformation matrices
      CALL TRANSS(TS)
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL TRANSU(L,RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
	IREC=1
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	IREC=IREC+1
	READ(HUBCOPY(IREC),FMT='(A)')TEXT
C SET LDA+U OCCUPATION NUMBERS
	DO IOR=1,NROR
	DO ISP=1,NRSP
	IF(NORBS.EQ.1)IORBS=1
	IF(NORBS.EQ.2)IORBS=IOR
	IF(NSPIN.EQ.1)IUPDN=1
	IF(NSPIN.EQ.2)IUPDN=ISP
	DO IRM=1,IRE
	IREC=IREC+1
	READ(HUBCOPY(IREC),FMT='(A)')TEXT
	DO M1=-L,L
	IREC=IREC+1
	READ(HUBCOPY(IREC),*)(OCR(M),M=-L,L)
	DO M=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1  !  (l,m' m) packed
	IF(IRM.EQ.1)OCP(IU)=DCMPLX(OCR(M),0.D0)
	IF(IRM.EQ.2)OCP(IU)=DCMPLX(DREAL(OCP(IU)),OCR(M))
	ENDDO                                  !!! over m
	ENDDO                                  !!! over m1
	ENDDO                                  !!! over i
C TRANSFER TO SPHERICAL HARMONICS IF NECESSARY
	IF(ICUBI.EQ.1)THEN
	DO MY1=-L,L
	DO MY =-L,L
      IUY=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MY)+L+MY1+1  !  (l,m' m) packed
	OCS(IUY)=(0.D0,0.D0)
	DO MX1=-L,L
	DO MX =-L,L
      IUX=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MX)+L+MX1+1  !  (l,m' m) packed
      OCS(IUY)=OCS(IUY)+CONJG(TS(MX1+L+1,MY1,L))*
     &OCP(IUX)*TS(MX+L+1,MY,L)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ELSEIF(ICUBI.EQ.2)THEN
	DO M1=-L,L
	DO M=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
      OCS(IU)=OCP(IU)
	ENDDO
	ENDDO
	ENDIF
C TRANSFER TO GLOBAL COORDINATE SYSTEM
	CALL GTURNU(OCP(1),OCS(1),L,ULR(1,IATOM),LIMB2MEM)
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IF(IDVR.EQ.1)
     &DHUB(IUMK,IATOM,IUPDN,IORBS)=
     &DHUB(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK)
	IF(IDVR.EQ.2)
     &DLDA(IUMK,IATOM,IUPDN,IORBS)=
     &DLDA(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK)
	IF(IDVR.EQ.3)
     &VCNS(IUMK,IATOM,IUPDN,IORBS)=
     &VCNS(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK)
	IF(IDVR.EQ.4)
     &VHUB(IUMK,IATOM,IUPDN,IORBS)=
     &VHUB(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK)
	IF(IDVR.EQ.5)
     &SIG0(IUMK,IATOM,IUPDN,IORBS)=
     &SIG0(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK)
	IF(IDVR.EQ.6)
     &DEL0(IUMK,IATOM,IUPDN,IORBS)=
     &DEL0(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK)
	IF(IDVR.EQ.7)
     &OIMP(IUMK,IATOM,IUPDN,IORBS)=
     &OIMP(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK)
	IF(IDVR.EQ.8)
     &EIMP(IUMK,IATOM,IUPDN,IORBS)=
     &EIMP(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK)
      ENDDO
      ENDDO
	ENDDO                                  !!! over isp
	ENDDO                                  !!! over ior
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
      DEALLOCATE(HUBCOPY)
 1000	CONTINUE
C EXPAND ARRAYS
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	IUP=1
	IDN=1
	DFC=1.D0
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)IDN=NSPIN
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)DFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	DHUB(IUMK,IATOM,IUP,1)=DHUB(IUMK,IATOM,IUP,1)*DFC
      DHUB(IUMK,IATOM,IDN,1)=DHUB(IUMK,IATOM,IUP,1)
      DLDA(IUMK,IATOM,IUP,1)=DLDA(IUMK,IATOM,IUP,1)*DFC
      DLDA(IUMK,IATOM,IDN,1)=DLDA(IUMK,IATOM,IUP,1)
      VCNS(IUMK,IATOM,IDN,1)=VCNS(IUMK,IATOM,IUP,1)
      VHUB(IUMK,IATOM,IDN,1)=VHUB(IUMK,IATOM,IUP,1)
      SIG0(IUMK,IATOM,IDN,1)=SIG0(IUMK,IATOM,IUP,1)
      DEL0(IUMK,IATOM,IDN,1)=DEL0(IUMK,IATOM,IUP,1)
      OIMP(IUMK,IATOM,IDN,1)=OIMP(IUMK,IATOM,IUP,1)
      EIMP(IUMK,IATOM,IDN,1)=EIMP(IUMK,IATOM,IUP,1)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C MEMORY DEALLOCATIONS
	DEALLOCATE(OCR,OCP,OCS,ULR)
	END



	SUBROUTINE READHUB_SIGW(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
*  Reads self-energy                                  *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
	USE COM_INPTTR  ,ONLY : TFERMI                          !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE  !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV                 !!! from readhub
	USE COM_MSBMESH ,ONLY : NMSB,NMSI,WMSB,WMSI,TEMP        !!! from msbmesh
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from readhub
	USE COM_DMFDAT  ,ONLY : SIGW,DELW                       !!! from readhub
      USE COM_LMTINP  ,ONLY : ICHUB,HUBFILE                   !!! from readini
      USE COM_LMTOUT  ,ONLY : SCRFILE                         !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                           !!! from channels
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                     !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	REAL*8, ALLOCATABLE :: OCR(:)
	COMPLEX*16, ALLOCATABLE :: SM(:)
	COMPLEX*16, ALLOCATABLE :: SL(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:)
	COMPLEX*16, ALLOCATABLE :: OCS(:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
	COMPLEX*16, ALLOCATABLE :: SIGL(:,:,:,:,:)
	COMPLEX*16, ALLOCATABLE :: DELL(:,:,:,:,:)
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      COMPLEX*16 TS(7,-3:3,0:3)
      LOGICAL EXIST,MODIFIED,DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
	DATA IMA/0/   !!! use D(-iW)=D(iW)^{*} property or not
C MEMORY ALLOCATIONS
	ALLOCATE(OCR(-LMAXBMEM:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM))
	ALLOCATE(OCS(LIMB2MEM))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
C INITIALIZE DMFT ARRAYS
      DO IORBS=1,NORBS                          !!! over iorbs
      DO IUPDN=1,NSPIN                          !!! over iupdn
	DO IATOM=1,NATOM
	ISORT=IS(IATOM) 
      DO L=0,LMAXBMEM                           !!! over l
      DO M=-L,L                                 !!! over m
      DO K=-L,L                                 !!! over k
      LM=L*(L+1)+M+1
      LK=L*(L+1)+K+1
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      SIGW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      DELW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ENDDO                                     !!! over k
      ENDDO                                     !!! over m
      ENDDO                                     !!! over l
      ENDDO                                     !!! over atoms
      ENDDO                                     !!! over iupdn
      ENDDO                                     !!! over iorbs
C Estimate record lenght
	NRECL=0
	DO IATOM=1,NATOM           !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	NRECL=MAX(NRECL,IUMAX*NVAL*2*NSPIN*NORBS)
	ENDDO
C Allocate storage file
	NMSI=NMSB
	ALLOCATE(WMSI(0:NMSI))
	DO IMSB=0,NMSB
	WMSI(IMSB)=WMSB(IMSB)
	ENDDO
	OPEN(UNIT=IUSGM,FILE=SCRFILE(IUSGM),STATUS='NEW',ACCESS='DIRECT',
     &FORM='UNFORMATTED',RECL=NRECL)
	IREC=0
	DO IMSB=0,MAX(NMSB,NMSI)   !!! over Matsubara frequencies
	DO IATOM=1,NATOM           !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                      !!! over atoms
	ENDDO                      !!! over Matsubara frequencies
C DO OVER TWO SECTIONS
	DO 1000 IDVR=1,2                         !!! over sigw/delw
	IF(IDVR.EQ.1)
     &CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=sigw',13,'<section=',9,'ok',2)
	IF(IDVR.EQ.2)
     &CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=delw',13,'<section=',9,'ok',2)
	IF(MSEC.EQ.0)GOTO 1000
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
	IF(IDVR.EQ.1)
     &HELP='! DMFT SELF-ENERGY                      '
	IF(IDVR.EQ.2)
     &HELP='! HYBRIDIZATION FUNCTION                '
      IF(MSEC.NE.0)HUBDATA(ISEC)(IPOS+1:IPOS+40)=HELP
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Prepare transformation matrices
      CALL TRANSS(TS)
C Process NMSI
      NMSI=NMSB
      HELP='! # of input Matsubara frequencies      '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'nmsi',4,NMSI,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
C Reallocate storage file	
	IF(IDVR.EQ.1)THEN
	DEALLOCATE(WMSI)
	ALLOCATE(WMSI(0:NMSI))
	CLOSE(UNIT=IUSGM,STATUS='DELETE')
	OPEN(UNIT=IUSGM,FILE=SCRFILE(IUSGM),STATUS='NEW',ACCESS='DIRECT',
     &FORM='UNFORMATTED',RECL=NRECL)
	IREC=0
	DO IMSB=0,MAX(NMSB,NMSI)   !!! over Matsubara frequencies
	DO IATOM=1,NATOM           !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                      !!! over atoms
	ENDDO                      !!! over Matsubara frequencies
	ENDIF
C Start reading the data
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL TRANSU(L,RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
	IPOS=2
	IREC=0
	DO IMSB=0,NMSI                         !!! over imsb
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	IPOS=IPOS+1
	READ(HUBCOPY(IPOS),FMT='(A)')TEXT
	STRING=HUBCOPY(IPOS)
C Process Frequency
	HELP='                                       '
	LHLP=40
	CALL READINI_CHAR(HUBCOPY(IPOS),STRING,1,1,
     &  'wmsb',4,IN,1,0,WFREQ,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.FALSE.)
	WMSI(IMSB)=WFREQ
	IF(NMSB.EQ.NMSI)THEN
	IF(ABS(WMSB(IMSB)-WMSI(IMSB)).LE.1.D-05)
     &WMSI(IMSB)=WMSB(IMSB)
	ENDIF
C READ SELF-ENERGY/HYBRIDIZATION
	DO IOR=1,NROR                          !!! over ior
	DO ISP=1,NRSP                          !!! over isp
	IF(NORBS.EQ.1)IORBS=1
	IF(NORBS.EQ.2)IORBS=IOR
	IF(NSPIN.EQ.1)IUPDN=1
	IF(NSPIN.EQ.2)IUPDN=ISP
	DO IRM=1,IRE
	IPOS=IPOS+1
	READ(HUBCOPY(IPOS),FMT='(A)')TEXT
	DO M1=-L,L
	IPOS=IPOS+1
	READ(HUBCOPY(IPOS),*)(OCR(M),M=-L,L)
	DO M=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1  !  (l,m' m) packed
	IF(IRM.EQ.1)OCP(IU)=DCMPLX(OCR(M),0.D0)
	IF(IRM.EQ.2)OCP(IU)=DCMPLX(DREAL(OCP(IU)),OCR(M))
	ENDDO                                  !!! over m
	ENDDO                                  !!! over m1
	ENDDO                                  !!! over i
C TRANSFER TO SPHERICAL HARMONICS IF NECESSARY
	IF(ICUBI.EQ.1)THEN
	DO MY1=-L,L
	DO MY =-L,L
      IUY=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MY)+L+MY1+1  !  (l,m' m) packed
	OCS(IUY)=(0.D0,0.D0)
	DO MX1=-L,L
	DO MX =-L,L
      IUX=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MX)+L+MX1+1  !  (l,m' m) packed
      OCS(IUY)=OCS(IUY)+CONJG(TS(MX1+L+1,MY1,L))*
     &OCP(IUX)*TS(MX+L+1,MY,L)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ELSEIF(ICUBI.EQ.2)THEN
	DO M1=-L,L
	DO M=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
      OCS(IU)=OCP(IU)
	ENDDO
	ENDDO
	ENDIF
C TRANSFER TO GLOBAL COORDINATE SYSTEM
	CALL GTURNU(OCP(1),OCS(1),L,ULR(1,IATOM),LIMB2MEM)
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IF(IDVR.EQ.1)
     &SIGW(IUMK,IATOM,IUPDN,IORBS)=
     &SIGW(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK)
	IF(IDVR.EQ.2)
     &DELW(IUMK,IATOM,IUPDN,IORBS)=
     &DELW(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK)
      ENDDO
      ENDDO
	ENDDO                                  !!! over isp
	ENDDO                                  !!! over ior
	ENDDO                                  !!! over l
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                                  !!! over atoms
	ENDDO                                  !!! over imsb
      DEALLOCATE(HUBCOPY)
 1000	CONTINUE                               !!! over sigw/delw
C REINTERPOLATE ONTO NEW MATSUBARA GRID
	ALLOCATE(SM(0:NMSI))
	ALLOCATE(SL(0:NMSB))
C START LOOPING OVER ATOMS
	DO 3000 IATOM=1,NATOM               !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO 2000 L=0,ILSB(ISORT)             !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE
C ALLOCATE ARRAYS
	NM=MAX(NMSB,NMSI)
	ALLOCATE(SIGL(-L:L,-L:L,0:NM,NSPINMEM,NORBSMEM))
	ALLOCATE(DELL(-L:L,-L:L,0:NM,NSPINMEM,NORBSMEM))
C READ SELF-ENERGY
	IREC=0
	DO IMSB=0,NMSI                      !!! over Matsubara frequencies
	DO JATOM=1,NATOM                    !!! over atoms
	JSORT=IS(JATOM)
	IF(MCRL(JATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IF(JATOM.NE.IATOM)CYCLE
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                               !!! over atoms
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO K=-L,L
	DO M=-L,L
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
	SIGL(K,M,IMSB,IUPDN,IORBS)=SIGW(IU,IATOM,IUPDN,IORBS)
	DELL(K,M,IMSB,IUPDN,IORBS)=DELW(IU,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO                               !!! over Matsubara frequencies
C START LOOPING:
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO K=-L,L
	DO M=-L,L
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
C SELF-ENERGY
	DO IMSI=0,NMSI
	SM(IMSI)=SIGL(K,M,IMSI,IUPDN,IORBS)
	ENDDO
	NEND=0
	DO IMSB=0,NMSB
	IF(WMSB(IMSB).LE.WMSI(NMSI))NEND=IMSB
	ENDDO
	IF(NEND.EQ.0)CALL FULLSTOP('<READHUB_SIGW> : Wrong grid')
      CALL CSPLINES(NMSI+1,WMSI,SM,NEND+1,WMSB,SL)
	DO IMSB=0,NEND
	SIGL(K,M,IMSB,IUPDN,IORBS)=SL(IMSB)
	ENDDO
	DO IMSB=NEND+1,NMSB
	SIGL(K,M,IMSB,IUPDN,IORBS)=DCMPLX(DREAL(SL(NEND)),0.D0)
	ENDDO
C HYBRIDIZATION
	DO IMSI=0,NMSI
	SM(IMSI)=DELL(K,M,IMSI,IUPDN,IORBS)
	ENDDO
	NEND=0
	DO IMSB=0,NMSB
	IF(WMSB(IMSB).LE.WMSI(NMSI))NEND=IMSB
	ENDDO
	IF(NEND.EQ.0)CALL FULLSTOP('<READHUB_SIGW> : Wrong grid')
      CALL CSPLINES(NMSI+1,WMSI,SM,NEND+1,WMSB,SL)
	DO IMSB=0,NEND
	DELL(K,M,IMSB,IUPDN,IORBS)=SL(IMSB)
	ENDDO
	DO IMSB=NEND+1,NMSB
	DELL(K,M,IMSB,IUPDN,IORBS)=DCMPLX(0.D0,0.D0)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C STORE SELF-ENERGY
	IREC=0
	DO IMSB=0,NMSB                      !!! over Matsubara frequencies
	DO JATOM=1,NATOM                    !!! over atoms
	JSORT=IS(JATOM)
	IF(MCRL(JATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IF(JATOM.NE.IATOM)CYCLE
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
	DO K=-L,L
	DO M=-L,L
	IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+K+1 !!! (l,k m)
	SIGW(IU,IATOM,IUPDN,IORBS)=SIGL(K,M,IMSB,IUPDN,IORBS)
	DELW(IU,IATOM,IUPDN,IORBS)=DELL(K,M,IMSB,IUPDN,IORBS)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	IUP=1
	IDN=1
	DFC=1.D0
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)IDN=NSPIN
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)DFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      SIGW(IU,IATOM,IDN,1)=SIGW(IU,IATOM,IUP,1)
      DELW(IU,IATOM,IDN,1)=DELW(IU,IATOM,IUP,1)
	ENDDO
	ENDDO
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                               !!! over atoms
	ENDDO                               !!! over Matsubara frequencies
	DEALLOCATE(SIGL,DELL)
 2000	CONTINUE                            !!! over l
 3000	CONTINUE                            !!! over atoms
C MEMORY DEALLOCATIONS
	DEALLOCATE(OCR,OCP,OCS,ULR,SM,SL)
	END



	SUBROUTINE READHUB_SGMP(HUBDATA,NLEN,NROR,NRSP,ICUBI)
*******************************************************
* Reads Self-energy settings                          *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_LMTMEM                                          !!! from setup
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
	USE COM_INPTTR  ,ONLY : TFERMI                          !!! from readini
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE  !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV                 !!! from readhub
	USE COM_MSBMESH ,ONLY : NMSE,WMSE,TEMP                  !!! from msbmesh
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from readhub
	USE COM_DMFDAT  ,ONLY : SIGW                            !!! from readhub
      USE COM_LMTINP  ,ONLY : ICHUB,HUBFILE                   !!! from readini
      USE COM_LMTOUT  ,ONLY : SCRFILE                         !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                           !!! from channels
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
      USE COM_MASHDEP ,ONLY : NVAL,LENCMP                     !!! from setup
C SET LOCAL VARIABLES 
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER, INTRINSIC :: SIZE
	REAL*8, ALLOCATABLE :: OCR(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:)
	COMPLEX*16, ALLOCATABLE :: OCS(:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      CHARACTER*150, ALLOCATABLE :: HUBCOPY(:)
      COMPLEX*16 TS(7,-3:3,0:3)
      LOGICAL EXIST,MODIFIED,DEF,ASK,VAL,NTH,MDF
      CHARACTER*150, HUBDATA(NLEN)
      CHARACTER TEXT*64,TCTRL*10,STRING*150,HELP*40,SNOT*20
      DATA PI/3.141592653589793D0/
	DATA IMA/0/   !!! use D(-iW)=D(iW)^{*} property or not
C MEMORY ALLOCATIONS
	ALLOCATE(OCR(-LMAXBMEM:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM))
	ALLOCATE(OCS(LIMB2MEM))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
C INITIALIZE SELF-ENERGY ARRAY
      DO IORBS=1,NORBS                          !!! over iorbs
      DO IUPDN=1,NSPIN                          !!! over iupdn
	DO IATOM=1,NATOM
	ISORT=IS(IATOM) 
      DO L=0,LMAXBMEM                           !!! over l
      DO M=-L,L                                 !!! over m
      DO K=-L,L                                 !!! over k
      LM=L*(L+1)+M+1
      LK=L*(L+1)+K+1
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      SIGW(IU,IATOM,IUPDN,IORBS)=(0.D0,0.D0)
      ENDDO                                     !!! over k
      ENDDO                                     !!! over m
      ENDDO                                     !!! over l
      ENDDO                                     !!! over atoms
      ENDDO                                     !!! over iupdn
      ENDDO                                     !!! over iorbs
C Estimate record lenght
	NRECL=0
	DO IATOM=1,NATOM           !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	NRECL=MAX(NRECL,IUMAX*NVAL*NSPIN*NORBS)
	ENDDO
C READ SECTION
      CALL READINI_SECT(HUBDATA(1),NLEN,ISEC,NSEC,MSEC,
     &'<section=sgmp',13,'<section=',9,'ok',2)
	IF(MSEC.EQ.0)CALL READHUB_MESG01('No self-energy information')
C Set section title
      DO IPOS=1,150
      IF(HUBDATA(ISEC)(IPOS:IPOS).EQ.'>')EXIT
      ENDDO
      DO JPOS=IPOS+1,150
      IF(MSEC.NE.0)HUBDATA(ISEC)(JPOS:JPOS)=' '
      ENDDO
      IF(IPOS.LE.50)THEN
      HELP='! EXTERNAL SELF-ENERGY                  '
      ENDIF
C Allocate temporary file
      ALLOCATE(HUBCOPY(NSEC))
      DO ILIN=1,MSEC
      HUBCOPY(ILIN)=HUBDATA(ILIN+ISEC-1)
      ENDDO
C Prepare transformation matrices
      CALL TRANSS(TS)
C Process NMSE
      NMSE=0
      HELP='! # of input frequencies                '
      LHLP=40
      CALL READINI_CHAR(HUBDATA(ISEC),HUBCOPY(1),NSEC,MSEC,
     &'nmse',4,NMSE,1,1,RE,1,0,CM,1,0,' ',1,0,HELP,LHLP,
     &DEF,ASK,VAL,NTH,.TRUE.)
C Allocate storage file	
	ALLOCATE(WMSE(0:NMSE))
	OPEN(UNIT=IUSGM,FILE=SCRFILE(IUSGM),STATUS='NEW',ACCESS='DIRECT',
     &FORM='UNFORMATTED',RECL=NRECL)
	IREC=0
	DO IMSB=0,NMSE             !!! over frequencies
	DO IATOM=1,NATOM           !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                      !!! over atoms
	ENDDO                      !!! over frequencies
C Start reading the data
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL TRANSU(L,RINPAXE(1,L,IATOM),RINPANG(L,IATOM),
     &RINPINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
	IPOS=2
	IREC=0
	DO IMSB=0,NMSE                         !!! over imsb
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	IPOS=IPOS+1
	READ(HUBCOPY(IPOS),FMT='(A)')TEXT
	STRING=HUBCOPY(IPOS)
C Process Frequency
	HELP='                                       '
	LHLP=40
	CALL READINI_CHAR(HUBCOPY(IPOS),STRING,1,1,
     &  'wmse',4,IN,1,0,WFREQ,1,1,CM,1,0,' ',1,0,HELP,LHLP,
     &  DEF,ASK,VAL,NTH,.FALSE.)
	WMSE(IMSB)=WFREQ
C READ EXTERNAL SELF-ENERGY
	DO IOR=1,NROR                          !!! over ior
	DO ISP=1,NRSP                          !!! over isp
	IF(NORBS.EQ.1)IORBS=1
	IF(NORBS.EQ.2)IORBS=IOR
	IF(NSPIN.EQ.1)IUPDN=1
	IF(NSPIN.EQ.2)IUPDN=ISP
	DO IRM=1,IRE
	IPOS=IPOS+1
	READ(HUBCOPY(IPOS),FMT='(A)')TEXT
	DO M1=-L,L
	IPOS=IPOS+1
	READ(HUBCOPY(IPOS),*)(OCR(M),M=-L,L)
	DO M=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1  !  (l,m' m) packed
	IF(IRM.EQ.1)OCP(IU)=DCMPLX(OCR(M),0.D0)
	IF(IRM.EQ.2)OCP(IU)=DCMPLX(DREAL(OCP(IU)),OCR(M))
	ENDDO                                  !!! over m
	ENDDO                                  !!! over m1
	ENDDO                                  !!! over i
C TRANSFER TO SPHERICAL HARMONICS IF NECESSARY
	IF(ICUBI.EQ.1)THEN
	DO MY1=-L,L
	DO MY =-L,L
      IUY=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MY)+L+MY1+1  !  (l,m' m) packed
	OCS(IUY)=(0.D0,0.D0)
	DO MX1=-L,L
	DO MX =-L,L
      IUX=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MX)+L+MX1+1  !  (l,m' m) packed
      OCS(IUY)=OCS(IUY)+CONJG(TS(MX1+L+1,MY1,L))*
     &OCP(IUX)*TS(MX+L+1,MY,L)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ELSEIF(ICUBI.EQ.2)THEN
	DO M1=-L,L
	DO M=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
      OCS(IU)=OCP(IU)
	ENDDO
	ENDDO
	ENDIF
C TRANSFER TO GLOBAL COORDINATE SYSTEM
	CALL GTURNU(OCP(1),OCS(1),L,ULR(1,IATOM),LIMB2MEM)
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      SIGW(IUMK,IATOM,IUPDN,IORBS)=
     &SIGW(IUMK,IATOM,IUPDN,IORBS)+OCP(IUMK)
      ENDDO
      ENDDO
	ENDDO                                  !!! over isp
	ENDDO                                  !!! over ior
	ENDDO                                  !!! over l
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                                  !!! over atoms
	ENDDO                                  !!! over imsb
      DEALLOCATE(HUBCOPY)
C STORE SELF-ENERGY
	DO 3000 IATOM=1,NATOM                  !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO 2000 L=0,ILSB(ISORT)                !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	IREC=0
	DO IMSB=0,NMSE                         !!! over Matsubara frequencies
	DO JATOM=1,NATOM                       !!! over atoms
	JSORT=IS(JATOM)
	IF(MCRL(JATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IF(JATOM.NE.IATOM)CYCLE
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	IUP=1
	IDN=1
	DFC=1.D0
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)IDN=NSPIN
	IF(NSPIN.EQ.2.AND.NRSP.EQ.1)DFC=0.5D0
      DO M=-L,L
      DO K=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      SIGW(IU,IATOM,IDN,1)=SIGW(IU,IATOM,IUP,1)
	ENDDO
	ENDDO
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	WRITE(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	ENDDO                               !!! over atoms
	ENDDO                               !!! over Matsubara frequencies
 2000	CONTINUE                            !!! over l
 3000	CONTINUE                            !!! over atoms
C MEMORY DEALLOCATIONS
	DEALLOCATE(OCR,OCP,OCS,ULR)
	CONTAINS
      SUBROUTINE READHUB_MESG01(MESSG)
      CHARACTER MESSG*(*)
      WRITE(IUT,*)'Terminated in <READHUB>:'
	WRITE(IUT,*)MESSG
      CALL FULLSTOP(' ')
      END SUBROUTINE READHUB_MESG01
	END SUBROUTINE READHUB_SGMP






	SUBROUTINE WRITEHUB(KW)
*******************************************************
*  Updates HUBFILE.                                   * 
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
	USE COM_MSBMESH ,ONLY : NMSB,NMSL,TEMP,OBND             !!! from readhub
	USE COM_DMFPAR  ,ONLY : NDIT,EPSDMF,BRDDMF,MIXBRD,
     &                        ADMIXL,MIXFUN                   !!! from readhub
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE,IWR! from readhub
     &                       ,KSOLVER,KDBLCNT                 !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV,CHSYS,CHAXE,CHANG,CHINV
      USE COM_FSLATR  ,ONLY : FSLG,FSLU                       !!! from readhub
	USE COM_IMPDAT  ,ONLY : KCRDSYS,NRUNIMP,KSTATES
     &                       ,NREPIMP,NCNFIMP,KREPIMP,LREPIMP
     &                       ,TPARIMP,UPARIMP,GPARIMP,ELOCIMP
     &                       ,SIG0IMP,SIGIIMP,ZMASIMP
	USE COM_QMCDAT  ,ONLY : NTAUIMP,NSWPIMP                 !!! from readhub
	USE COM_ZCWDAT  ,ONLY : XSUNIMP,YSUNIMP,PCRFIMP,XCRFIMP,YCRFIMP
	USE COM_FERMI   ,ONLY : EF                              !!! from ttrints
      USE COM_LMTINP  ,ONLY : ICHUB,HUBFILE                   !!! from readini
      USE COM_LMTOUT  ,ONLY : ICHBR,HBRFILE                   !!! from readini
      USE COM_CHANNELS,ONLY : IUINP                           !!! from readini
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: FHUB(:)
	REAL*8, ALLOCATABLE :: OCR(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:)
	COMPLEX*16, ALLOCATABLE :: OCS(:)
      LOGICAL EXIST
      CHARACTER TEXT*64,CTHUB*2,CNHUB*1,CLHUB*1,TCTRL*10
     &   ,TL(0:3)*1,TCUBC(-3:3,0:3,2)*10,TPR(-3:3,-3:3)*10,TDP*10
     &   ,TRE(2)*4,TSP(2,2)*5,STRING*150,FILE*130,HELP*40
      DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
     &   ,TRE/'REAL','IMAG'/,TSP/'Up-Up','Dn-Dn','Up-Dn','Dn-Up'/
C RETURN KEYS
	IF(ICHUB.EQ.0.AND.KW.EQ.1)RETURN
	IF(ICHBR.EQ.0.AND.KW.EQ.2)RETURN
	CALL INTCUBC(TCUBC)
C MEMORY ALLOCATIONS
	ALLOCATE(FHUB(0:LMAXBMEM))
	ALLOCATE(OCR(-LMAXBMEM:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM))
	ALLOCATE(OCS(LIMB2MEM))
C ARE THERE CORRELATIONS?
	IPRINT=0
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	IPRINT=IPRINT+1
	ENDDO
	ENDDO
C CHECK EXISTENCE
	IF(KW.EQ.1)FILE=HUBFILE
	IF(KW.EQ.2)FILE=HBRFILE
      INQUIRE(FILE=FILE,EXIST=EXIST)
      IF(EXIST)THEN
      OPEN(UNIT=IUINP,FILE=FILE,STATUS='OLD',FORM='FORMATTED')
	CLOSE(UNIT=IUINP,STATUS='DELETE')
	ENDIF
      OPEN(UNIT=IUINP,FILE=FILE,STATUS='NEW',FORM='FORMATTED')
C START STORAGE
	WRITE(IUINP,10100)
	WRITE(IUINP,10110)
10100	FORMAT(1X,'<FILE=HUBFILE,INPUT=MODERN,TRACE=TRUE>')
10110 FORMAT('****************************************************')
C PRINT CONTROL PARAMETERS
	WRITE(IUINP,10120)
10120 FORMAT(1X,'<SECTION=CTRL>',14X,'! CONTROL PARAMETERS:')
C Process SCHM
      HELP='! LDA+U/LDA+C/LDA+DMFT/LDA+SGM          '
	IF(ISCHM.EQ.00)TCTRL='LDA      '      
	IF(ISCHM.EQ.10)TCTRL='LDA+U    ' 
	IF(ISCHM.EQ.20)TCTRL='LDA+C    '    
	IF(ISCHM.EQ.30)TCTRL='LDA+DMFT '  
	IF(ISCHM.EQ.40)TCTRL='LDA+SGM  '
	IF(ISCHM.EQ.50)TCTRL='LDA+GWA  '
	WRITE(IUINP,10130)TCTRL,HELP
10130 FORMAT(1X,'Scheme=',A10,11X,A40)
C Process CUBC
      HELP='! Cubic/Spherical harmonics (output)    '
	IF(ICUBC.EQ.1)TCTRL='cubic    '
	IF(ICUBC.EQ.2)TCTRL='spherical'
	WRITE(IUINP,10150)TCTRL,HELP
10150 FORMAT(1X,'Yharm =',A10,11X,A40)
C Process CUBI
      HELP='! Cubic/Spherical harmonics (input)     '
	IF(ICUBC.EQ.1)TCTRL='cubic    '
	IF(ICUBC.EQ.2)TCTRL='spherical'
	WRITE(IUINP,10160)TCTRL,HELP
10160 FORMAT(1X,'Iharm =',A10,11X,A40)
C Process KOR
      IF(NORBS.EQ.1)TCTRL='One     '
      IF(NORBS.EQ.2)TCTRL='Both    '
      HELP='! One/Both orbits to read               '
	WRITE(IUINP,10170)TCTRL,HELP
10170 FORMAT(1X,'Rorbs =',A10,11X,A40)
C Process KSP
      IF(NSPIN.EQ.1)TCTRL='One     '
      IF(NSPIN.EQ.2)TCTRL='Both    '
      HELP='! One/Both spins  to read               '
	WRITE(IUINP,10180)TCTRL,HELP
10180 FORMAT(1X,'Rspin =',A10,11X,A40)
C Process REIM
      HELP='! Real/Complex input/output             '
	IF(IRE.EQ.1)TCTRL='real      '
	IF(IRE.EQ.2)TCTRL='complex   '
	WRITE(IUINP,10190)TCTRL,HELP
10190 FORMAT(1X,'Format=',A10,11X,A40)
C Process NHUB
      HELP='! # of correlated states                '
	WRITE(IUINP,10200)NHUB,HELP
10200 FORMAT(1X,'Ncrl  =',I3,18X,A40)
C PRINT DMFT DATA
	WRITE(IUINP,10310)
10310 FORMAT(1X,'<SECTION=DMFT>',14X,'! DMFT SETTINGS:')
C Process NMSB
      HELP='! # of Matsubara frequencies            '
	WRITE(IUINP,10330)NMSB,HELP
10330 FORMAT(1X,'Nmsb  =',I10,11X,A40)
C Process OBND
      HELP='! Effective bandwidth for the grid      '
	WRITE(IUINP,10331)OBND,HELP
10331 FORMAT(1X,'Obnd  =',G14.7,7X,A40)
C Process NMSL
      HELP='! # of linear Matsubara frequencies     '
	WRITE(IUINP,10332)NMSL,HELP
10332 FORMAT(1X,'Nmsl  =',I10,11X,A40)
C Process NDIT
      HELP='! # of DMFT iterations                  '
	WRITE(IUINP,10340)NDIT,HELP
10340 FORMAT(1X,'Ndit  =',I10,11X,A40)
C Process Broyden
      HELP='! Use Broyden mixing for DMFT loop      '
	IF(MIXBRD)WRITE(IUINP,10341)HELP
	IF(.NOT.MIXBRD)WRITE(IUINP,10342)HELP
10341 FORMAT(1X,'Broy  = yes',17X,A40)
10342 FORMAT(1X,'Broy  = no ',17X,A40)
C Process Mixing function
      HELP='! Mix Delta(w)/Sigma(w) in DMFT loop    '
	IF(MIXFUN.EQ.1)WRITE(IUINP,10343)HELP
	IF(MIXFUN.EQ.2)WRITE(IUINP,10344)HELP
10343 FORMAT(1X,'MixFun= Delta',15X,A40)
10344 FORMAT(1X,'MixFun= Sigma',15X,A40)
C Process AdmixL
      HELP='! Mixing parameter in DMFT loop         '
	WRITE(IUINP,10345)ADMIXL,HELP
10345 FORMAT(1X,'Mixing=',G14.7,7X,A40)
C Process Ef
      HELP='! Fermi energy (Ry)                     '
	WRITE(IUINP,10350)EF,HELP
10350 FORMAT(1X,'Efermi=',G14.7,7X,A40)
C Process EpsDMF
      HELP='! Accuracy                              '
	WRITE(IUINP,10360)EPSDMF,HELP
10360 FORMAT(1X,'EpsDMF=',G14.7,7X,A40)
C Process BrdDMF
      HELP='! Broadening                            '
	WRITE(IUINP,10370)BRDDMF,HELP
10370 FORMAT(1X,'BrdDMF=',G14.7,7X,A40)
C PRINT CORRELATED STATES
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	WRITE(IUINP,10510)
10510 FORMAT(1X,'<SECTION=CORR>',14X,'! CORRELATED STATES:')
C Process LHUB
      HELP='! Correlated state pointer              '
	CALL SET_STATE(TCTRL,10,TXTAT(ISORT),IATOM,0,
     &LCRL(L,IATOM),L,-100,ICUBC)
	WRITE(IUINP,10540)TCTRL,HELP
10540 FORMAT(1X,'Cstate  =',A10,9X,A40)
C ************************
C Output coordinate system
      HELP='! global/local coordinate system        '
	CALL FIND_LBLANK(CHSYS(L,IATOM),10,NBL)
	IF(NBL.GT.0)
     &WRITE(IUINP,10550)CHSYS(L,IATOM),HELP
10550 FORMAT(1X,'OutSys  =',A10,9X,A40)
      HELP='! rotational axis                       '
	CALL FIND_LBLANK(CHAXE(L,IATOM),80,NBL)
	IF(NBL.LE.18.AND.NBL.GT.0)
     &WRITE(IUINP,10560)CHAXE(L,IATOM)(1:18),HELP
	IF(NBL.GT.18)
     &WRITE(IUINP,10565)CHAXE(L,IATOM)
10560 FORMAT(1X,'OutAxis =',A18,1X,A40)
10565 FORMAT(1X,'OutAxis =',A80)
      HELP='! rotational angle                      '
	CALL FIND_LBLANK(CHANG(L,IATOM),80,NBL)
	IF(NBL.LE.18.AND.NBL.GT.0)
     &WRITE(IUINP,10570)CHANG(L,IATOM)(1:18),HELP
	IF(NBL.GT.18)
     &WRITE(IUINP,10575)CHANG(L,IATOM)
10570 FORMAT(1X,'OutAngle=',A18,1X,A40)
10575 FORMAT(1X,'OutAngle=',A80)
C Apply inversion
      HELP='! apply inversion after rotation        '
	CALL FIND_LBLANK(CHINV(L,IATOM),10,NBL)
	IF(NBL.GT.0)
     &WRITE(IUINP,10580)CHINV(L,IATOM),HELP
10580 FORMAT(1X,'OutInv  =',A10,9X,A40)
C ************************
C Input coordinate system
      HELP='! global/local coordinate system        '
	CALL FIND_LBLANK(CHSYS(L,IATOM),10,NBL)
	IF(NBL.GT.0)
     &WRITE(IUINP,10590)CHSYS(L,IATOM),HELP
10590 FORMAT(1X,'InpSys  =',A10,9X,A40)
      HELP='! rotational axis                       '
	CALL FIND_LBLANK(CHAXE(L,IATOM),80,NBL)
	IF(NBL.LE.18.AND.NBL.GT.0)
     &WRITE(IUINP,10600)CHAXE(L,IATOM)(1:18),HELP
	IF(NBL.GT.18)
     &WRITE(IUINP,10605)CHAXE(L,IATOM)
10600 FORMAT(1X,'InpAxis =',A18,1X,A40)
10605 FORMAT(1X,'InpAxis =',A80)
      HELP='! rotational angle                      '
	CALL FIND_LBLANK(CHANG(L,IATOM),80,NBL)
	IF(NBL.LE.18.AND.NBL.GT.0)
     &WRITE(IUINP,10610)CHANG(L,IATOM)(1:18),HELP
	IF(NBL.GT.18)
     &WRITE(IUINP,10615)CHANG(L,IATOM)
10610 FORMAT(1X,'InpAngle=',A18,1X,A40)
10615 FORMAT(1X,'InpAngle=',A80)
C Apply inversion
      HELP='! apply inversion after rotation        '
	CALL FIND_LBLANK(CHINV(L,IATOM),10,NBL)
	IF(NBL.GT.0)
     &WRITE(IUINP,10620)CHINV(L,IATOM),HELP
10620 FORMAT(1X,'InpInv  =',A10,9X,A40)
C **************************************
C Process Slater Integrals, F(0,ll),F(2,ll),F(4,ll), etc.
	DO L2=0,2*L,2
      HELP='! Slater integrals                      '
	IF(L2.EQ.0)
     &WRITE(IUINP,10710)FSLU(L2,L,L,IATOM),HELP
	IF(L2.EQ.2)
     &WRITE(IUINP,10720)FSLU(L2,L,L,IATOM),HELP
	IF(L2.EQ.4)
     &WRITE(IUINP,10730)FSLU(L2,L,L,IATOM),HELP
	IF(L2.EQ.6)
     &WRITE(IUINP,10740)FSLU(L2,L,L,IATOM),HELP
	IF(L2.EQ.8)
     &WRITE(IUINP,10750)FSLU(L2,L,L,IATOM),HELP
10710 FORMAT(1X,'F0 =',G14.7,10X,A40)
10720 FORMAT(1X,'F2 =',G14.7,10X,A40)
10730 FORMAT(1X,'F4 =',G14.7,10X,A40)
10740 FORMAT(1X,'F6 =',G14.7,10X,A40)
10750 FORMAT(1X,'F8 =',G14.7,10X,A40)
	ENDDO
C **************************************
C Process Impurity Solver Choice
      HELP='! impurity solver choice                '
	IF(KSOLVER(L,IATOM).EQ.0)TCTRL='None      '
	IF(KSOLVER(L,IATOM).EQ.1)TCTRL='Hub1      '  
	IF(KSOLVER(L,IATOM).EQ.2)TCTRL='qmcD      '
	IF(KSOLVER(L,IATOM).EQ.3)TCTRL='qmcM      '
	IF(KSOLVER(L,IATOM).EQ.4)TCTRL='qmcF      '
	IF(KSOLVER(L,IATOM).EQ.5)TCTRL='ZCWA      '
	IF(KSOLVER(L,IATOM).EQ.6)TCTRL='FLEX      '
	WRITE(IUINP,10760)TCTRL,HELP
10760 FORMAT(1X,'Solver =',A10,10X,A40)
C **************************************
C Process double counting choice
      HELP='! double counting choice                '
	IF(KDBLCNT(L,IATOM).EQ.0)TCTRL='None      '
	IF(KDBLCNT(L,IATOM).EQ.1)TCTRL='lda1      '  
	IF(KDBLCNT(L,IATOM).EQ.2)TCTRL='lda2      '
	IF(KDBLCNT(L,IATOM).EQ.3)TCTRL='lda3      '
	IF(KDBLCNT(L,IATOM).EQ.4)TCTRL='lda4      '
	IF(KDBLCNT(L,IATOM).EQ.5)TCTRL='lda5      '
	IF(KDBLCNT(L,IATOM).EQ.6)TCTRL='sig0      '
	IF(KDBLCNT(L,IATOM).EQ.7)TCTRL='sigI      '
	WRITE(IUINP,10770)TCTRL,HELP
10770 FORMAT(1X,'Dcount =',A10,10X,A40)
C **************************************
	IF(IDMF.EQ.2)THEN
C Process coordinate system for impurity solver
      HELP='! system cubic/spherical/diagonal       '
	IF(KCRDSYS(L,IATOM).EQ.1)TCTRL='cubic     '
	IF(KCRDSYS(L,IATOM).EQ.2)TCTRL='spherical '
	IF(KCRDSYS(L,IATOM).EQ.3)TCTRL='diagonal  '
	WRITE(IUINP,10775)TCTRL,HELP
10775 FORMAT(1X,'CrdSys =',A10,10X,A40)
C **************************************
C Process # of runs for impurity solver
      HELP='! # of runs for impurity solver         '
	WRITE(IUINP,10780)NRUNIMP(L,IATOM),HELP
10780 FORMAT(1X,'ImpRun =',I10,10X,A40)
C **************************************
C For each run process states which participate
	NDEG=2*(2*L+1)
	DO IRUN=1,NRUNIMP(L,IATOM)  !!! over runs
      HELP='! states pointer in each run            '
	IF(L.EQ.0)
     &WRITE(IUINP,10785)(KSTATES(IDEG,IRUN,L,IATOM),IDEG=1,NDEG),HELP
10785 FORMAT(1X,'kStates(:)=',2(I1,1X),13X,A40)
	IF(L.EQ.1)
     &WRITE(IUINP,10786)(KSTATES(IDEG,IRUN,L,IATOM),IDEG=1,NDEG),HELP
10786 FORMAT(1X,'kStates(:)=',3(I1,1X),3X,3(I1,1X),1X,A40)
	IF(L.EQ.2)
     &WRITE(IUINP,10787)(KSTATES(IDEG,IRUN,L,IATOM),IDEG=1,NDEG)
10787 FORMAT(1X,'kStates(:)=',5(I1,1X),3X,4(I1,1X),I2)
	IF(L.EQ.3)
     &WRITE(IUINP,10788)(KSTATES(IDEG,IRUN,L,IATOM),IDEG=1,NDEG)
10788 FORMAT(1X,'kStates(:)=',7(I1,1X),3X,2(I1,1X),5(I2,1X))
	ENDDO
	ENDIF
	IF(IDMF.EQ.2.AND.(KSOLVER(L,IATOM).EQ.2
     &              .OR.KSOLVER(L,IATOM).EQ.3
     &              .OR.KSOLVER(L,IATOM).EQ.4))THEN
C **************************************
C Process NSWP
      HELP='! # of sweeps in Monte-Carlo            '
	WRITE(IUINP,10790)NSWPIMP(L,IATOM),HELP
10790 FORMAT(1X,'Nswp  =',I10,11X,A40)
C Process NTAU
      HELP='! # of time slices in QMC               '
	WRITE(IUINP,10791)NTAUIMP(L,IATOM),HELP
10791 FORMAT(1X,'Ntau  =',I10,11X,A40)
	ENDIF
	ENDDO                                  !!! over correlated states
	ENDDO                                  !!! over correlated states
C PRINT PARTIAL OCCUPATION NUMBERS, CONSTRAINED POTENTIAL 
	DO IDVR=1,8
	CALL PRINTH(IUINP,IDVR)
	ENDDO
	IF(IDMF.EQ.2)THEN                      !!! for DMFT=2 calculation
C PRINT IMPURITY LEVELS INFO
	IF(IPRINT.NE.0)WRITE(IUINP,10800)
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	NDEG=2*(2*L+1)
	CALL SET_STATE(TCTRL,10,TXTAT(ISORT),IATOM,0,LCRL(L,IATOM),
     &L,-100,ICUBC)
	WRITE(IUINP,10810)TCTRL
	WRITE(IUINP,10820)NREPIMP(L,IATOM),NCNFIMP(L,IATOM)
	DO IREP=1,NREPIMP(L,IATOM)
	WRITE(IUINP,10830)    KREPIMP(IREP,L,IATOM),LREPIMP(IREP,L,IATOM),
     &TPARIMP(IREP,L,IATOM),ELOCIMP(IREP,L,IATOM),ZMASIMP(IREP,L,IATOM),
     &SIG0IMP(IREP,L,IATOM),SIGIIMP(IREP,L,IATOM)
	ENDDO
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
10800 FORMAT(1X,'<SECTION=IMPL>',14X,'! IMPURITY LEVELS:')
10810	FORMAT('! Impurtity data for the state ',A)
10820	FORMAT(1X,I3,1X,I3)
10830 FORMAT(1X,I2,1X,I2,5(1X,G12.5))
10840 FORMAT(3X,I2,2X,G14.7,2X,G14.7,2X,G14.7)
10850 FORMAT(1X,I4,2X,G14.7,2X,G14.7,2X,G14.7)
C PRINT PARTIAL OCCUPATION NUMBERS, CONSTRAINED POTENTIAL 
	DO IDVR=1,2
	CALL PRINTM(IUINP,IDVR)
	ENDDO
C PRINT ZCWA DATA
C	IF(IPRINT.NE.0)WRITE(IUINP,10900)
C	DO IATOM=1,NATOM                       !!! over atoms
C	ISORT=IS(IATOM)
C	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
C	DO L=0,ILSB(ISORT)                     !!! over l
C	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C	NDEG=2*(2*L+1)
C	CALL SET_STATE(TCTRL,10,TXTAT(ISORT),IATOM,0,LCRL(L,IATOM),
C    &L,-100,ICUBC)
C	WRITE(IUINP,10910)TCTRL
C	DO ICNF=1,NCNFIMP(L,IATOM)             !!! over configurations
C	WRITE(IUINP,10920)ICNF,XCRFIMP(ICNF,L,IATOM),
C    &YCRFIMP(ICNF,L,IATOM) ,PCRFIMP(ICNF,L,IATOM)
C	ENDDO                                  !!! over configurations
C	ENDDO                                  !!! over l
C	ENDDO                                  !!! over atoms
10900 FORMAT(1X,'<SECTION=ZCWA>',14X,'! CORRELATIONAL FUNCTIONS:')
10910	FORMAT('! Impurtity data for the state ',A)
10920 FORMAT(1X,I4,2X,G14.7,2X,G14.7,2X,G14.7)
	ENDIF
C PRINT INPUT SELF-ENERGY DATA IF DMFT=3
	IF(IDMF.EQ.3)THEN
	CALL PRINTD(IUINP)
	ENDIF
	CLOSE(IUINP)
C MEMORY DEALLOCATIONS
	DEALLOCATE(FHUB,OCR,OCP,OCS)
	END




	SUBROUTINE PRINTH(ICHAN,IDVR)
*******************************************************
*  Prints occupancies to the terminal                 *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
	USE COM_MSBMESH ,ONLY : NMSB,TEMP                       !!! from msbmesh
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from msbmesh
	USE COM_DMFDAT  ,ONLY : SIG0,DEL0,SIGW,DELW,GINV,OIMP,EIMP
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE,IWR! from readhub
      USE COM_HUBCOM  ,ONLY : VHUB,DHUB,VCNS,DLDA             !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV
      USE COM_LMTINP  ,ONLY : ICHUB                           !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                           !!! from readini
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: FHUB(:)
	REAL*8, ALLOCATABLE :: OCR(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:)
	COMPLEX*16, ALLOCATABLE :: OCS(:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16 TC(-3:3,7,0:3)
      LOGICAL EXIST
      CHARACTER TEXT*64,CTHUB*2,CNHUB*1,CLHUB*1,TCTRL*10
     &   ,TL(0:3)*1,TCUBC(-3:3,0:3,2)*10,TPR(-3:3,-3:3)*10,TDP*10
     &   ,TRE(2)*4,TSP(2,2)*5,STRING*150,FILE*130,HELP*40
      DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
     &   ,TRE/'REAL','IMAG'/,TSP/'Up-Up','Dn-Dn','Up-Dn','Dn-Up'/
C RETURN KEYS
      IF(ICHUB.LE.0)RETURN
	CALL INTCUBC(TCUBC)
C MEMORY ALLOCATIONS
	ALLOCATE(FHUB(0:LMAXBMEM))
	ALLOCATE(OCR(-LMAXBMEM:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM))
	ALLOCATE(OCS(LIMB2MEM))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
	IPRINT=0
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	IPRINT=IPRINT+1
	ENDDO
	ENDDO
	IF(IPRINT.EQ.0)GOTO 1000
C Prepare transformation matrices
      CALL TRANSC(TC)
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL TRANSU(L,ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C PRINT PARTIAL OCCUPATION NUMBERS, CONSTRAINED POTENTIAL 
	IF(IDVR.EQ.1)WRITE(ICHAN,10010)
10010 FORMAT(1X,'<SECTION=DHUB>',14X,'! PARTIAL OCCUPANCIES:')
	IF(IDVR.EQ.2)WRITE(ICHAN,10020)
10020 FORMAT(1X,'<SECTION=DLDA>',14X,'! LDA OCCUPANCIES:')
	IF(IDVR.EQ.3)WRITE(ICHAN,10030)
10030 FORMAT(1X,'<SECTION=VCNS>',14X,'! CONSTRAINED POTENTIAL:')
	IF(IDVR.EQ.4)WRITE(ICHAN,10040)
10040 FORMAT(1X,'<SECTION=VHUB>',14X,'! LDA+U POTENTIAL:')
	IF(IDVR.EQ.5)WRITE(ICHAN,10050)
10050 FORMAT(1X,'<SECTION=SIG0>',14X,'! SELF-ENERGY(inf):')
	IF(IDVR.EQ.6)WRITE(ICHAN,10060)
10060 FORMAT(1X,'<SECTION=DEL0>',14X,'! HYBRIDIZATION(inf):')
	IF(IDVR.EQ.7)WRITE(ICHAN,10070)
10070 FORMAT(1X,'<SECTION=OINV>',14X,'! INVERSE OVERLAP INVERSE:')
	IF(IDVR.EQ.8)WRITE(ICHAN,10080)
10080 FORMAT(1X,'<SECTION=EIMP>',14X,'! IMPURITY LEVELS:')
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	CALL SET_STATE(TCTRL,10,TXTAT(ISORT),IATOM,0,
     &LCRL(L,IATOM),L,-100,ICUBC)
	WRITE(ICHAN,10100)TCTRL
C SET PARTIAL OCCUPATION NUMBERS
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IF(IDVR.EQ.1)
     &OCP(IUMK)=DHUB(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.2)
     &OCP(IUMK)=DLDA(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.3)
     &OCP(IUMK)=VCNS(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.4)
     &OCP(IUMK)=VHUB(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.5)
     &OCP(IUMK)=SIG0(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.6)
     &OCP(IUMK)=DEL0(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.7)
     &OCP(IUMK)=OIMP(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.8)
     &OCP(IUMK)=EIMP(IUMK,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
C  TRANSFER TO LOCAL COORDINATE SYSTEM 
	CALL LTURNU(OCS,OCP,L,ULR(1,IATOM),LIMB2MEM)
C  TRANSFORM TO CUBIC HARMONICS IF REQUIRED
	IF(ICUBC.EQ.1)THEN
	DO MX1=-L,L
	DO MX =-L,L
      IUX=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MX)+L+MX1+1  !  (l,m' m) packed
	OCP(IUX)=(0.D0,0.D0)
	DO MY1=-L,L
	DO MY =-L,L
      IUY=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MY)+L+MY1+1  !  (l,m' m) packed
      OCP(IUX)=OCP(IUX)+
     &CONJG(TC(MY1,MX1+L+1,L))*OCS(IUY)*TC(MY,MX+L+1,L)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ELSEIF(ICUBC.EQ.2)THEN
	DO M1=-L,L
	DO M=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
	OCP(IU)=OCS(IU)
	ENDDO
	ENDDO
	ENDIF
C  WRITE REAL/IMAG PARTS SEPARATELY
	DO I=1,IRE                             !!! over re/im
	IF(L.EQ.0)
     &WRITE(ICHAN,10110)(TCUBC(M,L,ICUBC),M=-L,L),
     &TRE(I),TSP(IUPDN,IORBS)
	IF(L.EQ.1)
     &WRITE(ICHAN,10120)(TCUBC(M,L,ICUBC),M=-L,L),
     &TRE(I),TSP(IUPDN,IORBS)
	IF(L.EQ.2)
     &WRITE(ICHAN,10130)(TCUBC(M,L,ICUBC),M=-L,L),
     &TRE(I),TSP(IUPDN,IORBS)
	IF(L.EQ.3)
     &WRITE(ICHAN,10140)(TCUBC(M,L,ICUBC),M=-L,L),
     &TRE(I),TSP(IUPDN,IORBS)
	DO M1=-L,L
	DO M=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
      IF(I.EQ.1)TPAR=DREAL(OCP(IU))
      IF(I.EQ.2)TPAR=DIMAG(OCP(IU))
      IF(ABS(TPAR).GT.1.D-07)TPR(M1,M)=TDP(TPAR,1.D-07)
      IF(ABS(TPAR).LE.1.D-07)TPR(M1,M)=' 0.0000000'
	ENDDO
	ENDDO
	DO M1=-L,L
	IF(L.EQ.0)WRITE(ICHAN,10150)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.1)WRITE(ICHAN,10160)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.2)WRITE(ICHAN,10170)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.3)WRITE(ICHAN,10180)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	ENDDO
	ENDDO                                  !!! over re/im
	ENDDO                                  !!! over ispin
	ENDDO                                  !!! over iorbs
	ENDDO                                  !!! over l
	ENDDO                                  !!! over iatom
C MEMORY DEALLOCATIONS
 1000	CONTINUE
	DEALLOCATE(FHUB,OCR,OCP,OCS,ULR)
C FORMATS
10100 FORMAT(1X,'cState=',A10,11X, '! spin up/dn-up/dn data are :')
10105 FORMAT(1X,'cState=',A10,' ; wMsb = ',G14.7)
10110 FORMAT(1X,1(A10,2X),2X,A4,',',A5)
10120 FORMAT(1X,3(A10,2X),2X,A4,',',A5)
10130 FORMAT(1X,5(A10,2X),2X,A4,',',A5)
10140 FORMAT(1X,7(A10,2X),2X,A4,',',A5)
10150 FORMAT(1X,1(A10,2X),2X,A10)
10160 FORMAT(1X,3(A10,2X),2X,A10)
10170 FORMAT(1X,5(A10,2X),2X,A10)
10180 FORMAT(1X,7(A10,2X),2X,A10)
	END



	SUBROUTINE PRINTM(ICHAN,IDVR)
*******************************************************
*  Writes self-energy to the channel ICHAN            *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
	USE COM_MSBMESH ,ONLY : NMSB,TEMP,WMSB                  !!! from msbmesh
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from msbmesh
	USE COM_DMFDAT  ,ONLY : SIG0,SIGW,DELW,GINV,OIMP        !!! from readhub
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE,IWR! from readhub
      USE COM_HUBCOM  ,ONLY : VHUB,DHUB,VCNS,DLDA             !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV,CHSYS,CHAXE,CHANG,CHINV
      USE COM_FSLATR  ,ONLY : FSLG,FSLU                       !!! from readhub
      USE COM_LMTINP  ,ONLY : ICHUB                           !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                           !!! from readini
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: FHUB(:)
	REAL*8, ALLOCATABLE :: OCR(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:)
	COMPLEX*16, ALLOCATABLE :: OCS(:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16 TC(-3:3,7,0:3)
      LOGICAL EXIST
      CHARACTER TEXT*64,CTHUB*2,CNHUB*1,CLHUB*1,TCTRL*10
     &   ,TL(0:3)*1,TCUBC(-3:3,0:3,2)*10,TPR(-3:3,-3:3)*10,TDP*10
     &   ,TRE(2)*4,TSP(2,2)*5,STRING*150,FILE*130,HELP*40
      DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
     &   ,TRE/'REAL','IMAG'/,TSP/'Up-Up','Dn-Dn','Up-Dn','Dn-Up'/
	IF(IDMF.NE.2)RETURN
      IF(ICHUB.LE.0)RETURN
	CALL INTCUBC(TCUBC)
C MEMORY ALLOCATIONS
	ALLOCATE(FHUB(0:LMAXBMEM))
	ALLOCATE(OCR(-LMAXBMEM:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM))
	ALLOCATE(OCS(LIMB2MEM))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
	IPRINT=0
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	IPRINT=IPRINT+1
	ENDDO
	ENDDO
	IF(IPRINT.EQ.0)GOTO 1000
C Prepare transformation matrices
      CALL TRANSC(TC)
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL TRANSU(L,ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C PRINT PARTIAL OCCUPATION NUMBERS, CONSTRAINED POTENTIAL 
	IF(IDVR.EQ.1)WRITE(ICHAN,10010)
10010 FORMAT(1X,'<SECTION=SIGW>',14X,'! DMFT SELF-ENERGY:')
	IF(IDVR.EQ.2)WRITE(ICHAN,10020)
10020 FORMAT(1X,'<SECTION=DELW>',14X,'! HYBRIDIZATION FUNCTION:')
	IREC=0
C Process NMSB
      HELP='! # of frequencies                      '
	WRITE(ICHAN,10030)NMSB,HELP
10030 FORMAT(1X,'nMsi  =',I10,11X,A40)
	DO IMSB=0,NMSB                         !!! over Matsubara frequencies
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS),
     &(((DELW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	CALL SET_STATE(TCTRL,10,TXTAT(ISORT),IATOM,0,
     &LCRL(L,IATOM),L,-100,ICUBC)
	WRITE(ICHAN,10100)TCTRL,WMSB(IMSB)
C SET LDA+U OCCUPATION NUMBERS
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
	IF(IDVR.EQ.1)
     &OCP(IUMK)=SIGW(IUMK,IATOM,IUPDN,IORBS)
	IF(IDVR.EQ.2)
     &OCP(IUMK)=DELW(IUMK,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
C  TRANSFER TO LOCAL COORDINATE SYSTEM 
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      OCS(IUMK)=(0.D0,0.D0)
      DO M1=-L,L
      DO K1=-L,L
      IUM1M =L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M )+L+M1+1       !  (l,m1 m) packed
      IUK1K =L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K )+L+K1+1       !  (l,k1 k) packed
      IUM1K1=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K1)+L+M1+1       !  (l,m1 k1) packed
C  Tlocal(m,k)=U(m1,m)*Tglobal(m1,k1)*conjg(U(k1,k))
      OCS(IUMK)=OCS(IUMK)+
     &ULR(IUM1M,IATOM)*OCP(IUM1K1)*CONJG(ULR(IUK1K,IATOM))
      ENDDO
      ENDDO
      ENDDO
      ENDDO
C  TRANSFORM TO CUBIC HARMONICS IF REQUIRED
	IF(ICUBC.EQ.1)THEN
	DO MX1=-L,L
	DO MX =-L,L
      IUX=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MX)+L+MX1+1  !  (l,m' m) packed
	OCP(IUX)=(0.D0,0.D0)
	DO MY1=-L,L
	DO MY =-L,L
      IUY=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MY)+L+MY1+1  !  (l,m' m) packed
      OCP(IUX)=OCP(IUX)+
     &CONJG(TC(MY1,MX1+L+1,L))*OCS(IUY)*TC(MY,MX+L+1,L)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ELSEIF(ICUBC.EQ.2)THEN
	DO M1=-L,L
	DO M=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
	OCP(IU)=OCS(IU)
	ENDDO
	ENDDO
	ENDIF
C  WRITE REAL/IMAG PARTS SEPARATELY
	DO I=1,IRE                             !!! over re/im
	IF(L.EQ.0)
     &WRITE(ICHAN,10110)(TCUBC(M,L,ICUBC),M=-L,L),
     &TRE(I),TSP(IUPDN,IORBS)
	IF(L.EQ.1)
     &WRITE(ICHAN,10120)(TCUBC(M,L,ICUBC),M=-L,L),
     &TRE(I),TSP(IUPDN,IORBS)
	IF(L.EQ.2)
     &WRITE(ICHAN,10130)(TCUBC(M,L,ICUBC),M=-L,L),
     &TRE(I),TSP(IUPDN,IORBS)
	IF(L.EQ.3)
     &WRITE(ICHAN,10140)(TCUBC(M,L,ICUBC),M=-L,L),
     &TRE(I),TSP(IUPDN,IORBS)
	DO M1=-L,L
	DO M=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
      IF(I.EQ.1)TPAR=DREAL(OCP(IU))
      IF(I.EQ.2)TPAR=DIMAG(OCP(IU))
      IF(ABS(TPAR).GT.1.D-07)TPR(M1,M)=TDP(TPAR,1.D-07)
      IF(ABS(TPAR).LE.1.D-07)TPR(M1,M)=' 0.0000000'
	ENDDO
	ENDDO
	DO M1=-L,L
	IF(L.EQ.0)WRITE(ICHAN,10150)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.1)WRITE(ICHAN,10160)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.2)WRITE(ICHAN,10170)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.3)WRITE(ICHAN,10180)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	ENDDO
	ENDDO                                  !!! over re/im
	ENDDO                                  !!! over ispin
	ENDDO                                  !!! over iorbs
	ENDDO                                  !!! over l
	ENDDO                                  !!! over iatom
	ENDDO                                  !!! over Matsubara frequencies
C MEMORY DEALLOCATIONS
 1000	CONTINUE
	DEALLOCATE(FHUB,OCR,OCP,OCS,ULR)
C FORMATS
10100 FORMAT(1X,'cState=',A10,' ; wMsb = ',G14.7)
10110 FORMAT(1X,1(A10,2X),2X,A4,',',A5)
10120 FORMAT(1X,3(A10,2X),2X,A4,',',A5)
10130 FORMAT(1X,5(A10,2X),2X,A4,',',A5)
10140 FORMAT(1X,7(A10,2X),2X,A4,',',A5)
10150 FORMAT(1X,1(A10,2X),2X,A10)
10160 FORMAT(1X,3(A10,2X),2X,A10)
10170 FORMAT(1X,5(A10,2X),2X,A10)
10180 FORMAT(1X,7(A10,2X),2X,A10)
	END





	SUBROUTINE PRINTD(ICHAN)
*******************************************************
*  Writes self-energy to the channel ICHAN            *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY : LMTO,IASA,IDMF                  !!! from readini
      USE COM_PARAM   ,ONLY : NATOM,NSORT,NSPIN,NORBS,IS,ILSB !!! from readini
      USE COM_KAPPAR  ,ONLY : NPAN,NKAP                       !!! from readini
      USE COM_PANPAR  ,ONLY : LTPAN,LSPAN                     !!! from readini
	USE COM_MSBMESH ,ONLY : NMSE,TEMP,WMSE                  !!! from msbmesh
	USE COM_DMFPAR  ,ONLY : NDIT                            !!! from msbmesh
	USE COM_DMFDAT  ,ONLY : SIG0,SIGW,DELW,GINV,OIMP        !!! from readhub
      USE COM_HUBDAT  ,ONLY : NHUB,MCRL,LCRL,ISCHM,ICUBC,IRE,IWR! from readhub
      USE COM_HUBCOM  ,ONLY : VHUB,DHUB,VCNS,DLDA             !!! from readhub
      USE COM_HUBSYS  ,ONLY : RINPAXE,RINPANG,ROUTAXE,ROUTANG !!! from readhop
     &                       ,RINPINV,ROUTINV,CHSYS,CHAXE,CHANG,CHINV
      USE COM_FSLATR  ,ONLY : FSLG,FSLU                       !!! from readhub
      USE COM_LMTINP  ,ONLY : ICHUB                           !!! from readini
      USE COM_CHANNELS,ONLY : IUSGM                           !!! from readini
      USE COM_LABELS  ,ONLY : TXTAT,TXTSP,TXTEL               !!! from readini
      USE COM_UNIT    ,ONLY : IUN,IUR,IUT                     !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: FHUB(:)
	REAL*8, ALLOCATABLE :: OCR(:)
	COMPLEX*16, ALLOCATABLE :: OCP(:)
	COMPLEX*16, ALLOCATABLE :: OCS(:)
      COMPLEX*16, ALLOCATABLE :: ULR(:,:)
      COMPLEX*16 TC(-3:3,7,0:3)
      LOGICAL EXIST
      CHARACTER TEXT*64,CTHUB*2,CNHUB*1,CLHUB*1,TCTRL*10
     &   ,TL(0:3)*1,TCUBC(-3:3,0:3,2)*10,TPR(-3:3,-3:3)*10,TDP*10
     &   ,TRE(2)*4,TSP(2,2)*5,STRING*150,FILE*130,HELP*40
      DATA PI/3.141592653589793D0/,TL/'s','p','d','f'/
     &   ,TRE/'REAL','IMAG'/,TSP/'Up-Up','Dn-Dn','Up-Dn','Dn-Up'/
	IF(IDMF.NE.3)RETURN
      IF(ICHUB.LE.0)RETURN
	CALL INTCUBC(TCUBC)
C MEMORY ALLOCATIONS
	ALLOCATE(FHUB(0:LMAXBMEM))
	ALLOCATE(OCR(-LMAXBMEM:LMAXBMEM))
	ALLOCATE(OCP(LIMB2MEM))
	ALLOCATE(OCS(LIMB2MEM))
      ALLOCATE(ULR(LIMB2MEM,NATOMMEM))
	IPRINT=0
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE
	DO L=0,ILSB(ISORT)
	IF(LCRL(L,IATOM).EQ.0)CYCLE
	IPRINT=IPRINT+1
	ENDDO
	ENDDO
	IF(IPRINT.EQ.0)GOTO 1000
C Prepare transformation matrices
      CALL TRANSC(TC)
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
C Build transformation matrices. Operation transforms GLOBAL system to LOCAL system
C (maybe vice versa, it has to be checked!)
      CALL TRANSU(L,ROUTAXE(1,L,IATOM),ROUTANG(L,IATOM),
     &ROUTINV(L,IATOM ),ULR(1,IATOM),LIMB2MEM)
	ENDDO                                  !!! over l
	ENDDO                                  !!! over atoms
C PRINT PARTIAL OCCUPATION NUMBERS, CONSTRAINED POTENTIAL 
	WRITE(ICHAN,10010)
10010 FORMAT(1X,'<SECTION=SGMP>',14X,'! IMPUT SELF-ENERGY:')
	IREC=0
C Process NMSE
      HELP='! # of frequencies                      '
	WRITE(ICHAN,10030)NMSE,HELP
10030 FORMAT(1X,'nMse  =',I10,11X,A40)
	DO IMSB=0,NMSE                         !!! over Matsubara frequencies
	DO IATOM=1,NATOM                       !!! over atoms
	ISORT=IS(IATOM)
	IF(MCRL(IATOM).EQ.0)CYCLE              !!! at_corr
	IREC=IREC+1
	IUMAX=(ILSB(ISORT)+1)*(2*ILSB(ISORT)+1)*(2*ILSB(ISORT)+3)/3
	READ(IUSGM,REC=IREC)
     &(((SIGW(IU,IATOM,IUPDN,IORBS),IU=1,IUMAX),IUPDN=1,NSPIN),
     &IORBS=1,NORBS)
	DO L=0,ILSB(ISORT)                     !!! over l
	IF(LCRL(L,IATOM).EQ.0)CYCLE            !!! l_corr
	CALL SET_STATE(TCTRL,10,TXTAT(ISORT),IATOM,0,
     &LCRL(L,IATOM),L,-100,ICUBC)
	WRITE(ICHAN,10100)TCTRL,WMSE(IMSB)
C SET LDA+U OCCUPATION NUMBERS
	DO IORBS=1,NORBS
	DO IUPDN=1,NSPIN
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      OCP(IUMK)=SIGW(IUMK,IATOM,IUPDN,IORBS)
	ENDDO
	ENDDO
C  TRANSFER TO LOCAL COORDINATE SYSTEM 
      DO M=-L,L
      DO K=-L,L
      IUMK=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K)+L+M+1           !  (l,m k) packed
      OCS(IUMK)=(0.D0,0.D0)
      DO M1=-L,L
      DO K1=-L,L
      IUM1M =L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M )+L+M1+1       !  (l,m1 m) packed
      IUK1K =L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K )+L+K1+1       !  (l,k1 k) packed
      IUM1K1=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+K1)+L+M1+1       !  (l,m1 k1) packed
C  Tlocal(m,k)=U(m1,m)*Tglobal(m1,k1)*conjg(U(k1,k))
      OCS(IUMK)=OCS(IUMK)+
     &ULR(IUM1M,IATOM)*OCP(IUM1K1)*CONJG(ULR(IUK1K,IATOM))
      ENDDO
      ENDDO
      ENDDO
      ENDDO
C  TRANSFORM TO CUBIC HARMONICS IF REQUIRED
	IF(ICUBC.EQ.1)THEN
	DO MX1=-L,L
	DO MX =-L,L
      IUX=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MX)+L+MX1+1  !  (l,m' m) packed
	OCP(IUX)=(0.D0,0.D0)
	DO MY1=-L,L
	DO MY =-L,L
      IUY=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+MY)+L+MY1+1  !  (l,m' m) packed
      OCP(IUX)=OCP(IUX)+
     &CONJG(TC(MY1,MX1+L+1,L))*OCS(IUY)*TC(MY,MX+L+1,L)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	ELSEIF(ICUBC.EQ.2)THEN
	DO M1=-L,L
	DO M=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
	OCP(IU)=OCS(IU)
	ENDDO
	ENDDO
	ENDIF
C  WRITE REAL/IMAG PARTS SEPARATELY
	DO I=1,IRE                             !!! over re/im
	IF(L.EQ.0)
     &WRITE(ICHAN,10110)(TCUBC(M,L,ICUBC),M=-L,L),
     &TRE(I),TSP(IUPDN,IORBS)
	IF(L.EQ.1)
     &WRITE(ICHAN,10120)(TCUBC(M,L,ICUBC),M=-L,L),
     &TRE(I),TSP(IUPDN,IORBS)
	IF(L.EQ.2)
     &WRITE(ICHAN,10130)(TCUBC(M,L,ICUBC),M=-L,L),
     &TRE(I),TSP(IUPDN,IORBS)
	IF(L.EQ.3)
     &WRITE(ICHAN,10140)(TCUBC(M,L,ICUBC),M=-L,L),
     &TRE(I),TSP(IUPDN,IORBS)
	DO M1=-L,L
	DO M=-L,L
      IU=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M)+L+M1+1     !  (l,m' m) packed
      IF(I.EQ.1)TPAR=DREAL(OCP(IU))
      IF(I.EQ.2)TPAR=DIMAG(OCP(IU))
      IF(ABS(TPAR).GT.1.D-07)TPR(M1,M)=TDP(TPAR,1.D-07)
      IF(ABS(TPAR).LE.1.D-07)TPR(M1,M)=' 0.0000000'
	ENDDO
	ENDDO
	DO M1=-L,L
	IF(L.EQ.0)WRITE(ICHAN,10150)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.1)WRITE(ICHAN,10160)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.2)WRITE(ICHAN,10170)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	IF(L.EQ.3)WRITE(ICHAN,10180)(TPR(M1,M),M=-L,L),TCUBC(M1,L,ICUBC)
	ENDDO
	ENDDO                                  !!! over re/im
	ENDDO                                  !!! over ispin
	ENDDO                                  !!! over iorbs
	ENDDO                                  !!! over l
	ENDDO                                  !!! over iatom
	ENDDO                                  !!! over Matsubara frequencies
C MEMORY DEALLOCATIONS
 1000 DEALLOCATE(FHUB,OCR,OCP,OCS,ULR)
C FORMATS
10100 FORMAT(1X,'cState=',A10,' ; wMse = ',G14.7)
10110 FORMAT(1X,1(A10,2X),2X,A4,',',A5)
10120 FORMAT(1X,3(A10,2X),2X,A4,',',A5)
10130 FORMAT(1X,5(A10,2X),2X,A4,',',A5)
10140 FORMAT(1X,7(A10,2X),2X,A4,',',A5)
10150 FORMAT(1X,1(A10,2X),2X,A10)
10160 FORMAT(1X,3(A10,2X),2X,A10)
10170 FORMAT(1X,5(A10,2X),2X,A10)
10180 FORMAT(1X,7(A10,2X),2X,A10)
	END

