	SUBROUTINE MAKESMT 
**********************************************************
* Tries to find optimal MT spheres by tracing the maxima *
* in superposed free atom Hartree potential.             *
**********************************************************
C SET ARRAY SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
	USE COM_CONTROL ,ONLY: IASA                             !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,IS,NHSORT            !!! from readini
      USE COM_SPHERE  ,ONLY: ZNUC,SMT,SMTS,SASA,OVRMAX,SWZ    !!! from readini
	USE COM_LATPAR  ,ONLY: PAR,OMEGAC                       !!! from readini
	USE COM_LATTICE ,ONLY: RBAS,TAU                         !!! from readstr
	USE COM_LIBMESH ,ONLY: NLIB,HLIB,RLIB,DRLIB             !!! from libmesh
      USE COM_SITEGEN ,ONLY: NSITES,SITES,IREFSITES           !!! from sitegen
      USE COM_LABELS  ,ONLY: TXTAT,TXTEL,ELEMENT,NAMEDENS     !!! from readini
      USE COM_LMTINP  ,ONLY: SMTMADE                          !!! from readini
	USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from readini
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
	INTEGER, ALLOCATABLE :: ISIT(:)
	REAL*8, ALLOCATABLE :: RHOATOM(:,:)
	REAL*8, ALLOCATABLE :: POTATOM(:,:)
	REAL*8, ALLOCATABLE :: POTLINE(:,:)
	REAL*8, ALLOCATABLE :: POTSP(:,:,:)
	REAL*8, ALLOCATABLE :: TAUT(:,:)
	REAL*8, ALLOCATABLE :: CSIT(:,:)
	REAL*8, ALLOCATABLE :: WORK1(:)
	REAL*8, ALLOCATABLE :: WORK2(:)
	REAL*8, ALLOCATABLE :: SFND(:)
	REAL*8, ALLOCATABLE :: SBLW(:)
	REAL*8, ALLOCATABLE :: SSET(:)
	REAL*8, ALLOCATABLE :: SOVR(:)
	CHARACTER FILE*130,TDP*10,SPR*10
	DIMENSION CENT(3)
      LOGICAL EXIST
      DATA PI/3.141592653589793D0/
C BUILT LIBERMAN MESH
	CALL LIBMESH
      WRITE(IUN,1010)
1010  FORMAT(1X,'Start finding optimal MT spheres:'/)
C ****************************************************
C MEMORY ALLOCATIONS FOR SCRATCH ARRAYS
	ALLOCATE(RHOATOM(0:NLIBMEM,NSORTMEM))
	ALLOCATE(POTATOM(0:NLIBMEM,NSORTMEM))
	ALLOCATE(POTSP(0:NLIBMEM,3,NSORTMEM))
	ALLOCATE(WORK1(0:NLIBMEM))
	ALLOCATE(WORK2(0:NLIBMEM))
C READ ATOMIC DENSITIES
	DO 1000 ISORT=1,NSORT                   !!! over atoms
      NUCZ=NINT(ZNUC(ISORT))
      CALL FIND_LBLANK(NAMEDENS,130,NBL)
      FILE=NAMEDENS(1:NBL)//ELEMENT(NUCZ)
      INQUIRE(FILE=FILE,EXIST=EXIST)
      IF(.NOT.EXIST)CALL MAKESMT_MESG01
	OPEN(UNIT=1,FILE=FILE,STATUS='OLD',FORM='FORMATTED')
      READ(1,*)
      READ(1,*)JC,JM,NLIBC
	IF(NLIBC.NE.NLIB)CALL MAKESMT_MESG03
      READ(1,*)(RHOATOM(IRAD,ISORT),IRAD=1,NLIB)
	CLOSE(1)
C INTERPOLATE TO THE NUCLEUS
      RHOATOM(0,ISORT)=RHOATOM(1,ISORT)
C CHECK CHARGE
      DO IRAD=0,NLIB
      WORK1(IRAD)=RHOATOM(IRAD,ISORT)*4.D0*PI*RLIB(IRAD,ISORT)**2*
     &DRLIB(IRAD,ISORT)
	ENDDO
      CALL QD(HLIB(ISORT),WORK1,WORK2,NLIB)
      CHARGE=WORK2(NLIB-1)
C PRINT CHARGE
      CALL FIND_LBLANK(FILE,130,NBL)
      WRITE(IUN,1020)ZNUC(ISORT),CHARGE,TXTEL(ISORT),FILE(1:NBL)
1020  FORMAT(1X,'Charge checking: ',G12.5,1X,G12.5,1X,A10,';',A)
C BUILT HARTREE POTENTIAL
      DO IRAD=0,NLIB
      WORK1(IRAD)=RHOATOM(IRAD,ISORT)*4.D0*PI*RLIB(IRAD,ISORT)**2*
     &DRLIB(IRAD,ISORT)
	ENDDO
      CALL QD(HLIB(ISORT),WORK1,WORK2,NLIB)
      POTATOM(0,ISORT)=0.D0
      DO IRAD=1,NLIB-1
      POTATOM(IRAD,ISORT)=2.D0*WORK2(IRAD)/RLIB(IRAD,ISORT)
      ENDDO
      WORK1(0)=0.D0
      DO IRAD=1,NLIB
      WORK1(IRAD)=RHOATOM(IRAD,ISORT)*4.D0*PI*RLIB(IRAD,ISORT)*
     &DRLIB(IRAD,ISORT)
      ENDDO
      CALL QD(HLIB(ISORT),WORK1,WORK2,NLIB)
      DO IRAD=1,NLIB-1
      POTATOM(IRAD,ISORT)=POTATOM(IRAD,ISORT)+
     &2.D0*(WORK2(NLIB-1)-WORK2(IRAD))
      ENDDO
C ADD NUCLEAR PART
	DO IRAD=1,NLIB
	POTATOM(IRAD,ISORT)=POTATOM(IRAD,ISORT)-
     &  2.D0*ZNUC(ISORT)/RLIB(IRAD,ISORT)
	ENDDO
	POTATOM(0,ISORT)=POTATOM(1,ISORT)
      POTATOM(NLIB,ISORT)=POTATOM(NLIB-1,ISORT)
C BUILT SPLINE INTERPOLATION
      CALL SPLKFF(NLIB+1,RLIB(0,ISORT),POTATOM(0,ISORT),
     &POTSP(0,1,ISORT),POTSP(0,2,ISORT),POTSP(0,3,ISORT))
1000	CONTINUE                               !!! over atoms
C FIND NEAREST NEIGHBORS
	ALLOCATE(TAUT(3,NATOMMEM))
	ALLOCATE(CSIT(3,NATOMMEM))
	ALLOCATE(ISIT(NATOMMEM))
	DO 3000 IATOM=1,NATOM
	ISORT=IS(IATOM)
      DO JATOM=1,NATOM
      TAUT(1,JATOM)=TAU(1,JATOM)-TAU(1,IATOM)
      TAUT(2,JATOM)=TAU(2,JATOM)-TAU(2,IATOM)
      TAUT(3,JATOM)=TAU(3,JATOM)-TAU(3,IATOM)
      ENDDO
	RMAX=10.D0/PAR  ! Set optimal cluster site.
2000  CALL SITEGEN(RMAX,RBAS,TAUT,NATOM)
    	NRST=NSITES+1
	DIST=RMAX+1.D0
      DO ISITES=0,NSITES                     !!! over atoms
      ABST=SQRT(SITES(1,ISITES)**2+
     &          SITES(2,ISITES)**2+
     &          SITES(3,ISITES)**2)
	IF(ABST.LT.DIST.AND.ABST.GT.1.D-08)THEN
C Do not consider empty spheres as nearest.
	IWHO=IREFSITES(ISITES)
	IF(NINT(ZNUC(IS(IWHO))).NE.0)THEN
	NRST=ISITES
	DIST=ABST
	ENDIF 
	ENDIF
	ENDDO
	IF(NRST.EQ.NSITES+1)CALL MAKESMT_MESG02
	IF(NRST.EQ.NSITES+1)GOTO2000
	ISIT(IATOM)=IREFSITES(NRST)
	CSIT(1,IATOM)=SITES(1,NRST)
	CSIT(2,IATOM)=SITES(2,NRST)
	CSIT(3,IATOM)=SITES(3,NRST)
	DEALLOCATE(SITES)
	DEALLOCATE(IREFSITES)
3000	CONTINUE                               !!! over atoms
C FIND POTENTIAL BETWEEN NEAREST NEIGHBORS
	NLIN=100
	ALLOCATE(POTLINE(0:NLIN,NATOMMEM))
	DO 4000 IATOM=1,NATOM                  !!! over atoms
	CENT(1)=CSIT(1,IATOM)/2.D0+TAU(1,IATOM)
	CENT(2)=CSIT(2,IATOM)/2.D0+TAU(2,IATOM)
	CENT(3)=CSIT(3,IATOM)/2.D0+TAU(3,IATOM)
	DIST=SQRT(CSIT(1,IATOM)**2+CSIT(2,IATOM)**2+CSIT(3,IATOM)**2)
	RMAX=3.D0*DIST
	RMAX=MAX(RMAX,10.D0/PAR)
      DO JATOM=1,NATOM
      TAUT(1,JATOM)=TAU(1,JATOM)-CENT(1)
      TAUT(2,JATOM)=TAU(2,JATOM)-CENT(2)
      TAUT(3,JATOM)=TAU(3,JATOM)-CENT(3)
      ENDDO
      CALL SITEGEN(RMAX,RBAS,TAUT,NATOM)
C Do over points along the line
	STEP=1.D0/DBLE(NLIN)
	DO I=0,NLIN                            !!! over lines
	RX=TAU(1,IATOM)+DBLE(I)*STEP*CSIT(1,IATOM)
	RY=TAU(2,IATOM)+DBLE(I)*STEP*CSIT(2,IATOM)
	RZ=TAU(3,IATOM)+DBLE(I)*STEP*CSIT(3,IATOM)
	DX1=RX-TAU(1,IATOM)
	DY1=RY-TAU(2,IATOM)
	DZ1=RZ-TAU(3,IATOM)
	DX2=RX-TAU(1,IATOM)-CSIT(1,IATOM)
	DY2=RY-TAU(2,IATOM)-CSIT(2,IATOM)
	DZ2=RZ-TAU(3,IATOM)-CSIT(3,IATOM)
	DS1=PAR*SQRT(DX1*DX1+DY1*DY1+DZ1*DZ1)  !!!
	DS2=PAR*SQRT(DX2*DX2+DY2*DY2+DZ2*DZ2)
C Compute Hartree potential for the point Rx,Ry,Rz
	POTLINE(I,IATOM)=0.D0
	DO ISITES=0,NSITES                     !!! over nearest neighbors
	JATOM=IREFSITES(ISITES)
	JSORT=IS(JATOM)
	TX=RX-CENT(1)-SITES(1,ISITES)
	TY=RY-CENT(2)-SITES(2,ISITES)
	TZ=RZ-CENT(3)-SITES(3,ISITES)
C Tx,Ty,Tz is a vector from the site to R.
	DT=PAR*SQRT(TX**2+TY**2+TZ**2)
C Find the potential at the point DT
      POTT=SPLPNT(NLIB+1,DT,RLIB(0,JSORT),POTATOM(0,JSORT),
     &POTSP(0,1,JSORT),POTSP(0,2,JSORT),POTSP(0,3,JSORT))
	POTLINE(I,IATOM)=POTLINE(I,IATOM)+POTT
	ENDDO                                  !!! over nearest neighbors
	ENDDO                                  !!! over line points
	DEALLOCATE(SITES)
	DEALLOCATE(IREFSITES)
4000	CONTINUE                               !!! over atoms
C POTENTIALS ALONG SHORTEST DISTANCES FOUND, SET MT SPHERES
	ALLOCATE(SFND(NATOMMEM))
	DO IATOM=1,NATOM
	SFND(IATOM)=1.D+15
	ENDDO
 	DO 5000 IATOM=1,NATOM                  !!! over atoms
	JATOM=ISIT(IATOM)
	JSORT=IS(JATOM)
	STEP=1.D0/DBLE(NLIN)
	POTMAX=-1.D+15
	DO I=0,NLIN
	RX=TAU(1,IATOM)+DBLE(I)*STEP*CSIT(1,IATOM)
	RY=TAU(2,IATOM)+DBLE(I)*STEP*CSIT(2,IATOM)
	RZ=TAU(3,IATOM)+DBLE(I)*STEP*CSIT(3,IATOM)
	DX1=RX-TAU(1,IATOM)
	DY1=RY-TAU(2,IATOM)
	DZ1=RZ-TAU(3,IATOM)
	DX2=RX-TAU(1,IATOM)-CSIT(1,IATOM)
	DY2=RY-TAU(2,IATOM)-CSIT(2,IATOM)
	DZ2=RZ-TAU(3,IATOM)-CSIT(3,IATOM)
	DS1=PAR*SQRT(DX1*DX1+DY1*DY1+DZ1*DZ1)  !!!
	DS2=PAR*SQRT(DX2*DX2+DY2*DY2+DZ2*DZ2)
	IF(POTMAX.LT.POTLINE(I,IATOM))S1=DS1
	IF(POTMAX.LT.POTLINE(I,IATOM))S2=DS2
	IF(POTMAX.LT.POTLINE(I,IATOM))POTMAX=POTLINE(I,IATOM)
	ENDDO
	IF(S1.LT.SFND(IATOM))SFND(IATOM)=S1
	IF(S2.LT.SFND(JATOM))SFND(JATOM)=S2
5000	CONTINUE                               !!! over atoms
C SPHERES FOR EMPTY SITES ARE PROBABLY ZEROES
	DO IATOM=1,NATOM
	IF(SFND(IATOM).LT.0.01D0)SFND(IATOM)=0.01D0
	ENDDO
C BLOW SPHERES UNTIL THEY TOUCH
	ALLOCATE(SBLW(NATOMMEM))
      CALL BLOWMTS(SFND,SBLW,RBAS(1,1),RBAS(1,2),RBAS(1,3),TAU,NATOM)
	DO IATOM=1,NATOM
	SBLW(IATOM)=SBLW(IATOM)*PAR
	ENDDO
C NON-OVERLAPPED SPHERES ARE READY NOW
	ALLOCATE(SSET(NSORTMEM))
	DO ISORT=1,NSORT
	SSET(ISORT)=0.D0
	ENDDO
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
	SSET(ISORT)=SSET(ISORT)+SBLW(IATOM)/DBLE(NHSORT(ISORT))
	ENDDO
C LEAVE THREE DIGITS ONLY
	DO ISORT=1,NSORT
	SPR(1:10)=TDP(SSET(ISORT),1.D-05)
	SPR(7:10)='D0  '
	READ(SPR(1:10),*)SSET(ISORT)
	ENDDO
C TAKE INTO ACCOUNT INPUT
	DO ISORT=1,NSORT
      IF(ABS(SMTS(ISORT)).GT.1.D-10)SSET(ISORT)=SMTS(ISORT)
	ENDDO 
C PRINT NON-OVERLAPPED SPHERES
	SMTMADE=.FALSE.
	DO IATOM=1,NATOM
      ISORT=IS(IATOM)
      WRITE(IUN,1110)(TAU(I,IATOM),I=1,3),TXTEL(IS(IATOM))
      IF(ABS(SMTS(ISORT)).GT.1.D-10)WRITE(IUN,1120)SMTS(ISORT)
      IF(ABS(SMTS(ISORT)).LE.1.D-10)WRITE(IUN,1130)
      IF(ABS(SMTS(ISORT)).LE.1.D-10)SMTMADE=.TRUE.
      IF(ABS(SMTS(ISORT)).LE.1.D-10)SMTS(ISORT)=SSET(ISORT)
	WRITE(IUN,1140)SFND(IATOM)
	WRITE(IUN,1150)SBLW(IATOM)
      WRITE(IUN,1160)SMTS(ISORT)
	ENDDO
C NON-OVERLAPPING SPHERES ARE READY NOW.
C ************************************************************ 
C MAKE SPHERES WHICH CAN OVERLAP TO MAXIMUM OVERLAP=OVRMAX
C MAKE SPACE FILLING (CLASSICAL ATOMIC) SPHERES
	ALLOCATE(SOVR(NSORTMEM))
	OMEGAM=0.D0
	DO IATOM=1,NATOM
	ISORT=IS(IATOM)
      OMEGAM=OMEGAM+4.D0/3.D0*PI*SMTS(ISORT)**3
	ENDDO
	XFIL=(OMEGAC/OMEGAM)**0.33333333333333D0 
	IF(XFIL.GT.OVRMAX)XCUR=OVRMAX
	IF(XFIL.LE.OVRMAX)XCUR=XFIL
	DO ISORT=1,NSORT
      SOVR(ISORT)=SMTS(ISORT)*XCUR
      SASA(ISORT)=SMTS(ISORT)*XFIL
	ENDDO
C DEFINE WIGNER-ZEITS SPHERE
      SWZ=(3.D0*OMEGAC/DBLE(NATOM)/4.D0/PI)**0.3333333333333D0
C PRINT OVERLAP
	WRITE(IUN,1180)OVRMAX
	WRITE(IUN,1190)XFIL
	WRITE(IUN,1200)XCUR
      IF(IASA.EQ.1)WRITE(IUN,1210)
C ******************************************************************
C SET AUGMENTATION SPHERES NOW:        
	DO ISORT=1,NSORT
      IF(IASA.EQ.1)SMT(ISORT)=SOVR(ISORT)
      IF(IASA.NE.1)SMT(ISORT)=SMTS(ISORT)
	ENDDO
C CHECK THEM
	DO ISORT=1,NSORT
	IF(SMT(ISORT).LE.0.5D0.AND.SMTMADE)CALL MAKESMT_MESG04
	ENDDO 
1110  FORMAT(1X,'Position : (',G12.5,',',G12.5,',',G12.5,') ',A10)
1120	FORMAT(1X,'MT-sphere read from input  INIFILE = ',G14.7)
1130	FORMAT(1X,'MT-sphere from INIFILE is not known! See below:')
1140  FORMAT(1X,'MT-sphere from  Hartree  potential = ',G14.7)
1150  FORMAT(1X,'MT-sphere blowed   until  touching = ',G14.7)
1160  FORMAT(1X,'MT-sphere is now set to the  value = ',G14.7)
1180	FORMAT(/1X,'Maximum overlap allowed from setup = ',G14.7)
1190	FORMAT(1X,'Overlap required  to  fill  volume = ',G14.7)
1200	FORMAT(1X,'Overlap finally set  to  the value = ',G14.7)
1210	FORMAT(1X,'All MT-spheres are reset to overlapped spheres!'/)
C MEMORY DEALLOCATIONS
	DEALLOCATE(RHOATOM)
	DEALLOCATE(POTATOM)
	DEALLOCATE(POTLINE)
	DEALLOCATE(WORK1)
	DEALLOCATE(WORK2)
	DEALLOCATE(CSIT)
	DEALLOCATE(ISIT)
	DEALLOCATE(TAUT)
	DEALLOCATE(SFND)
	DEALLOCATE(SBLW)
	DEALLOCATE(SSET)
	DEALLOCATE(SOVR)
C ERROR HANDLINGS
	CONTAINS
	SUBROUTINE MAKESMT_MESG01
	WRITE(IUT,*)'Terminated in <MAKESMT>:'
	WRITE(IUT,*)'Atomic density file ',FILE
	WRITE(IUT,*)'cannot be found. Check path to the atomic data'
	WRITE(IUT,*)'library specified in <SETUP> subroutine.'
	CALL FULLSTOP(' ')
	END SUBROUTINE MAKESMT_MESG01
	SUBROUTINE MAKESMT_MESG02
	WRITE(IUT,*)'Warning from <MAKESMT>:'
	WRITE(IUT,*)'Cluster size is too small. Trying to increase...'
	WRITE(IUT,*)'New Rmax =',1.3D0*RMAX*PAR
	RMAX=1.3D0*RMAX
	IF(RMAX*PAR.GT.30.D0)
     &CALL FULLSTOP('Cannot find nearest neighbor.')
	END SUBROUTINE MAKESMT_MESG02
	SUBROUTINE MAKESMT_MESG03
	WRITE(IUT,*)'Terminated in <MAKESMT>:'
	WRITE(IUT,*)'# of radial points in atomic density file ',FILE
	WRITE(IUT,*)'does not coincide with array declaration.' 
	WRITE(IUT,*)'Declared # =',NLIB,'; # read =',NLIBC
	CALL FULLSTOP(' ')
	END SUBROUTINE MAKESMT_MESG03
	SUBROUTINE MAKESMT_MESG04
	WRITE(IUT,*)'Terminated in <MAKESMT>:'
	WRITE(IUT,*)
     &'MT-sphere is found to be too small (less than 0.5 a.u)'
	WRITE(IUT,*)SMT(ISORT),TXTEL(ISORT)
	WRITE(IUT,*)'If this is an empty sphere, try to remove it.'
	WRITE(IUT,*)
     &'In another case, the algorithm for finding MT-spheres'
	WRITE(IUT,*)'seems to do not work properly in this situation.'
	CALL FULLSTOP(' ')
	END SUBROUTINE MAKESMT_MESG04
	END SUBROUTINE MAKESMT
