      subroutine padecoef( a,z,g,n )
c|-----------------------------------------------------------------------|
c|                                                                       |
c|               Computation of Pade coefficients.                       |
c|                                                                       |
c|    Inputs:                                                            |
cio    a  -  On input is a function for approximation, on output         |
c|           contains Pade coefficients.                                 |
ci     z  -  complex points in which function a is determined.           |
ci     n  -  size of arrays.                                             |
c|    Works:                                                             |
cw     g  -  work array.                                                 |
cr     Remarks:                                                          |
c|           (J. of Low Temp. Phys., v29, n3/4, 1977)                    |     
c|                                                                       |
c|-----------------------------------------------------------------------|

      implicit none
c--->  Passed variables
      integer n
      complex*16 a(n),z(n),g(n,n)
c--->  Local variables 
      integer i,j
       
      do i = 1,n
	g(1,i) = a(i)
      enddo

      do j = 2,n
	do i = 2,j
	  g(i,j) = ( g(i-1,i-1)-g(i-1,j) )/( z(j)-z(i-1) )/g(i-1,j)
	enddo
c       write(6,*)j,(i,g(i,j),i=1,n)
      enddo
      do i = 1,n
        a(i) = g(i,i)
      enddo
      
      end

       subroutine pade( f,z1,z,p,n )
c|----------------------------------------------------------------------|
c|                                                                      |
c|            Calculation of analitically continued function            |
c|    for a given Pade coefficients in the arbitrary complex point.     |
c|                                                                      |
c|    Inputs:                                                           |
ci     p  -  Pade coefficients.                                         |
ci     z  -  set of points from which analytical continue is performed. |
ci     n  -  size of arrays.                                            |
ci     z1 -  complex point.                                             |
c|    Outputs:                                                          |
ci     f  -  value of function.                                         |
c|                                                                      |
c|----------------------------------------------------------------------|

      implicit none
c---> Passed variables
       integer n
       complex*16 z(n),p(n)
       real*8 sum
       complex*16 z1,f
c---> Local variables
       integer i,np
       real*8 cof1,cof2
       complex*16 a1,a2,anew,b1,b2,bnew
c---> Intrinsic
       intrinsic cdabs,dnint,dlog10,idint
 
      sum = 0.d0
      do i = 1,n-1
        sum = sum + cdabs( (z1-z(i))*p(i+1) )
      end do  
      if( sum.ge.1.d0 )then
        sum = dnint(n*sum/(n-1)) 
        sum = dlog10(sum)
        np  = idint(sum) + 1
        cof1 = 1.d-250
        cof2 = 1.d0
        if( np.gt.275 ) cof2 = 1.d-275
      else  
        sum = dnint((n-1)/(n*sum)) 
        sum = dlog10(sum)
        np  = -idint(sum) - 1
        cof1 = 1.d+250
        cof2 = 1.d0
        if( np.lt.-275 ) cof2 = 1.d+275
      endif

      a1 = dcmplx(0.d0,0.d0)
      a2 = cof1*p(1)
      b1 = dcmplx(cof1,0.d0)
      b2 = dcmplx(cof1,0.d0)
      do i = 1,n-1
	anew = a2+(z1-z(i))*p(i+1)*a1
	bnew = b2+(z1-z(i))*p(i+1)*b1
        a1 = a2
        b1 = b2
        a2 = anew
        b2 = bnew
      end do
      a2 = cof2*a2
      b2 = cof2*b2
      f = a2/b2

      end
