	MODULE COM_RADMESH
***************************************************************
*  Containes radial mesh                                      *
***************************************************************
	INTEGER, SAVE, ALLOCATABLE :: NRAD(:)         !!! # of points
	REAL*8,  SAVE, ALLOCATABLE :: H(:)            !!! step
	REAL*8,  SAVE, ALLOCATABLE :: R(:,:)          !!! radial mesh r(i)
	REAL*8,  SAVE, ALLOCATABLE :: DR(:,:)         !!! dr(i)
	REAL*8,  SAVE, ALLOCATABLE :: WEIGHT(:,:)     !!! 4pi*r^2*dr
	REAL*8,  SAVE, ALLOCATABLE :: RL(:,:,:)       !!! r^l 
	END MODULE COM_RADMESH



	MODULE COM_LIBMESH
*************************************************************
*  Contains Liberman mesh from atomic calculations.         *
*************************************************************
	INTEGER, SAVE :: NLIB
	REAL*8, SAVE, ALLOCATABLE :: HLIB(:)
	REAL*8, SAVE, ALLOCATABLE :: RLIB(:,:)
	REAL*8, SAVE, ALLOCATABLE :: DRLIB(:,:)
	END MODULE COM_LIBMESH
	


	MODULE COM_POTDAT
*************************************************************
*  Containes potential and pseudopotential.                 *
*************************************************************
	REAL*8, SAVE, ALLOCATABLE :: POTSPH(:,:,:)     !!! spherical potential
	REAL*8, SAVE, ALLOCATABLE :: POTRWF(:,:,:)     !!! frozen potenital
	COMPLEX*16, SAVE, ALLOCATABLE :: POT(:,:,:,:,:)!!! full potential
	COMPLEX*16, SAVE, ALLOCATABLE :: POTPS(:,:,:)  !!! pseudopotential
	COMPLEX*16, SAVE, ALLOCATABLE :: POTPS1(:,:,:) !!! auxilary potential
	END MODULE COM_POTDAT



	MODULE COM_DENDAT
*************************************************************
*  Containes potential and pseudopotential.                 *
*************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: RHO(:,:,:,:,:) !!! full density
	COMPLEX*16, SAVE, ALLOCATABLE :: RHOPS(:,:,:)   !!! pseudodensity
	COMPLEX*16, SAVE, ALLOCATABLE :: RHOPN(:,:,:)   !!! new pseudodensity
	COMPLEX*16, SAVE, ALLOCATABLE :: RHOPS1(:,:,:)  !!! auxilary density
	REAL*8, SAVE, ALLOCATABLE :: RHOCOR(:,:,:)      !!! core density
	REAL*8, SAVE, ALLOCATABLE :: POTFRZ(:,:,:)      !!! frozen potenital
	END MODULE COM_DENDAT


	MODULE COM_MULTRO
*************************************************************
*  Containes multipole data.                                *
*************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: AMTOT(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: PSMOM(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: PSMOM1(:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: QMAD(:,:,:,:)
	END MODULE COM_MULTRO


	MODULE COM_POTMTS
*************************************************************
*  Containes multipole data.                                *
*************************************************************
	REAL*8, SAVE :: VMT0
	REAL*8, SAVE :: VSF0
	REAL*8, SAVE :: VAV0
	REAL*8, SAVE, ALLOCATABLE :: VMT(:,:)
	REAL*8, SAVE, ALLOCATABLE :: VSF(:,:)
	END MODULE COM_POTMTS



	MODULE COM_FERMI
*************************************************************
*  Containes Fermi energy and partial TOS, DOS.             *
*************************************************************
	REAL*8, SAVE :: EF      ! Fermi energy 
	COMPLEX*16, SAVE, ALLOCATABLE :: ATOS(:,:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: ADOS(:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: PTOS(:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: PFOS(:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: PDOS(:,:,:,:,:,:)
	END MODULE COM_FERMI





	MODULE COM_KMESH
**************************************************************
*  Containes k-mesh.                                         *
**************************************************************
	INTEGER, SAVE, ALLOCATABLE :: NPNT(:)       !!! # of k-points
	INTEGER, SAVE, ALLOCATABLE :: IPQ(:,:,:)    !!! k-pointer
	INTEGER, SAVE, ALLOCATABLE :: IGQ(:,:,:)    !!! g-pointer
	INTEGER, SAVE, ALLOCATABLE :: ITQ(:,:,:,:)  !!! t-pointer
	INTEGER, SAVE :: KCUT(3,4,6)                !!! ttr-pointer
	REAL*8, SAVE, ALLOCATABLE :: PNT(:,:,:)     !!! k-point list
	REAL*8, SAVE, ALLOCATABLE :: WGT(:,:)       !!! geometrical weights
	END MODULE COM_KMESH



	MODULE COM_WMESH
**************************************************************
*  Containes k-mesh.                                         *
**************************************************************
	INTEGER, SAVE, ALLOCATABLE :: NPNW(:)       !!! # of k-points
	INTEGER, SAVE, ALLOCATABLE :: IPW(:,:,:)    !!! k-pointer
	INTEGER, SAVE, ALLOCATABLE :: IGW(:,:,:)    !!! g-pointer
	INTEGER, SAVE, ALLOCATABLE :: ITW(:,:,:,:)  !!! t-pointer
	INTEGER, SAVE :: KCUW(3,4,6)                !!! ttr-pointer
	REAL*8, SAVE, ALLOCATABLE :: PNW(:,:,:)     !!! k-point list
	REAL*8, SAVE, ALLOCATABLE :: WGW(:,:)       !!! geometrical weights
	END MODULE COM_WMESH



	MODULE COM_ENERGY
**************************************************************
*  Containes energy bands and integration weights.           *
**************************************************************
	REAL*8, SAVE, ALLOCATABLE :: EARR(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: EARW(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: WEIGH(:,:)
	REAL*8, SAVE, ALLOCATABLE :: WEIGD(:,:)
	END MODULE COM_ENERGY



	MODULE COM_POTPAR
**************************************************************
*  Containes matrix elements between radial wave functions.  *
**************************************************************
	REAL*8, SAVE, ALLOCATABLE :: FF(:,:,:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: FFP(:,:,:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: FPF(:,:,:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: FPFP(:,:,:,:,:)
	END MODULE COM_POTPAR


	MODULE COM_LMTPAR
**************************************************************
*  Containes potential parameters for radial wave functions. *
**************************************************************
	REAL*8, SAVE, ALLOCATABLE :: GAM(:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: AAH(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: BBH(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: AAJ(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: BBJ(:,:,:,:,:)
	END MODULE COM_LMTPAR

	MODULE COM_ASAPAR
**************************************************************
*  Containes Hankels and Bessels at the MT spheres.          *
**************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: HS(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: BS(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: DH(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: DJ(:,:,:,:,:)
	END MODULE COM_ASAPAR


	MODULE COM_KKRPAR
**************************************************************
*  Containes KKR centers of bands and bandwidths.            *
**************************************************************
	REAL*8, SAVE, ALLOCATABLE :: CNY(:,:,:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: WNY(:,:,:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: VNY(:,:,:,:,:)
	END MODULE COM_KKRPAR


	MODULE COM_OVRPAR
**************************************************************
*  Containes overlap potential parameters.                   *
**************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: OPARHH(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: OPARHJ(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: OPARJH(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: OPARJJ(:,:,:,:,:)
	END MODULE COM_OVRPAR

	MODULE COM_MTSPAR
**************************************************************
*  Containes MT-sphere potential parameters.                 *
**************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: OMTHH(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: OMTJH(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: OMTHJ(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: OMTJJ(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: HMTHH(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: HMTJH(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: HMTHJ(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: HMTJJ(:,:,:,:,:)
	END MODULE COM_MTSPAR


	MODULE COM_SUMPAR
**************************************************************
*  Containes interstitial potential parameters.              *
**************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: OHH(:,:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: OJH(:,:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: OHJ(:,:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: OJJ(:,:,:,:,:,:,:)
	END MODULE COM_SUMPAR



	MODULE COM_INTPAR
**************************************************************
*  Containes interstitial potential parameters.              *
**************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: WHH(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: WJH(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: WHJ(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: WJJ(:,:,:,:)
	END MODULE COM_INTPAR


	MODULE COM_ORBPAR
**************************************************************
*  Containes interstitial potential parameters.              *
**************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: SPARHH(:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: SPARHJ(:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: SPARJH(:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: SPARJJ(:,:,:,:,:,:)
	END MODULE COM_ORBPAR



	MODULE COM_LMTIND
**************************************************************
*  Containes pointer to active basis functions.              *
**************************************************************
	INTEGER, SAVE, ALLOCATABLE :: LMTIND(:,:,:,:,:)
	END MODULE COM_LMTIND


	MODULE COM_PULAYF
**************************************************************
*  Containes Pulay forces and error estimates.               *
**************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: PFVAL(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: PILF(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: RESRHO(:,:,:)
	END MODULE COM_PULAYF



	MODULE COM_FORCES
**************************************************************
*  Contains different forces                                 *
**************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: FPTN(:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: FCOU(:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: FEXC(:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: FHLF(:,:,:)
	END MODULE COM_FORCES



	MODULE COM_SYMMLM
**************************************************************
*  Containes pointer to non-zero spherical harmonics.        *
**************************************************************
	INTEGER, SAVE, ALLOCATABLE :: NSYM(:)    !!! # of non-zero elements
	INTEGER, SAVE, ALLOCATABLE :: LMSYM(:,:) !!! lm-pointer       
	LOGICAL, SAVE, ALLOCATABLE :: SYM(:,:)   !!! lm-pointer
	END MODULE COM_SYMMLM



	MODULE COM_GAUNT
**************************************************************
*  Containes Gaunt coefficients.                             *
**************************************************************
      REAL*8, SAVE, ALLOCATABLE :: C(:,:,:)     !!! Gaunts array
      REAL*8, SAVE, ALLOCATABLE :: GGNT(:,:,:)  !!! g-factors array
	END MODULE COM_GAUNT




	MODULE COM_SCRINP
**************************************************************
* Containes screening parameters ALPHA and Rloc's.           *
**************************************************************
	REAL*8, SAVE, ALLOCATABLE :: RLOC(:)            !!! cluster size
	COMPLEX*16, SAVE, ALLOCATABLE :: ALPHA(:,:,:,:) !!! screening constants
	COMPLEX*16, SAVE, ALLOCATABLE :: ALPDOT(:,:,:,:)!!! energy derivatives
	COMPLEX*16, SAVE, ALLOCATABLE :: DTBLOG(:,:,:,:)!!! logarithmic derivs
	END MODULE COM_SCRINP



	MODULE COM_ENVSITE
**************************************************************
* Containes nearest-neighbor information for every site.     *
**************************************************************
	INTEGER, SAVE, ALLOCATABLE :: NENV(:)
	INTEGER, SAVE, ALLOCATABLE :: ISENV(:,:)
	INTEGER, SAVE, ALLOCATABLE :: IPENV(:,:,:)
	INTEGER, SAVE, ALLOCATABLE :: IRENV(:,:)
	INTEGER, SAVE, ALLOCATABLE :: ICENV(:,:,:)
	INTEGER, SAVE, ALLOCATABLE :: NSHL(:)
	INTEGER, SAVE, ALLOCATABLE :: NATS(:,:)
	INTEGER, SAVE, ALLOCATABLE :: ISSHL(:,:,:)
	REAL*8,  SAVE, ALLOCATABLE :: TENV(:,:,:)
	END MODULE COM_ENVSITE




	MODULE COM_GROUPS
*************************************************************
* Containes point-group information.                        *
*************************************************************
	INTEGER, SAVE :: NGROUP                     ! # of group oprations
	INTEGER, SAVE :: INV                        ! inversion operation
	INTEGER, SAVE, ALLOCATABLE :: KELTAB(:,:)   ! Keli table
	INTEGER, SAVE, ALLOCATABLE :: IKOV(:)       ! operation pointer
	INTEGER, SAVE, ALLOCATABLE :: IP(:,:)       ! atom pointer
	REAL*8, SAVE, ALLOCATABLE :: RDC(:,:,:)     ! Atomic phase shift
	REAL*8, SAVE, ALLOCATABLE :: SHIFT(:,:)     ! Non-symmorphic shift
	COMPLEX*16, SAVE, ALLOCATABLE :: SOPHS(:,:) ! Spin phase shift
	COMPLEX*16, SAVE, ALLOCATABLE :: SJ(:,:,:)  ! Wigner matrices 
	COMPLEX*16, SAVE, ALLOCATABLE :: U(:,:)     ! Wigner matrices 
	COMPLEX*16, SAVE :: UJ(2,2,48)              ! Wigner matrices
	END MODULE COM_GROUPS


	MODULE COM_PLWMESH
*************************************************************
* Containes plane-wave data.                                *
*************************************************************
	INTEGER, SAVE :: NPLW                    ! # of plane waves
	INTEGER, SAVE :: KGMIN(3)                ! FFT grid
	INTEGER, SAVE :: KGMAX(3)                ! FFT grid
	INTEGER, SAVE, ALLOCATABLE :: KVEC(:,:)  ! plane waves
	INTEGER, SAVE, ALLOCATABLE :: INDG(:)    ! pointer
	REAL*8,  SAVE, ALLOCATABLE :: GPLW(:,:)  ! plane waves
	END MODULE COM_PLWMESH


	MODULE COM_PLW2MSH
*************************************************************
* Containes plane-wave data.                                *
*************************************************************
	INTEGER, SAVE :: N2PLW                   ! # of plane waves
	INTEGER, SAVE, ALLOCATABLE :: INDG2(:)   ! pointer
	REAL*8,  SAVE, ALLOCATABLE :: G2PLW(:,:) ! plane waves
	COMPLEX*16, SAVE, ALLOCATABLE :: GINT(:) ! integrals
	END MODULE COM_PLW2MSH


	MODULE COM_FFTMESH
*************************************************************
* Containes fast Fourier transform data.                    *
*************************************************************
	INTEGER, SAVE :: NFFTG(3)
	INTEGER, SAVE, ALLOCATABLE :: INTFFT(:)
	REAL*8, SAVE :: AD(3,3)
	REAL*8, SAVE :: RD(3,3)
	END MODULE COM_FFTMESH


	MODULE COM_TEILORG
*************************************************************
* Containes accelerations of plane-wave expansions          *
*************************************************************
	INTEGER, SAVE :: KEYT
	INTEGER, SAVE :: MBAS
	REAL*8, SAVE :: BZM 
	REAL*8, SAVE :: GOPT
	END MODULE COM_TEILORG



	MODULE COM_ETAPAR
*************************************************************
* Containes pseudoHankel data.                              *
*************************************************************
	INTEGER, SAVE :: NBAS
      INTEGER, SAVE, ALLOCATABLE :: LBAS(:,:,:,:)
	REAL*8,  SAVE, ALLOCATABLE :: ETA(:,:,:,:)
	REAL*8,  SAVE, ALLOCATABLE :: CUT(:,:,:,:)
	END MODULE COM_ETAPAR


	MODULE COM_ACCURG
*************************************************************
* Containes accuracies for pseudoHankels                    *
*************************************************************
	REAL*8, SAVE :: EPSR
	REAL*8, SAVE :: EPSG
	REAL*8, SAVE :: ECUTG
	REAL*8, SAVE, ALLOCATABLE :: BET(:,:)
	END MODULE COM_ACCURG



	MODULE COM_TBBPAR
*************************************************************
* Containes pointer to tight-binding orbitals.              *
*************************************************************
	INTEGER, SAVE :: NTBAS
	INTEGER, SAVE, ALLOCATABLE :: INBAS(:,:)
	INTEGER, SAVE, ALLOCATABLE :: MQBAS(:,:)
	LOGICAL, SAVE, ALLOCATABLE :: LTBAS(:,:)
	LOGICAL, SAVE, ALLOCATABLE :: LSBAS(:,:)
	END MODULE COM_TBBPAR


      MODULE COM_TBBSYS
*************************************************************
* Containes local coordinate systems for tb-basis           *
*************************************************************
      REAL*8, SAVE, ALLOCATABLE :: RINPAXE(:,:,:)
      REAL*8, SAVE, ALLOCATABLE :: ROUTAXE(:,:,:)
      REAL*8, SAVE, ALLOCATABLE :: RINPANG(:,:)
      REAL*8, SAVE, ALLOCATABLE :: ROUTANG(:,:)
      REAL*8, SAVE, ALLOCATABLE :: RINPINV(:,:)
      REAL*8, SAVE, ALLOCATABLE :: ROUTINV(:,:)
	CHARACTER*10, SAVE, ALLOCATABLE :: CHSYS(:,:)
	CHARACTER*80, SAVE, ALLOCATABLE :: CHAXE(:,:)
	CHARACTER*80, SAVE, ALLOCATABLE :: CHANG(:,:)
	CHARACTER*10, SAVE, ALLOCATABLE :: CHINV(:,:)
      END MODULE COM_TBBSYS



	MODULE COM_HOPCOM
*************************************************************
* Containes tight-binding hamiltonian and overlap matrix.   *
*************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: TMAT(:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: SMAT(:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: PMAT(:,:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: DMAT(:,:,:,:,:,:)
	END MODULE COM_HOPCOM



	MODULE COM_HOPDAT
*************************************************************
* Controls hopping-integral calculation.                    *
************************************************************* 
      INTEGER, SAVE :: NINF,NHOP,ISCHM,ICUBC,IRE,IWR,IHO
      CHARACTER*60, SAVE, ALLOCATABLE ::  HINF(:)
      CHARACTER*60, SAVE, ALLOCATABLE ::  HIND(:)
	END MODULE COM_HOPDAT


