	MODULE COM_HUBDAT
*************************************************************
* Controls correlated electrons.                            *
*************************************************************
	INTEGER, SAVE :: NHUB                         !!! # of correlated states
	INTEGER, SAVE :: NCLM                         !!! # of correlated basis functions
	INTEGER, SAVE, ALLOCATABLE :: MCRL(:)         !!! correlated atoms pointer
	INTEGER, SAVE, ALLOCATABLE :: LCRL(:,:)       !!! correlated orbitals pointer
	INTEGER, SAVE, ALLOCATABLE :: KSOLVER(:,:)    !!! impurty solver choice
	INTEGER, SAVE, ALLOCATABLE :: KDBLCNT(:,:)    !!! double counting choice
	INTEGER, SAVE :: ISCHM
	INTEGER, SAVE :: ICUBC
	INTEGER, SAVE :: IRE
	INTEGER, SAVE :: IWR 
	END MODULE COM_HUBDAT



      MODULE COM_HUBSYS
*************************************************************
* Containes local coordinate systems for hubbard occupancies*
*************************************************************
      REAL*8, SAVE, ALLOCATABLE :: RINPAXE(:,:,:)
      REAL*8, SAVE, ALLOCATABLE :: ROUTAXE(:,:,:)
      REAL*8, SAVE, ALLOCATABLE :: RINPANG(:,:)
      REAL*8, SAVE, ALLOCATABLE :: ROUTANG(:,:)
      REAL*8, SAVE, ALLOCATABLE :: RINPINV(:,:)
      REAL*8, SAVE, ALLOCATABLE :: ROUTINV(:,:)
	CHARACTER*10, SAVE, ALLOCATABLE :: CHSYS(:,:)
	CHARACTER*80, SAVE, ALLOCATABLE :: CHAXE(:,:)
	CHARACTER*80, SAVE, ALLOCATABLE :: CHANG(:,:)
	CHARACTER*10, SAVE, ALLOCATABLE :: CHINV(:,:)
      END MODULE COM_HUBSYS




	MODULE COM_FSLATR
*************************************************************
* Containes Slater integrals.                               *
*************************************************************
      REAL*8, SAVE, ALLOCATABLE :: FSLG(:,:,:,:)
      REAL*8, SAVE, ALLOCATABLE :: FSLU(:,:,:,:)
	END MODULE COM_FSLATR




	MODULE COM_MSBMESH
*************************************************************
* Containes frequency mesh data including Matsubara grid
*************************************************************
	INTEGER, SAVE :: NMSB
	INTEGER, SAVE :: NMSI
	INTEGER, SAVE :: NMSL
	INTEGER, SAVE :: NMSE
	REAL*8,  SAVE :: TEMP
	REAL*8,  SAVE :: OBND
	REAL*8,  SAVE :: SMSB
	REAL*8,  SAVE :: SMSI
	REAL*8,  SAVE :: SMSL
	REAL*8,  SAVE :: SMSE
	REAL*8, SAVE, ALLOCATABLE :: WMSB(:)
	REAL*8, SAVE, ALLOCATABLE :: DMSB(:)
	REAL*8, SAVE, ALLOCATABLE :: FMSB(:)
	REAL*8, SAVE, ALLOCATABLE :: WMSI(:)
	REAL*8, SAVE, ALLOCATABLE :: DMSI(:)
	REAL*8, SAVE, ALLOCATABLE :: WMSL(:)
	REAL*8, SAVE, ALLOCATABLE :: DMSL(:)
	REAL*8, SAVE, ALLOCATABLE :: WMSE(:)
	REAL*8, SAVE, ALLOCATABLE :: DMSE(:)
	END MODULE COM_MSBMESH


	MODULE COM_DMFPAR
*************************************************************
* Containes DMFT data
*************************************************************
	INTEGER, SAVE :: NDIT      !!! # of DMFT iterations
	REAL*8,  SAVE :: EPSDMF    !!! accuracy in DMFT loop
	REAL*8,  SAVE :: BRDDMF    !!! bradening
	REAL*8,  SAVE :: ADMIXL    !!! mixing
	LOGICAL, SAVE :: MIXBRD    !!! use broyden
	INTEGER, SAVE :: MIXFUN    !!! 1 for Delta(w), 2 for Sigma(w)
	END MODULE COM_DMFPAR


	MODULE COM_HUBCOM
*************************************************************
* Contains Hubbard parameters for LDA+U.                    *
*************************************************************
      COMPLEX*16, SAVE, ALLOCATABLE :: VHUB(:,:,:,:)
      COMPLEX*16, SAVE, ALLOCATABLE :: DHUB(:,:,:,:)
      COMPLEX*16, SAVE, ALLOCATABLE :: VCNS(:,:,:,:)
      COMPLEX*16, SAVE, ALLOCATABLE :: DLDA(:,:,:,:)
	END MODULE COM_HUBCOM


	MODULE COM_DMFDAT
*************************************************************
* Containes Green function data
*************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: SIG0(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: SIGI(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: SIGW(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: DELW(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: DEL0(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: GINV(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: OIMP(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: EIMP(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: AINV(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: ALAM(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: ELAM(:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: RCON(:,:,:,:)
	END MODULE COM_DMFDAT



	MODULE COM_GRFDAT
*************************************************************
* Containes Green function data
*************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: EPOL(:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: EPOW(:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: GWGT(:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: GWGD(:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: GW0T(:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: GW0D(:,:)
	REAL*8, SAVE, ALLOCATABLE :: GWTS(:,:,:,:)
	END MODULE COM_GRFDAT


	MODULE COM_ZCWDAT
*************************************************************
* Containes data for impurity solver
*************************************************************
	REAL*8, SAVE, ALLOCATABLE :: PSUNIMP(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: XSUNIMP(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: YSUNIMP(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: PCRFIMP(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: XCRFIMP(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: YCRFIMP(:,:,:)
	END MODULE COM_ZCWDAT

	MODULE COM_QMCDAT
*************************************************************
* Containes data for impurity solver
*************************************************************
	INTEGER, SAVE, ALLOCATABLE :: NTAUIMP(:,:)
	INTEGER, SAVE, ALLOCATABLE :: NSWPIMP(:,:)
	END MODULE COM_QMCDAT



	MODULE COM_IMPDAT
*************************************************************
* Containes data for impurity solver
*************************************************************
	INTEGER, SAVE, ALLOCATABLE :: KCRDSYS(:,:)    !!! coordiate system choice
	INTEGER, SAVE, ALLOCATABLE :: NRUNIMP(:,:)    !!! # of runs for impurity solver
	INTEGER, SAVE, ALLOCATABLE :: KSTATES(:,:,:,:)!!! states to include in impurity solver
	INTEGER, SAVE, ALLOCATABLE :: NREPIMP(:,:)
	INTEGER, SAVE, ALLOCATABLE :: NCNFIMP(:,:)
	INTEGER, SAVE, ALLOCATABLE :: KREPIMP(:,:,:)
	INTEGER, SAVE, ALLOCATABLE :: LREPIMP(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: UPARIMP(:,:)
	REAL*8, SAVE, ALLOCATABLE :: GPARIMP(:,:)
	REAL*8, SAVE, ALLOCATABLE :: ZMASIMP(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: TPARIMP(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: ELOCIMP(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: SIG0IMP(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: SIGIIMP(:,:,:)
	END MODULE COM_IMPDAT




	MODULE COM_GZWDAT
*************************************************************
* Containes data for Gutzwiller approximation
*************************************************************
	INTEGER, SAVE, ALLOCATABLE :: NMASGZW(:,:)
	INTEGER, SAVE, ALLOCATABLE :: NLAMGZW(:,:)
	INTEGER, SAVE, ALLOCATABLE :: MZWALK(:,:,:)
	INTEGER, SAVE, ALLOCATABLE :: LZWALK(:,:,:)
	INTEGER, SAVE, ALLOCATABLE :: IZWALK(:,:,:)
	INTEGER, SAVE, ALLOCATABLE :: MEWALK(:,:,:)
	INTEGER, SAVE, ALLOCATABLE :: LEWALK(:,:,:)
	INTEGER, SAVE, ALLOCATABLE :: IEWALK(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: ZSWALK(:,:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: ESWALK(:,:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: DPARGZW(:,:,:)
	REAL*8, SAVE, ALLOCATABLE :: TPARGZW(:,:,:)
	END MODULE COM_GZWDAT



	MODULE COM_GWADAT
*************************************************************
* Containes Green function and self energy for GWA
*************************************************************
	COMPLEX*16, SAVE, ALLOCATABLE :: SGMW(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: SGM0(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: GRFW(:,:,:,:,:)
	COMPLEX*16, SAVE, ALLOCATABLE :: GRF0(:,:,:,:,:)
	END MODULE COM_GWADAT


