	SUBROUTINE SETOPT
************************************************************
*   Saves optical matrix                                   *
************************************************************
C SET EXTERNAL CONNECTIONS
	USE COM_LMTMEM                                        !!! from setup
	USE COM_CONTROL ,ONLY: LIFT,LMTO                      !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,NORBS,NHSORT !!! from readini
     &                      ,IS,ILST,ILSB
      USE COM_LATPAR  ,ONLY: PAR,OMEGAC                     !!! from readini
	USE COM_KAPPAR  ,ONLY: NPAN,NKAP                      !!! from readini
      USE COM_PANPAR  ,ONLY: LTPAN,LSPAN                    !!! from readini
	USE COM_INPTTR  ,ONLY: NDIV,NDIC,NDIW                 !!! from readini
	USE COM_KMESH   ,ONLY: NPNT,PNT                       !!! from tetra
      USE COM_FREQU   ,ONLY: NOMGOPT,OMGOPT                 !!! from readini
      USE COM_FERMI   ,ONLY: EF                             !!! from ttrints
	USE COM_OPTMAT  ,ONLY: E2W,V2A                        !!! from optmat
	USE COM_UNIT    ,ONLY: IUN,IUR,IUT,IUI                !!! from setup
      USE COM_CHANNELS,ONLY: IUAIN                          !!! from channels
	USE COM_LMTOUT  ,ONLY: ICOPT,OPTFILE                  !!! from readini
C SET LOCAL VARIABLES
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION VEL(3),SPTOS(2),SPDOS(2),SPVEL(3,2)
     &   ,VELM(3,3),SPVELM(3,3,2)
      COMPLEX*16 TRC(-3:3,7,0:3),TRS(7,-3:3,0:3),E2(3,3)
	CHARACTER HELP*40 
	LOGICAL EXIST
	DATA PI/3.141592653589793D0/
	IF(ICOPT.EQ.0)RETURN
C Build transformation matrices
	CALL TRANSC(TRC)
	CALL TRANSS(TRS)
C CHECK PLASMA FREUQNCIES FOR DRUDE CONTRIBUTION
      CALL TTRVEL(EF,DOS,VEL,SPTOS,SPDOS,SPVEL,VELM,SPVELM,0)
	INQUIRE(FILE=OPTFILE,EXIST=EXIST)
	IF(EXIST)THEN
      OPEN(UNIT=IUAIN,FILE=OPTFILE,STATUS='OLD',FORM='FORMATTED')
	CLOSE(UNIT=IUAIN,STATUS='DELETE')
	ENDIF
	OPEN(IUAIN,FILE=OPTFILE,STATUS='NEW',FORM='FORMATTED')
C STORE HEADER INFO.
	WRITE(IUAIN,10010)
	WRITE(IUAIN,10020)
10010	FORMAT(1X,'<FILE=OPTFILE,INPUT=MODERN>')
10020 FORMAT('****************************************************')
	WRITE(IUAIN,10030)
10030 FORMAT(1X,'<SECTION=CTRL>               ! CONTROL PARAMETERS:')
      HELP='! Frequency mesh for optics          '
	WRITE(IUAIN,10040)NOMGOPT,HELP
10040 FORMAT(1X,'nOmgOPT=',I5,16X,A40)
      HELP='! Min.Frequency (Ry)  for optics     '
	WRITE(IUAIN,10050)OMGOPT(0),HELP
10050 FORMAT(1X,'WminOPT=',G14.7,7X,A40)
      HELP='! Max.Frequency (Ry)  for optics     '
	WRITE(IUAIN,10060)OMGOPT(NOMGOPT),HELP
10060 FORMAT(1X,'WmaxOPT=',G14.7,7X,A40)
	HELP='! Tetrahedron mesh                     '
	WRITE(IUAIN,10070)NDIV,HELP
10070 FORMAT(1X,'nDiv(:)=',I2,1X,I2,1X,I2,1X,12X,A40)
	HELP='! Tetrahedron mesh for weights         '
	WRITE(IUAIN,10080)NDIW,HELP
10080 FORMAT(1X,'nDiw(:)=',I2,1X,I2,1X,I2,1X,12X,A40)
C STORE OPTICAL DATA
	WRITE(IUAIN,10110)
10110 FORMAT(1X,'<SECTION=DATA>               ! OPTICAL DATA:')
	WRITE(IUAIN,10200)NOMGOPT,OMGOPT(0),OMGOPT(NOMGOPT),PAR,OMEGAC
	DO IA=1,3
	DO IB=1,3
	E2(IA,IB)=(0.D0,0.D0)
	DO MU=-1,1
	DO NU=-1,1
	E2(IA,IB)=E2(IA,IB)+
     &DCONJG(TRC(MU,IA,1))*V2A(MU,NU)*TRC(NU,IB,1)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
C SET PLASMA FREUQNCIES UNITS, NOT VELOCITIES BEFORE STORING
	DO IA=1,3
	DO IB=1,3
	VELM(IA,IB)=SQRT(ABS(VELM(IA,IB)*4.D0*PI*2.D0/OMEGAC))
	E2(IA,IB)=SQRT(ABS(DREAL(E2(IA,IB))*4.D0*PI*2.D0/OMEGAC))
	IF(ABS(E2(IA,IB)).LE.1.D-07)E2(IA,IB)=(0.D0,0.D0)
	IF(ABS(VELM(IA,IB)).LE.1.D-07)VELM(IA,IB)=(0.D0,0.D0)
	ENDDO
	ENDDO
	WRITE(IUAIN,10210)((VELM(I,J),I=1,3),J=1,3)
	WRITE(IUAIN,10210)((DREAL(E2(I,J)),I=1,3),J=1,3)
	DO IOMG=0,NOMGOPT
C TRANSFORM TO CARTESIAN COORDINATES
	DO IA=1,3
	DO IB=1,3
	E2(IA,IB)=(0.D0,0.D0)
	DO MU=-1,1
	DO NU=-1,1
	E2(IA,IB)=E2(IA,IB)+
     &DCONJG(TRC(MU,IA,1))*E2W(IOMG,MU,NU)*TRC(NU,IB,1)
	ENDDO
	ENDDO
	IF(ABS(E2(IA,IB)).LE.1.D-07)E2(IA,IB)=(0.D0,0.D0)
	ENDDO
	ENDDO
	WRITE(IUAIN,10220)OMGOPT(IOMG),((DREAL(E2(I,J)),I=1,3),J=1,3)
	ENDDO
	CLOSE(IUAIN)
	RETURN
10200 FORMAT(1X,I5,1X,G14.7,1X,G14.7,1X,G14.7,1X,G14.7)
10210 FORMAT(1X,13X,9(G12.5,1X))
10220 FORMAT(1X,10(G12.5,1X))
	END SUBROUTINE SETOPT

