	SUBROUTINE MULTASA
**************************************************************
*  Calculates multipoles from atomic spheres.                *
**************************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_CONTROL ,ONLY: LMTO,IASA                        !!! from readini
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,IS,ILSV        !!! from readini
      USE COM_LATPAR  ,ONLY: PAR,OMEGAC                       !!! from readini
      USE COM_SPHERE  ,ONLY: ZNUC,ZCOR,SMT,SASA,SWZ           !!! from readini
      USE COM_LATTICE ,ONLY: TAU                              !!! from readstr
      USE COM_SYMMLM  ,ONLY: SYM,LMSYM                        !!! from symlmm
      USE COM_DENDAT  ,ONLY: RHO,RHOCOR                       !!! from rhofull
      USE COM_MULTRO  ,ONLY: AMTOT,QMAD                       !!! from multrho
      USE COM_RADMESH ,ONLY: NRAD,H,R,DR,WEIGHT,RL            !!! from radmesh
      USE COM_UNIT    ,ONLY: IUN,IUR,IUT                      !!! from setup
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16, ALLOCATABLE :: AMADD(:,:,:)
      COMPLEX*16, ALLOCATABLE :: WORK1(:)
      COMPLEX*16, ALLOCATABLE :: WORK2(:)
      COMPLEX*16, ALLOCATABLE :: SS(:,:)
      DATA PI/3.141592653589793D0/
      CALL ABORTNOW()
	IF(IASA.NE.1)RETURN
	SQPI4=2.D0*SQRT(PI)
C MEMORY ALLOCATIONS FOR SCRATCH ARRAYS
      ALLOCATE(WORK1(0:NRADMEM))
      ALLOCATE(WORK2(0:NRADMEM))
      ALLOCATE(SS(LIMV1MEM,LIMV1MEM))
	ALLOCATE(AMADD(NSYMMEM,NATOMMEM,NSPINMEM))
C CALCULATE CORRECTION TO THE MULTIPOLE MOMENTS
      ZLEFT=0.D0
      DO ISPIN=1,NSPIN
      DO IATOM=1,NATOM
      ISORT=IS(IATOM)
      DO L=0,ILSV(ISORT)
      DO M=-L,L
      LM=L*(L+1)+M+1
      IF(SYM(LM,IATOM))THEN       !!! only for non-zero components
      ISYM=LMSYM(LM,IATOM)
      AMADD(ISYM,IATOM,ISPIN)=RHO(NRAD(ISORT),ISYM,IATOM,ISPIN,1)/
     &SQPI4*4.D0*PI/DBLE(L+3)/SMT(ISORT)**L*
     &(SASA(ISORT)**(L+3)-SMT(ISORT)**(L+3))
      ENDIF                      !!! only for non-zero components
      ENDDO
      ENDDO
      ZLEFT=ZLEFT-AMTOT(1,IATOM,ISPIN,1)-AMADD(1,IATOM,ISPIN)
      ENDDO
      ENDDO
C CALCULATE MULTIPOLE FIELD
	DO 2000 ISPIN=1,NSPIN                    !!! over spins
	DO 1000 IATOM=1,NATOM                    !!! over iatom
	ISORT=IS(IATOM)
	DO L=0,ILSV(ISORT)                       !!! over l
	ST=SQRT(4.D0*PI)/SMT(ISORT)/(2.D0*L+1.D0)
	DO M=-L,L                                !!! over m
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	QMAD(1,ISYM,IATOM,ISPIN)=(0.D0,0.D0)
	QMAD(2,ISYM,IATOM,ISPIN)=(0.D0,0.D0)
	ENDIF
      ENDDO                                    !!! over m
      ENDDO                                    !!! over l 
	DO JATOM=1,NATOM                         !!! over jatom
	JSORT=IS(JATOM)
	CALL STRMAT(IATOM,JATOM,SS)              !!! restore str.matrix
	DO L=0,ILSV(ISORT)                       !!! over l
	ST=SQRT(4.D0*PI)/SMT(ISORT)/(2.D0*L+1.D0)
	DO M=-L,L                                !!! over m
	LM=L*(L+1)+M+1
	IF(SYM(LM,IATOM))THEN
	ISYM=LMSYM(LM,IATOM)
	DO L1=0,ILSV(JSORT)                      !!! over l1
	ST1=1.D0/SMT(JSORT)/(2.D0*L1+1.D0)
	SL1=SMT(JSORT)**L1
	DZ=0.D0
C     DZ=ZLEFT/2.D0/DBLE(NSPIN)
	DO M1=-L1,L1                             !!! over m1
	LM1=L1*(L1+1)+M1+1
	IF(SYM(LM1,JATOM))THEN
	JSYM=LMSYM(LM1,JATOM)
	QMAD(1,ISYM,IATOM,ISPIN)=QMAD(1,ISYM,IATOM,ISPIN)-
     &ST*SS(LM,LM1)*(AMTOT(JSYM,JATOM,ISPIN,1)+DZ)*ST1
	QMAD(2,ISYM,IATOM,ISPIN)=QMAD(2,ISYM,IATOM,ISPIN)-
     &ST*SS(LM,LM1)*(AMADD(JSYM,JATOM,ISPIN)+DZ)*ST1
	ENDIF
      ENDDO                                    !!! over m1
      ENDDO                                    !!! over l1
	ENDIF
      ENDDO                                    !!! over m
      ENDDO                                    !!! over l
      ENDDO                                    !!! over jatom
1000	CONTINUE                                 !!! over iatom
2000	CONTINUE                                 !!! over spins
C MEMORY DEALLOCATIONS
      DEALLOCATE(WORK1)
      DEALLOCATE(WORK2)
	DEALLOCATE(AMADD)
      DEALLOCATE(SS)
	END




	SUBROUTINE STRMAT(IATOM,JATOM,SS)
*******************************************************
*  The preparation of the Madelung matrix for the     *
*  calculation of multipoles.                         *
*******************************************************
C SET ARRAY-SIZE DATA
      USE DIM_LMTART
C SET EXTERNAL CONNECTIONS
      USE COM_PARAM   ,ONLY: NATOM,NSORT,NSPIN,IS,ILSV        !!! from readini
      USE COM_LATPAR  ,ONLY: PAR,OMEGAC                       !!! from readini
      USE COM_SPHERE  ,ONLY: ZNUC,ZCOR,SMT,SWZ                !!! from readini
      USE COM_GAUNT   ,ONLY: C,GGNT                           !!! from makegnt
      USE COM_LMSTRS  ,ONLY: IDELP                            !!! from vecgen
      USE COM_VECGEN  ,ONLY: LIMDT                            !!! from vecgen
C SET LOCAL VARIABLES
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16, ALLOCATABLE :: SIGMA(:)
	COMPLEX*16  SS(LIMV1MEM,LIMV1MEM)
	DATA PI/3.141592653589793D0/
C MEMORY ALLOCATIONS
	LPLMEM=(2*LMAXVMEM+3)**2
	ALLOCATE(SIGMA(LPLMEM))
C  RESTORE FULL S(L'L) MATRIX FOR ATOM "IATOM"
	ITAU=IDELP(IATOM,JATOM)
	READ(7,REC=ITAU)LIM4,(SIGMA(LM),LM=1,LIM4)
	DO L1=0,ILSV(IS(IATOM))+1                     !!! over l1
	DO L =0,ILSV(IS(JATOM))+1                     !!! over l
	SOS=(SMT(IS(IATOM))/SWZ)**(L1+0.5)*(SMT(IS(JATOM))/SWZ)**(L+0.5)
	QSOSQ=SQRT(SMT(IS(IATOM))/2.D0)*SOS*SQRT(SMT(IS(JATOM))/2.D0)
	L2=L1+L
	LL2=L2/2
	DO M1=-L1,L1                                  !!! over m1
	L1M1=L1*(L1+1)+M1+1
	DO M =-L,L                                    !!! over m
	LM=L*(L+1)+M+1
	M2=M1-M
	L2M2=L2*(L2+1)+M2+1
	SS(L1M1,LM)=C(LL2,L1M1,LM)*GGNT(LL2,L1,L)*SIGMA(L2M2)*
     &QSOSQ
      ENDDO                                         !!! over m
      ENDDO                                         !!! over m1
      ENDDO                                         !!! over l
	IF(L1.EQ.1)SS(L1M1,L1M1)=SS(L1M1,L1M1)+
     &3.D0/NATOM/SWZ**3*SMT(IS(IATOM))**2*SMT(IS(JATOM))**2
      ENDDO                                         !!! over l1
C MEMORY DEALLOCATIONS
	DEALLOCATE(SIGMA)
	END
