      SUBROUTINE SITEGEN(RMAX,RBAS,TAU,NATOM)
*****************************************************************
*  Generates  all sites within sphere Rmax.                     *
*    Rbas          - translations                               *
*    Tau,Natom     - basis                                      *
*  Output goes to module com_sitegen:                           *
*    Nsites        - number of all sites in the sphere          *
*    Sites         - vectors                                    *
*    IrefSites(n)  - shows the site sort                        *
*****************************************************************
C SET EXTERNAL CONNECTION
      USE COM_SITEGEN, ONLY: NSITES,SITES,IREFSITES   !!! from sitegen
	USE COM_UNIT, ONLY: IUT
C SET LOCAL VARIBLES
	IMPLICIT REAL*8 (A-H,O-Z)
	REAL*8, ALLOCATABLE :: VEC(:,:)
	REAL*8, ALLOCATABLE :: TAUC(:,:)
      DIMENSION RBAS(3,3),TAU(3,NATOM),GL(3),TSITE(3)
      DATA PI/3.141592653589793D0/
C FIRST, UNDERSTAND HOW MANY VECTORS REQUIRED
	ALLOCATE(TAUC(3,NATOM))
	DELR=0.D0
	DO IATOM=1,NATOM
	CALL SITEGEN_REDC(RBAS(1,1),RBAS(1,2),RBAS(1,3),
     &TAU(1,IATOM),TAUC(1,IATOM),0)
	S=SQRT(TAUC(1,IATOM)**2+TAUC(2,IATOM)**2+TAUC(3,IATOM)**2)
	DELR=MAX(DELR,S)
	ENDDO
	RADD=RMAX+DELR
      CALL SITEGEN_CROS(GL,RBAS(1,2),RBAS(1,3))
      OMEGA=ABS(RBAS(1,1)*GL(1)+RBAS(2,1)*GL(2)+RBAS(3,1)*GL(3))
	NVECMEM=2.D0*(4.D0/3.D0*PI*RADD**3)/OMEGA
	NVECMEM=MAX(NVECMEM,100)
	NSITMEM=NVECMEM*(NATOM+1)
C MEMORY ALLOCATIONS FOR MAIN CONTROL ARRAYS
	IF(ALLOCATED(SITES))DEALLOCATE(SITES)
	ALLOCATE(SITES(3,0:NSITMEM))
	IF(ALLOCATED(IREFSITES))DEALLOCATE(IREFSITES)
	ALLOCATE(IREFSITES(0:NSITMEM))
C MEMORY ALLOCATIONS FOR SCRATCH ARRAYS
	ALLOCATE(VEC(3,0:NVECMEM))
C GENERATE STRUCTURE VECTORS INSIDE THE SPHERE RMAX
	CALL SITEGEN_VECT(RADD,RBAS,VEC,NVEC,NVECMEM)
C SET SITES
      ISITES=-1
      DO IVEC=0,NVEC
      DO IATOM=1,NATOM
      TSITE(1)=VEC(1,IVEC)+TAUC(1,IATOM)
      TSITE(2)=VEC(2,IVEC)+TAUC(2,IATOM)
      TSITE(3)=VEC(3,IVEC)+TAUC(3,IATOM)
      S=SQRT(TSITE(1)**2+TSITE(2)**2+TSITE(3)**2)
      IF(S-RMAX.LE.1.D-08)THEN
      ISITES=ISITES+1
	IF(ISITES.GT.NSITMEM)CALL SITEGEN_MESG01
      IREFSITES(ISITES)=IATOM
      SITES(1,ISITES)=TSITE(1)
      SITES(2,ISITES)=TSITE(2)
      SITES(3,ISITES)=TSITE(3)
      ENDIF
      ENDDO
      ENDDO
      NSITES=ISITES
C ORDER SITES BY INCREASING LENGHT
      DO ISITES=0,NSITES
      ABSI=SQRT(SITES(1,ISITES)**2+
     &          SITES(2,ISITES)**2+
     &          SITES(3,ISITES)**2)
      DO JSITES=ISITES+1,NSITES
      ABSJ=SQRT(SITES(1,JSITES)**2+
     &          SITES(2,JSITES)**2+
     &          SITES(3,JSITES)**2)
      IF(ABSJ.LT.ABSI)THEN
	JATOM=IREFSITES(JSITES)
      RXJ=SITES(1,JSITES)
      RYJ=SITES(2,JSITES)
      RZJ=SITES(3,JSITES)
      SITES(1,JSITES)=SITES(1,ISITES)
      SITES(2,JSITES)=SITES(2,ISITES)
      SITES(3,JSITES)=SITES(3,ISITES)
	IREFSITES(JSITES)=IREFSITES(ISITES)
      SITES(1,ISITES)=RXJ
      SITES(2,ISITES)=RYJ
      SITES(3,ISITES)=RZJ
	IREFSITES(ISITES)=JATOM
      ABSI=ABSJ
      ENDIF
      ENDDO
      ENDDO
C MEMORY DEALLOCATIONS
	DEALLOCATE(TAUC)
	DEALLOCATE(VEC)
C ERROR HANDLINGS
	CONTAINS
	SUBROUTINE SITEGEN_MESG01
	WRITE(IUT,*)'Terminated in <SITEGEN>:'
	WRITE(IUT,*)'Subscript out of bounds detected.'
	WRITE(IUT,*)'The alocatable array SITES(3,NSITMEM)'
	WRITE(IUT,*)'is allocated with NSITMEM=',NSITMEM
	WRITE(IUT,*)'which is less than required. Change in the'
	WRITE(IUT,*)'algorithm is needed.'
	CALL FULLSTOP(' ')
	END SUBROUTINE SITEGEN_MESG01
	END SUBROUTINE SITEGEN





	SUBROUTINE SITEGEN_VECT(RMAX,RBAS,VEC,NVEC,NVECMEM)
**********************************************************
*  Generates vectors inside a sphere Rmax.               *
**********************************************************
	USE COM_UNIT, ONLY: IUT
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION RBAS(3,3),VEC(3,0:NVECMEM),GL(3)
      DATA PI/3.141592653589793D0/
C VOLUME PER SITE
      CALL SITEGEN_CROS(GL,RBAS(1,2),RBAS(1,3))
      OMEGA=ABS(RBAS(1,1)*GL(1)+RBAS(2,1)*GL(2)+RBAS(3,1)*GL(3))
C SET SPHERE  
      RADD=RMAX+1.D-10
C SET LIMITS
	AA=SQRT(RBAS(1,1)**2+RBAS(2,1)**2+RBAS(3,1)**2)
	BB=SQRT(RBAS(1,2)**2+RBAS(2,2)**2+RBAS(3,2)**2)
	CC=SQRT(RBAS(1,3)**2+RBAS(2,3)**2+RBAS(3,3)**2)
      N1=RADD/OMEGA*BB*CC+1
      N2=RADD/OMEGA*AA*CC+1
      N3=RADD/OMEGA*AA*BB+1
C SET VECTORS
	IVEC=-1
	DO M1=-N1,N1
	DO M2=-N2,N2
	DO M3=-N3,N3
      RX=M1*RBAS(1,1)+M2*RBAS(1,2)+M3*RBAS(1,3)
      RY=M1*RBAS(2,1)+M2*RBAS(2,2)+M3*RBAS(2,3)
      RZ=M1*RBAS(3,1)+M2*RBAS(3,2)+M3*RBAS(3,3)
	RR=SQRT(RX*RX+RY*RY+RZ*RZ)
	IF(RR.LE.RADD+1.D-08)THEN
	IVEC=IVEC+1
	IF(IVEC.GT.NVECMEM)CALL SITEGEN_MESG01
	VEC(1,IVEC)=RX
	VEC(2,IVEC)=RY
	VEC(3,IVEC)=RZ
	ENDIF
	ENDDO
	ENDDO
	ENDDO
	NVEC=IVEC
C ORDER VECTORS BY INCREASING LENGHT
C     DO IVEC=0,NVEC
C     ABSI=SQRT(VEC(1,IVEC)**2+VEC(2,IVEC)**2+VEC(3,IVEC)**2)
C     DO JVEC=IVEC+1,NVEC
C     ABSJ=SQRT(VEC(1,JVEC)**2+VEC(2,JVEC)**2+VEC(3,JVEC)**2)
C     IF(ABSJ.LT.ABSI)THEN
C     RXJ=VEC(1,JVEC)
C     RYJ=VEC(2,JVEC)
C     RZJ=VEC(3,JVEC)
C     VEC(1,JVEC)=VEC(1,IVEC)
C     VEC(2,JVEC)=VEC(2,IVEC)
C     VEC(3,JVEC)=VEC(3,IVEC)
C     VEC(1,IVEC)=RXJ
C     VEC(2,IVEC)=RYJ
C     VEC(3,IVEC)=RZJ
C     ABSI=ABSJ
C     ENDIF
C     ENDDO 
C     ENDDO
C ERROR HANDLINGS
	CONTAINS
	SUBROUTINE SITEGEN_MESG01
	WRITE(IUT,*)'Terminated in <SITEGEN>:'
	WRITE(IUT,*)'Subscript out of bounds detected.'
	WRITE(IUT,*)'The aalocatable array VEC(3,0:NVECMEM)'
	WRITE(IUT,*)'is allocated with NVECMEM=',NVECMEM
	WRITE(IUT,*)'which is less than required. Change in the'
	WRITE(IUT,*)'algorithm is needed.'
	CALL FULLSTOP(' ')
	END SUBROUTINE SITEGEN_MESG01
	END SUBROUTINE SITEGEN_VECT



      SUBROUTINE SITEGEN_CROS(A,B,C)
**********************************************************
*    Cross product (AX,AY,AZ)=(BX,BY,BZ)*(CX,CY,CZ)      *
**********************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(3),B(3),C(3)
      A(1)=B(2)*C(3)-B(3)*C(2)
      A(2)=B(3)*C(1)-B(1)*C(3)
      A(3)=B(1)*C(2)-B(2)*C(1)
      END SUBROUTINE SITEGEN_CROS




	subroutine sitegen_redc(a,b,c,rin,rout,key)
*************************************************************
*       Subroutine for reducing vector R in first (Key=1) , *
*       central (Key=0) lattice cell.If (key=-1) return the *
*       R in basis coordinate.                              *
*************************************************************
	implicit real*8 (a-h,o-z)
	dimension rin(3),rout(3),rw(3),a(3),b(3),c(3),abc(3,3)
     &    ,abci(3,3)
	data icall/1/
c
c start up:
c     if(icall.eq.1) then
	icall=2
	do 10 i=1,3
	abc(i,1)=a(i)
	abc(i,2)=b(i)
	abc(i,3)=c(i)
  10  continue
	abci(1,1)=b(2)*c(3)-b(3)*c(2)
	abci(1,2)=b(3)*c(1)-b(1)*c(3)
	abci(1,3)=b(1)*c(2)-b(2)*c(1)
	det=a(1)*abci(1,1)+a(2)*abci(1,2)+a(3)*abci(1,3)
	abci(2,1)=a(3)*c(2)-a(2)*c(3)
	abci(2,2)=a(1)*c(3)-a(3)*c(1)
	abci(2,3)=a(2)*c(1)-a(1)*c(2)
	abci(3,1)=a(2)*b(3)-a(3)*b(2)
	abci(3,2)=a(3)*b(1)-a(1)*b(3)
	abci(3,3)=a(1)*b(2)-a(2)*b(1)
	do 30 i=1,3
	do 20 j=1,3
	abci(i,j)=abci(i,j)/det
  20  continue
  30  continue
c     endif
c
c Rin in a,b,c basis:
	do 50 i=1,3
	s=0
	do 40 j=1,3
	s=s+abci(i,j)*rin(j)
  40  continue
	rout(i)=s
  50  continue
	if(key.lt.0) return
c
	rw(1)=rout(1)-aint(rout(1))
	rw(2)=rout(2)-aint(rout(2))
	rw(3)=rout(3)-aint(rout(3))
c first cell:
	if(rw(1).lt.0.) rw(1)=rw(1)+1
	if(rw(2).lt.0.) rw(2)=rw(2)+1
	if(rw(3).lt.0.) rw(3)=rw(3)+1
c return to old basis:
	do 70 i=1,3
	s=0
	do 60 j=1,3
	s=s+abc(i,j)*rw(j)
  60  continue
	rout(i)=s
  70  continue
	if(key.eq.1) return
c central cell:
	i0=0
	j0=0
	k0=0
	rc=rout(1)**2+rout(2)**2+rout(3)**2
	do 160 i=0,1
	do 150 j=0,1
	do 140 k=0,1
	rmod=0
	do 170 l=1,3
	rmod=rmod+(rout(l)-i*a(l)-j*b(l)-k*c(l))**2
 170  continue
	if(rmod.lt.rc) then
	i0=i
	j0=j
	k0=k
	rc=rmod
	endif
 140  continue
 150  continue
 160  continue
	do 180 l=1,3
	rout(l)=rout(l)-i0*a(l)-j0*b(l)-k0*c(l)
 180  continue
	return
	end
