	SUBROUTINE FERMIINT(XM,NC,GL,SK)
*******************************************************
* Returns approximation to the Fermi function and its *
* integrals:                                          *
*         g0(x)=       1/(exp(x)+1)-theta(x)*(1-sk)   *
*         g1(x)= int[  1/(exp(x)+1)-theta(x)*(1-sk)]  *
*         g2(x)= int[  x/(exp(x)+1)-theta(x)*(1-sk)]  *
*         g3(x)= int[x^2/(exp(x)+1)-theta(x)*(1-sk)]  *
*         g4(x)= int[x^3/(exp(x)+1)-theta(x)*(1-sk)]  *
* If sk=0, the T=0 theta(x) contribution will be      *
* eliminated from the defintions of these integrals.  *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	PARAMETER (NM=50)! this will work fine only if real*16 accuracy is used
	PARAMETER (LM=4) ! LM first images will be generated
	DIMENSION GL(0:NM),GF(0:LM),FN(NM,0:LM),GLP0(0:LM),GLM0(0:LM)
     &   ,ALPHA(NM,0:LM,NM),COEF(0:LM,NM)
	SAVE ALPHA,COEF
	LOGICAL FIRST
	DATA FIRST/.TRUE./
C Prepare coefficients
	IF(FIRST)THEN
 	CALL FERMICOF(ALPHA,COEF)
      GLM0(0)= 0.500000000000000D0 ! this is gl when x=-0
      GLM0(1)=-0.693147180557974D0 ! it will be subtracted
      GLM0(2)= 0.822467033419486D0
      GLM0(3)=-1.803085354723550D0
      GLM0(4)= 5.682196976916560D0
	GLP0(0)=GLM0(0)              ! this is gl when x=+0
	DO L=1,LM
	GLP0(L)=-GLM0(L)*(-1)**L
	ENDDO
	FIRST=.FALSE.
	ENDIF
C Check returns
	IF(NC.GT.NM)RETURN
	IF(LM.GT.4)RETURN
C Check limits
	IF(XM.GT.+30.D0)THEN
	DO L=0,LM
	GL(L)=0.D0
	ENDDO
	GOTO1000
	ELSEIF(XM.LT.-30.D0)THEN
	XL=1.D0
	DO L=0,LM
	IF(L.EQ.0)DL=1.D0
	IF(L.GT.0)DL=1.D0/DBLE(L)
	GL(L)=SK*XL*DL
	XL=XL*XM
	ENDDO
	GOTO1000
	ENDIF
	IF(XM.LE.0.D0)THEN
	SIGN=+0.5D0
	X=+XM
	ELSEIF(XM.GT.0.D0)THEN
	SIGN=-0.5D0
	X=-XM
	ENDIF
C  Set argument
	Y=1.D0-EXP(X)
C  Set Fn
	DO L=0,LM
	DO N=1,NC
	FN(N,L)=0.D0
	ENDDO
	ENDDO
	YI=1.D0
	DO I=1,NC
	YI=YI*Y
	DO L=0,LM
	DO N=1,NC
	FN(N,L)=FN(N,L)+YI*ALPHA(I,L,N)
	ENDDO
	ENDDO
	ENDDO
	FL=1.D0
	XL=1.D0
	SL=-1.D0
	DO L=0,LM
	IF(XM.GT.0.D0)SL=-SL
	IF(XM.LE.0.D0)SL=1.D0
	DO N=1,NC
	FN(N,L)=SK*XL/FL+(COEF(L,N)+FN(N,L))*SL
	ENDDO
	XL=XL*XM
	FL=FL*DBLE(L+1)
	ENDDO
C Set g
	DO L=0,LM
	GF(L)=0.D0
	ENDDO
	TN=1.D0
	DO N=1,NC
	TN=TN*0.5D0
	DO L=0,LM
	GF(L)=GF(L)+TN*FN(N,L)
	ENDDO
	ENDDO
	GL(0)=SIGN*GF(0)
	GL(1)=SIGN*GF(1)
	GL(2)=SIGN*(XM*GF(1)-GF(2))
	GL(3)=SIGN*(XM**2*GF(1)-2.D0*XM*GF(2)+2.D0*GF(3))
	GL(4)=SIGN*(XM**3*GF(1)-3.D0*XM**2*GF(2)+
     &                6.D0*XM*GF(3)-6.D0*GF(4))
	XL=1.D0
	DO L=0,LM
	IF(L.EQ.0)DL=0.5D0
	IF(L.GT.0)DL=0.5D0/DBLE(L)
	GL(L)=GL(L)+SK*XL*DL
	XL=XL*XM
	ENDDO
C Subtract limit x=0
 1000	CONTINUE
	DO L=1,LM ! do not touch l=0, it is not an integral !
	IF(XM.LE.0.D0)GL(L)=GL(L)-GLM0(L)
	IF(XM.GT.0.D0)GL(L)=GL(L)-GLP0(L)
C All integrals are defined now from 0 to x
	ENDDO
	END



	SUBROUTINE FERMITST(X,GT)
*******************************************************
* Returns Fermi function and its integrals:           *
*         g0(x)=       1/(exp(x)+1)                   *
*         g1(x)= int[  1/(exp(x)+1)                   *
*         g2(x)= int[  x/(exp(x)+1)                   *
*******************************************************
	IMPLICIT REAL*8  (A-H,O-Z)
	DIMENSION GT(0:1)
	DATA DLOG2/2.46740110027234D0/
C	DLOG2=DDILOG(2.D0)
	Y=EXP(X)+1.D0
	GT(0)=1.D0/Y
	GT(1)=X-LOG(Y)+LOG(2.D0)
C                ! ??? when x->0 behavior is wrong !!!
C	GT(2)=0.D0 ! 0.5D0*X**2-X*LOG(Y)-DDILOG(Y)+DLOG2 
C	GT(3)=0.D0 ! not known
	END


C	IMPLICIT REAL*8 (A-H,O-Z)
C	PARAMETER (LM=4) ! LM first images will be generated
C	DIMENSION GL(0:LM),GT(0:1)
C	N=40
C	DO X=-5.D0,5.D0,0.01D0
C	CALL FERMIINT(X,N,GL,1)
C	CALL FERMITST(X,GT)
C	WRITE(1,*)X,GL,GT(0),GT(1)
C	ENDDO
C	CLOSE(1)
C	END



